<?php
/**
* Class de base pour les factory de manager
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-foundation  
* @author    Pascal Martin 
* @copyright Boral Informations Stratgiques Inc.
* @license   http://www.boreal-is.com/license Borealis License
* @link      http://www.boreal-is.com
*/
    
/**
* Class de base pour les factory de manager
* 
* @category   PHP 
* @package    Borealis-foundation
* @subpackage Manager
* @author     Pascal Martin
*/
abstract class com_borealis_foundation_manager_Factory
{
    /**
    * Return a Manager
    * 
    * @param string $pstrClassName Manager class name
    * 
    * @throws com_borealis_security_exception_ClassNotFoundException
    * @throws com_borealis_foundation_exception_AssertException
    * 
    * @return object
    */
    protected function getManager($pstrClassName)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrClassName);
        
        if (class_exists($pstrClassName))
        {        
            if (method_exists($pstrClassName, 'getInstance'))
            {        
                $objManager = call_user_func(array($pstrClassName, 'getInstance'));
                
                if ($this->isGoodInstance($objManager))
                {
                    return $objManager;
                }
                else
                {
                    throw $this->getError('Class '.$pstrClassName.' not implements the manager');   
                }   
            }
            else
            {
                throw $this->getError('Class '.$pstrClassName.' not implements com_borealis_security_util_Singleton'); 
            }
        }
        else   
        {
            throw new com_borealis_foundation_exception_ClassNotFoundException('Class '.$pstrClassName.' not found');   
        }
    }
    
    /**
    * Vrifie que l'object passer est du bon instance
    * 
    * @param object $pobjManager object to test
    * 
    * @return bool
    */ 
    abstract protected function isGoodInstance($pobjManager);
    
    /**
    * Retourne une erreur
    * 
    * @param string $pstrMessage Message erreur
    * 
    * @return Exception
    */
    abstract protected function getError($pstrMessage);
}
    
    
?>