<?php
/**
* Manager qui sert a la fois d'AuthenticationManager, d'AuthorizationManager et de PropertyManager 
* Rcupre les informations du serveur josso.
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-security  
* @author    Pascal Martin 
* @copyright Boral Informations Stratgiques Inc.
* @license   http://www.boreal-is.com/license Borealis License
* @link      http://www.boreal-is.com
*/
    
/**
* Manager qui sert a la fois d'AuthenticationManager, d'AuthorizationManager et de PropertyManager 
* Rcupre les informations du serveur josso.
* 
* @category   PHP 
* @package    Borealis-security
* @subpackage Manager
* @author     Pascal Martin
*/
class com_borealis_security_manager_impl_josso_JossoManager implements com_borealis_security_manager_authentication_AuthenticationManager, com_borealis_security_manager_authorization_AuthorizationManager, com_borealis_security_manager_property_PropertyManager
{
    private static $s_objInstance = null;
    
    /**
    * @var com_borealis_security_session_JossoSSOSession
    */
    protected $m_objSSOSession = null;
 
    /**
    * @var SoapClient
    */
    private $m_objSoapClientSessionMgr = null;
    
    /**
    * @var SoapClient
    */
    private $m_objSoapClientIdentityMgr = null;
    
    /**
    * @var SoapClient
    */ 
    private $m_objSoapClientIdentityProvider = null;
 
    /**
    * Construit une nouvelle instance de com_borealis_security_manager_impl_josso_JossoManager
    * 
    * @return com_borealis_security_manager_impl_josso_JossoManager
    */
    private function __construct()
    {    
        $this->m_objSSOSession = self::getSSOSessionInStore();         
        $this->keepAlive();
    }
    
    /**
    * return instanceof com_borealis_security_manager_impl_josso_JossoManager
    * return instanceof com_borealis_security_manager_authentication_AuthenticationManager
    * return instanceof com_borealis_security_manager_authorization_AuthorizationManager
    * return instanceof com_borealis_security_manager_property_PropertyManager
    * return instanceof com_borealis_security_util_Singleton
    * 
    * @throws com_borealis_security_exception_HostNotFoundException
    * @throws com_borealis_security_exception_ServiceNotFoundException
    * @throws com_borealis_security_exception_BadSessionException
    * 
    * @return com_borealis_security_manager_impl_josso_JossoManager
    */
    public static function getInstance()
    {
        if (self::$s_objInstance === null)
        {
            self::$s_objInstance = new com_borealis_security_manager_impl_josso_JossoManager();
        }
        return self::$s_objInstance;
    } 
    
    // --------------------------------------------------------------------
    // com_borealis_security_manager_authentication_AuthenticationManager
    // --------------------------------------------------------------------
    
    /**
    * Try authentication of the user
    * 
    * @param string                                  $pstrUsername Nom d'utilisateur
    * @param com_borealis_security_password_Password $pobjPassword Mot de passe
    * 
    * @throws com_borealis_security_exception_HostNotFoundException
    * @throws com_borealis_security_exception_ServiceNotFoundException
    * @throws com_borealis_security_exception_UnknownUserException
    * 
    * @return com_borealis_security_session_JossoSSOSession
    */
    public function login($pstrUsername, com_borealis_security_password_Password $pobjPassword)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrUsername);
        
        if ($this->getSession() !== null)
        {
            $this->logout();
        }
             
        $objSession = new com_borealis_security_session_JossoSSOSession($pstrUsername);
        $this->addToken($objSession, $pobjPassword, com_borealis_security_SettingImpl::getJossoAuthenticationSecurityDomain());
        $this->addToken($objSession, new com_borealis_security_password_RawPassword(com_borealis_security_SettingImpl::getJossoAuthorizationPassword()), com_borealis_security_SettingImpl::getJossoAuthorizationSecurityDomain());
        $this->addToken($objSession, new com_borealis_security_password_RawPassword(com_borealis_security_SettingImpl::getJossoPropertyPassword()), com_borealis_security_SettingImpl::getJossoPropertySecurityDomain());
        
        $this->setSession($objSession);
        
        return $this->getSession();
    }
    
    /**
    * put your comment there...
    * 
    * @param com_borealis_security_session_SSOSession $pobjSSOSession Open session
    * 
    * @throws com_borealis_security_exception_HostNotFoundException
    * @throws com_borealis_security_exception_ServiceNotFoundException
    * @throws com_borealis_security_exception_BadSessionException
    * 
    * @return com_borealis_security_session_JossoSSOSession
    */
    public function loginWithSSOSession(com_borealis_security_session_SSOSession $pobjSSOSession)
    {
        if (!($pobjSSOSession instanceof com_borealis_security_session_JossoSSOSession))
        {
            throw new Exception("not supported");
        }  
        
        if ($this->getSession() !== null && $this->getSession() != $pobjSSOSession)
        {
            $this->logout();
        }        
        $this->setSession($pobjSSOSession);
        $this->keepAlive();  
        return $this->getSession();       
    }
    
    // @codeCoverageIgnoreStart To dificult to test because of use of header
    /**
    * Log un utilisateur avec la passerelle
    * 
    * @return com_borealis_security_user_User   
    */
    public function loginWithGateway()
    {
        $strHost  = $_SERVER['HTTP_HOST'];
        $strUri   = $_SERVER['PHP_SELF'];
        
        if (isset($_REQUEST['josso_assertion_id']))
        {
            $strAssertionId = $_REQUEST['josso_assertion_id'];  
             
            if (!headers_sent())
            {   
                if (ob_get_level() > 0)
                {
                    ob_end_clean();
                }
                header("Location: http://$strHost$strUri");    
            }
                        
            if ($this->getSession() !== null)
            {
                $this->logout();
            }
                    
            $strToken = $this->resolveAuthenticationAssertion($strAssertionId);
            $strUsername = '';
            $this->findUserInSession($strToken, $strUsername);            
            $objSession = new com_borealis_security_session_JossoSSOSession($strUsername);
            $objSession->addToken(com_borealis_security_SettingImpl::getJossoAuthenticationSecurityDomain(), $strToken); 
            $this->addToken($objSession, new com_borealis_security_password_RawPassword(com_borealis_security_SettingImpl::getJossoAuthorizationPassword()), com_borealis_security_SettingImpl::getJossoAuthorizationSecurityDomain());
            $this->addToken($objSession, new com_borealis_security_password_RawPassword(com_borealis_security_SettingImpl::getJossoPropertyPassword()), com_borealis_security_SettingImpl::getJossoPropertySecurityDomain());
            
            $this->setSession($objSession);  
 
            return $objSession;
        }
        else
        { 
            $strBackTo = urlencode("http://$strHost$strUri?".http_build_query($_GET));
            $strLoginUrl = com_borealis_security_SettingImpl::getJossoGatewayLogin()."?josso_back_to=$strBackTo"; //&josso_partnerapp_host='.com_borealis_security_SettingImpl::getJossoEndpoint();
            $this->forceRedirect($strLoginUrl);   
        }
    }
    
    /**
    * Redirection du navigateur sur l'url spcifier
    * 
    * @param string $pstrUrl Url de redirection
    * 
    * @return void
    */
    public function forceRedirect($pstrUrl)
    {
        if (!headers_sent())
        {
            if (ob_get_level() > 0)
            {
                ob_end_clean();
            }
            header("Location: " . $pstrUrl);
        }
        printf('<HTML>');
        printf('<META http-equiv="Refresh" content="0;url=%s">', $pstrUrl);
        printf('<BODY onload="try {self.location.href="%s" } catch(e) {}"><a href="%s">Redirect </a></BODY>', $pstrUrl, $pstrUrl);
        printf('</HTML>');
        die();
    }
    // @codeCoverageIgnoreEnd
    
    /**
    * Clear SSOSession
    * 
    * @throws com_borealis_security_exception_HostNotFoundException
    * @throws com_borealis_security_exception_ServiceNotFoundException
    * 
    * @return void
    */
    public function logout()
    {
        $objArraySecurityDomain = $this->getDistinctSecurityDomain();    
        $pobjIterator = $objArraySecurityDomain->getIterator();    
        while ($pobjIterator->valid()) 
        {
            $this->globalSignoff($pobjIterator->current());    
            $pobjIterator->next();
        } 

        $this->setSession(null);  
    }
    
    /**
    * Return current SSOSession if user are authenticate 
    * 
    * @return com_borealis_security_session_JossoSSOSession
    */
    public function getSession()
    {
        return $this->m_objSSOSession; 
    }
    
    // --------------------------------------------------------------------
    // com_borealis_security_manager_authorization_AuthorizationManager
    // --------------------------------------------------------------------
    
    /**
    * Return list of role for the session
    * 
    * @param com_borealis_security_session_Session $pobjSession target session
    * 
    * @throws com_borealis_security_exception_HostNotFoundException
    * @throws com_borealis_security_exception_ServiceNotFoundException
    * @throws com_borealis_security_exception_BadSessionException
    * 
    * @return ArrayObject
    */
    public function getRoles(com_borealis_security_session_Session $pobjSession)
    {
        if (!($pobjSession instanceof com_borealis_security_session_JossoSSOSession))
        {
            throw new Exception("not supported");
        }
        return $this->findRolesBySSOSessionId($pobjSession->getTokenBySecurityDomain(com_borealis_security_SettingImpl::getJossoAuthorizationSecurityDomain()));
    }
    
    // --------------------------------------------------------------------
    // com_borealis_security_manager_property_PropertyManager
    // --------------------------------------------------------------------
    
    /**
    * Return list of property for the session
    * 
    * @param com_borealis_security_session_Session $pobjSession target session
    * 
    * @throws com_borealis_security_exception_HostNotFoundException
    * @throws com_borealis_security_exception_ServiceNotFoundException
    * @throws com_borealis_security_exception_BadSessionException
    * 
    * @return ArrayObject of com_borealis_security_user_Property
    */
    public function getProperties(com_borealis_security_session_Session $pobjSession)
    {
        if (!($pobjSession instanceof com_borealis_security_session_JossoSSOSession))
        {
            throw new Exception("not supported");
        }
        $strUsername = '';
        return $this->findUserInSession($pobjSession->getTokenBySecurityDomain(com_borealis_security_SettingImpl::getJossoPropertySecurityDomain()), $strUsername);  
    } 
    
    
    // --------------------------------------------------------------------
    // josso manager method
    // -------------------------------------------------------------------- 
    
    /**
    * Sends a keep-alive notification to the SSO server so that SSO sesison is not lost.  
    * 
    * @return void
    */
    public function keepAlive()
    {
        if ($this->getSession() instanceof com_borealis_security_session_JossoSSOSession)
        {
            $objArraySecurityDomain = $this->getDistinctSecurityDomain();    
            $pobjIterator = $objArraySecurityDomain->getIterator();    
            while ($pobjIterator->valid()) 
            {
                $this->accessSession($pobjIterator->current());    
                $pobjIterator->next();
            }
        }   
    }
   
    /**
    * Return the list of dinstinc Security Domain use in this Manager
    * 
    * @return ArrayObject of string
    */
    private function getDistinctSecurityDomain()
    {
        $objArray = array();  
        array_push($objArray, com_borealis_security_SettingImpl::getJossoAuthenticationSecurityDomain());
        array_push($objArray, com_borealis_security_SettingImpl::getJossoAuthorizationSecurityDomain()); 
        array_push($objArray, com_borealis_security_SettingImpl::getJossoPropertySecurityDomain()); 
        return new ArrayObject(array_unique($objArray));
    }
    
    /**
    * Call web method accessSession
    *  
    * @param string $pstrSecurityDomain Nom du security domain
    * 
    * @return void
    */
    private function accessSession($pstrSecurityDomain)
    {
        if ($this->getSession() instanceof com_borealis_security_session_JossoSSOSession)
        {
            $objSoapClient = $this->getSoapClientSessionMgr();
            
            $objSoapVarSession = new SoapVar($this->getSession()->getTokenBySecurityDomain($pstrSecurityDomain), XSD_STRING, null, null, 'ssoSessionId');
            $objSoapVarRequest = new SoapVar(array($objSoapVarSession), SOAP_ENC_OBJECT, 'AccessSessionRequestType', 'http://josso.org/gateway/ws/1.1/protocol', 'AccessSessionRequest');
            $objArrayParams = array($objSoapVarRequest);
           
            try
            {
                $objResult = $this->soapCall($objSoapClient, 'accessSession', $objArrayParams);              
            
                // @codeCoverageIgnoreStart
                // Vrifie que la session est toujour ouverte
                if ($objResult === null || strcmp($objResult->ssoSessionId, $this->getSession()->getTokenBySecurityDomain($pstrSecurityDomain)) !== 0)
                {
                    // Close session quand la session a expirr sur le serveur josso                    
                    $this->setSession(null);  
                }
                // @codeCoverageIgnoreEnd    
            }
            catch(Exception $objError)
            {
                // Close session 
                $this->setSession(null);
                throw $objError;
            }
            
            // Session keep-alive succeed 
        }
    }
    
    /**
    * Call web method findUserInSession and return user property
    * 
    * @param string $pstrToken     ssoSessionId token
    * @param string &$pstrUsername Username, for return value 
    * 
    * @return ArrayObject of com_borealis_security_user_Property 
    */
    private function findUserInSession($pstrToken, &$pstrUsername) 
    {
        $objSoapClient = $this->getSoapClientIdentityMgr();
            
        $objSoapVarSSOSessionId = new SoapVar($pstrToken, XSD_STRING, null, null, 'ssoSessionId');
        $objSoapVarRequest = new SoapVar(array($objSoapVarSSOSessionId), SOAP_ENC_OBJECT, 'FindUserInSessionRequestType', 'http://josso.org/gateway/ws/1.1/protocol', 'FindUserInSessionRequest');
        $objArrayParams = array($objSoapVarRequest);
        $objArrayProperty = new ArrayObject();
                
        $objResult = $this->soapCall($objSoapClient, 'findUserInSession', $objArrayParams);              
        $pstrUsername = $objResult->SSOUser->name;
        foreach ($objResult->SSOUser->properties as &$objProperty)  
        {
            $objArrayProperty->append(new com_borealis_security_user_Property($objProperty->name, $objProperty->value));
        }   
 
        return $objArrayProperty;
    }
    
    /**
    * Call web method findRolesBySSOSessionId and return user role 
    * 
    * @param string $pstrToken ssoSessionId token  
    * 
    * @return ArrayObject of com_borealis_security_user_Role
    */
    private function findRolesBySSOSessionId($pstrToken)
    {       
        $objSoapClient = $this->getSoapClientIdentityMgr();
            
        $objSoapVarSSOSessionId = new SoapVar($pstrToken, XSD_STRING, null, null, 'ssoSessionId');
        $objSoapVarRequest = new SoapVar(array($objSoapVarSSOSessionId), SOAP_ENC_OBJECT, 'FindRolesBySSOSessionIdRequestType', 'http://josso.org/gateway/ws/1.1/protocol', 'FindRolesBySSOSessionIdRequest');
        $objArrayParams = array($objSoapVarRequest);
        $objArrayRoles = new ArrayObject();
            
        $objResult = $this->soapCall($objSoapClient, 'findRolesBySSOSessionId', $objArrayParams);              

        foreach ($objResult->roles as &$objRole) 
        {
            $objArrayRoles->append(new com_borealis_security_user_Role($objRole->name));
        }    
        
        return $objArrayRoles;
    }
    
    /**
    * Call web method globalSignoff 
    * 
    * @param string $pstrSecurityDomain Nom du security domain   
    * 
    * @return void
    */
    private function globalSignoff($pstrSecurityDomain)
    {
        if ($this->getSession() === null)
        {
            return;
        }
        
        $objSoapClient = $this->getSoapClientIdentityProvider();
            
        $objSoapVarSSOSessionId = new SoapVar($this->getSession()->getTokenBySecurityDomain($pstrSecurityDomain), XSD_STRING, null, null, 'ssoSessionId');
        $objSoapVarRequest = new SoapVar(array($objSoapVarSSOSessionId), SOAP_ENC_OBJECT, 'GlobalSignoffRequestType', 'http://josso.org/gateway/ws/1.1/protocol', 'GlobalSignoffRequest');
        $objArrayParams = array($objSoapVarRequest);
       
        try
        {
            $objResult = $this->soapCall($objSoapClient, 'globalSignoff', $objArrayParams);              
        }
        catch(com_borealis_security_exception_BadSessionException $objError)
        {
            // Ne rien faire
        }         
    }
    
    /**
    * Add token in pass session
    * 
    * @param com_borealis_security_session_JossoSSOSession &$pobjSession       Session
    * @param com_borealis_security_password_Password       $pobjPassword       Mod de passe
    * @param string                                        $pstrSecurityDomain Nom du Security Domain
    * 
    * @return void
    */
    private function addToken(com_borealis_security_session_JossoSSOSession &$pobjSession, com_borealis_security_password_Password $pobjPassword, $pstrSecurityDomain)
    {
        if ($pobjSession->getTokenBySecurityDomain($pstrSecurityDomain) === null)
        {
            $strAssertionId = $this->assertIdentityWithSimpleAuthentication($pobjSession->getUsername(), $pobjPassword->getValue(), $pstrSecurityDomain);
            $strToken = $this->resolveAuthenticationAssertion($strAssertionId);
            
            $pobjSession->addToken($pstrSecurityDomain, $strToken);
        }     
    }
    
    /**
    * Call web method assertIdentityWithSimpleAuthentication and return assertionId
    *     
    * @param string $pstrUsername       Nom d'utilisateur
    * @param string $pstrPassword       Mod de passe
    * @param string $pstrSecurityDomain Nom du Security Domain
    * 
    * @return string assertionId
    */
    private function assertIdentityWithSimpleAuthentication($pstrUsername, $pstrPassword, $pstrSecurityDomain)
    {    
        $objSoapClient = $this->getSoapClientIdentityProvider();
            
        $objSoapVarSecurityDomain = new SoapVar($pstrSecurityDomain, XSD_STRING, null, null, 'securityDomain');
        $objSoapVarUsername = new SoapVar($pstrUsername, XSD_STRING, null, null, 'username');
        $objSoapVarPassword = new SoapVar($pstrPassword, XSD_STRING, null, null, 'password');
        
        $objSoapVarRequest = new SoapVar(array($objSoapVarSecurityDomain, $objSoapVarUsername, $objSoapVarPassword), SOAP_ENC_OBJECT, 'AssertIdentityWithSimpleAuthenticationRequestType', 'http://josso.org/gateway/ws/1.1/protocol', 'AssertIdentityWithSimpleAuthenticationRequest');
        $objArrayParams = array($objSoapVarRequest);
       
        $objResult = $this->soapCall($objSoapClient, 'assertIdentityWithSimpleAuthentication', $objArrayParams);  
        return $objResult->assertionId;              
    }
    
    /**
    * Call web method resolveAuthenticationAssertion and return ssoSessionId    
    * 
    * @param string $pstrAssertionId assertionId
    * 
    * @return string ssoSessionId
    */
    private function resolveAuthenticationAssertion($pstrAssertionId)
    {        
        $objSoapClient = $this->getSoapClientIdentityProvider();
            
        $objSoapVarAssertionId = new SoapVar($pstrAssertionId, XSD_STRING, null, null, 'assertionId');
        $objSoapVarRequest = new SoapVar(array($objSoapVarAssertionId), SOAP_ENC_OBJECT, 'ResolveAuthenticationAssertionRequestType', 'http://josso.org/gateway/ws/1.1/protocol', 'ResolveAuthenticationAssertionRequest');
        $objArrayParams = array($objSoapVarRequest);
                 
        $objResult = $this->soapCall($objSoapClient, 'resolveAuthenticationAssertion', $objArrayParams);  
        return $objResult->ssoSessionId;                  
    }
    
    /**
    * Set current session
    * 
    * @param com_borealis_security_session_Session $pobjSession Session
    * 
    * @return void
    */
    private function setSession(com_borealis_security_session_Session $pobjSession = null)
    {
        $this->m_objSSOSession = $pobjSession;   
        $this->setSSOSessionInStore($pobjSession);
    }
    
    /**
    * Call web method add convert exception
    * 
    * @param SoapClient $pobjSoapClient  Soap client
    * @param string     $pstrMethod      Method name
    * @param array      $pobjArrayParams Method param
    * 
    * @return void
    */
    private function soapCall(SoapClient $pobjSoapClient, $pstrMethod, $pobjArrayParams)
    {
        try
        {
            ini_set('default_socket_timeout', com_borealis_security_SettingImpl::getJossoWsDefaultSocketTimeout());
            $objResult = $pobjSoapClient->__soapCall($pstrMethod, $pobjArrayParams);            
            ini_restore('default_socket_timeout'); 
            return $objResult; 
        }
        catch(SoapFault $objError)
        {
            $objException = $objError;
            if (isset($objError->detail) && isset($objError->detail->exceptionName) && strcmp($objError->detail->exceptionName, 'org.josso.gateway.ws._1_1.protocol.SSOIdentityProviderErrorType') === 0)
            {
                $strSecurityDomain = $pobjArrayParams[0]->enc_value[0]->enc_value;
                $strUsername = $pobjArrayParams[0]->enc_value[1]->enc_value;     
                $objException = new com_borealis_security_exception_UnknownUserException($objError->detail->SSOIdentityProviderError->errMessage." : Username : $strUsername, SecurityDomain: $strSecurityDomain");                    
            } 
            else if (isset($objError->detail) && isset($objError->detail->exceptionName) && strcmp($objError->detail->exceptionName, 'org.josso.gateway.ws._1_1.protocol.InvalidSessionErrorType') === 0)
            {
                $objException = new com_borealis_security_exception_BadSessionException($objError->detail->InvalidSessionError->sessionId);                    
            }
            else if (isset($objError->detail) && isset($objError->detail->exceptionName) && strcmp($objError->detail->exceptionName, 'org.josso.gateway.ws._1_1.protocol.NoSuchSessionErrorType') === 0)
            {
                $objException = new com_borealis_security_exception_BadSessionException($objError->detail->exceptionName.' : '.$objError->detail->NoSuchSessionError->sessionId);                    
            }     
            else if (strcmp($objError->faultcode, 'ns1:Server.NoService') === 0)
            {
                $objException = new com_borealis_security_exception_ServiceNotFoundException($objError->getMessage().' : '.$pobjSoapClient->location);                    
            }
            else if (strcmp($objError->faultcode, 'HTTP') === 0 && strcmp($objError->faultstring, 'Introuvable') === 0)
            {
                $objException = new com_borealis_security_exception_ServiceNotFoundException($objError->getMessage().' : '.$pobjSoapClient->location);                    
            }
            else if (strcmp($objError->faultcode, 'Client') === 0)
            {
                $objException = new com_borealis_security_exception_ServiceNotFoundException($objError->getMessage().' : '.$pobjSoapClient->location);                    
            }
            
            else if (strcmp($objError->faultcode, 'HTTP') === 0)
            {
                $objException = new com_borealis_security_exception_HostNotFoundException($objError->getMessage().' : '.com_borealis_security_SettingImpl::getJossoEndpoint() );                    
            }        
            else
            {
                error_log($objError); 
            }
            
            ini_restore('default_socket_timeout');
            throw $objException;
        }         
    }
    
    /**
    * Return Soap Client with IdentityMgr location
    * 
    * @return SoapClient
    */
    private function getSoapClientIdentityMgr()
    {
        $strLocation = com_borealis_security_SettingImpl::getJossoEndpoint()  . com_borealis_security_SettingImpl::getJossoWsSSOIdentitymanager();
        if ($this->m_objSoapClientIdentityMgr === null || strcmp($strLocation, $this->m_objSoapClientIdentityMgr->location) !==0)
        {
            $objArrayOptions = array(
                                'encoding'=>'ISO-8859-1',
                                'connection_timeout'=> com_borealis_security_SettingImpl::getJossoWsDefaultSocketTimeout(),
                                'exceptions' => true,            
                                'uri'  => 'http://josso.org/gateway/ws/1.1/wsdl/soapbinding/IdentityManager',            
                                'location' => $strLocation);   
                              
            $this->m_objSoapClientIdentityMgr = new SoapClient(null, $objArrayOptions); 
        }     
        return $this->m_objSoapClientIdentityMgr;        
    } 
    
    /**
    * Return Soap Client with SessionMgr location
    * 
    * @return SoapClient
    */    
    private function getSoapClientSessionMgr()
    {
        $strLocation = com_borealis_security_SettingImpl::getJossoEndpoint()  . com_borealis_security_SettingImpl::getJossoWsSSOSessionmanager();
        if ($this->m_objSoapClientSessionMgr === null || strcmp($strLocation, $this->m_objSoapClientSessionMgr->location) !==0)
        {
            $objArrayOptions = array(
                                'encoding'=>'ISO-8859-1',
                                'connection_timeout'=> 0,
                                'exceptions' => true,            
                                'uri'  => 'http://josso.org/gateway/ws/1.1/wsdl/soapbinding/SessionManager',            
                                'location' => $strLocation);   
                              
            $this->m_objSoapClientSessionMgr = new SoapClient(null, $objArrayOptions); 
        }     
        return $this->m_objSoapClientSessionMgr;        
    } 
    
    /**
    * Return Soap Client with IdentityProvider location
    * 
    * @return SoapClient
    */    
    private function getSoapClientIdentityProvider()
    {
        $strLocation = com_borealis_security_SettingImpl::getJossoEndpoint()  . com_borealis_security_SettingImpl::getJossoWsSSOIdentityProvider();
        if ($this->m_objSoapClientIdentityProvider === null || strcmp($strLocation, $this->m_objSoapClientIdentityProvider->location) !==0)
        {
            $objArrayOptions = array(
                                'encoding'=>'ISO-8859-1',
                                'connection_timeout'=> 0,
                                'exceptions' => true,            
                                'uri'  => 'http://josso.org/gateway/ws/1.1/wsdl/soapbinding/IdentityProvider', 
                                'location' => $strLocation);   
                              
            $this->m_objSoapClientIdentityProvider = new SoapClient(null, $objArrayOptions); 
        }     
        return $this->m_objSoapClientIdentityProvider;        
    }    
    
    /**
    * Return SSOSession store un a cookie
    * 
    * @return com_borealis_security_session_JossoSSOSession
    */
    private static function getSSOSessionInStore()
    {
        // @codeCoverageIgnoreStart
        if (!isset($_COOKIE['JOSSO_SSOSESSION']))
        {        
            self::setSSOSessionInStore(null);                  
        }
        // @codeCoverageIgnoreEnd
        return unserialize($_COOKIE['JOSSO_SSOSESSION']);     
    } 
    
    /**
    * Store SSOSession in cookie 
    * 
    * @param com_borealis_security_session_JossoSSOSession $pstrSSOSession Session to store
    * 
    * @return void
    */
    private static function setSSOSessionInStore(com_borealis_security_session_JossoSSOSession $pstrSSOSession = null)
    {   
        $_COOKIE['JOSSO_SSOSESSION'] = serialize($pstrSSOSession);  
        setcookie('JOSSO_SSOSESSION', $_COOKIE['JOSSO_SSOSESSION'], 0, "/"); // session cookie ...             
    } 
}
    
    
?>