<?php
/**
* Récupère les informations d'un fichier xml  
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-acl  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.boreal-is.com/license BSD License
* @link      http://www.boreal-is.com
*/

/**
* Récupère les informations d'un fichier xml       
* 
* @category   PHP 
* @package    Borealis-acl
* @subpackage DataStore  
* @author     Pascal Martin 
*/ 
class com_borealis_acl_datastore_XMLDataStore implements com_borealis_acl_datastore_DataStore
{
    /**
    * @var com_borealis_acl_datastore_XMLDataStore
    */
    private static $s_objInstance = null; 
    
    /**
    * @var string
    */
    private $m_strConfigFile = null; 
    
    /**
    * @var SimpleXMLElement
    */
    protected $m_objSimpleXMLElement = null;
        
    /**
    * Construit une nouvelle instance de com_borealis_acl_datastore_XMLDataStore
    * 
    * @return com_borealis_acl_datastore_XMLDataStore
    */
    private function __construct()
    {    
        $this->load();       
    }
    
    /**
    * return instanceof com_borealis_acl_datastore_XMLDataStore
    * return instanceof com_borealis_acl_datastore_DataStore 
    * 
    * @return com_borealis_acl_datastore_DatabaseDataStore 
    */
    public static function getInstance()
    {
        if (self::$s_objInstance === null)
        {
            self::$s_objInstance = new com_borealis_acl_datastore_XMLDataStore();
        }
        self::$s_objInstance->checkReload();
        return self::$s_objInstance;
    } 
    
    /**
    * Vérifie si le fichier de configuration xml est plus le mêne alors on charge le nouveau fichier
    * 
    * @return void
    */
    public function checkReload()
    {
        if ($this->m_strConfigFile != com_borealis_acl_SettingImpl::getXMLConfigFile())
        {
            $this->load();    
        }      
    }
    
    /**
    * Load la configuration xml
    * 
    * @return void
    */
    private function load()
    {
        $this->m_strConfigFile = com_borealis_acl_SettingImpl::getXMLConfigFile();
        try
        {
            if (!is_writable($this->m_strConfigFile))
            {                
                exec('attrib -r "'.$this->m_strConfigFile.'" /s'); // Supprime le read only sur le fichier
            }
            $this->m_objSimpleXMLElement = simplexml_load_file($this->m_strConfigFile);        
        }
        catch(Exception $objException)
        {
            throw new com_borealis_acl_exception_BadXMLConfigFileException($objException, $this->m_strConfigFile);   
        }
    }
    
    // ###################################################################
    // # Méthode de la l'interface  com_borealis_acl_datastore_DataStore
    // ###################################################################        

    /**
    * Delete in datastore
    * 
    * @param com_borealis_acl_object_Base $pobjBase Object
    * 
    * @throws com_borealis_foundation_exception_AssertException
    * 
    * @return void
    */
    function deleteByID(com_borealis_acl_object_Base $pobjBase)
    {
        $iID = $pobjBase->getID();
        $objDomDocument = new DOMDocument();
        $objDomDocument->loadXML($this->m_objSimpleXMLElement->asXML());  
        $strXPath = self::getXPathByObject($pobjBase)."[./id='$iID']";
        $objDomXpath = new DOMXpath($objDomDocument); 
        $objDomElements = $objDomXpath->query($strXPath);
        foreach ($objDomElements as $objDomElement)
        {
            $objDomElement->parentNode->removeChild($objDomElement);           
        }
        $this->m_objSimpleXMLElement = simplexml_import_dom($objDomDocument);
        $this->save();
    }

    /**
    * Update name in datastore
    * 
    * @param com_borealis_acl_object_Base $pobjBase Object   
    * @param string                       $pstrName Nouveau nom de l'action
    * 
    * @throws com_borealis_foundation_exception_AssertException
    * 
    * @return void
    */
    function update(com_borealis_acl_object_Base $pobjBase, $pstrName)
    {
        $iID = $pobjBase->getID();
        $strXPath = self::getXPathByObject($pobjBase)."[./id='$iID']";
        
        $objArraySimpleXMLElement = $this->m_objSimpleXMLElement->xpath($strXPath);     
        if ($objArraySimpleXMLElement !== false && count($objArraySimpleXMLElement) > 0)
        {
            $objArraySimpleXMLElement[0]->name = $pstrName; 
            $this->save();
        }   
    }
    
    /**
    * Insert in datastore
    * 
    * @param string $pstrType Type de l'objet
    * @param string $pstrName Nom de l'action
    * 
    * @throws com_borealis_foundation_exception_AssertException 
    * 
    * @return int ID
    */
    function insert($pstrType, $pstrName)
    {
        // check if not exist
        if ($this->selectByName($pstrType, $pstrName) !== null)
        {
            throw new Exception("$pstrType : $pstrName already exsit");    
        }
        
        // Get Next id
        $iID = 1;
        $objArrayObject = $this->selectAll($pstrType);
        foreach ($objArrayObject as $objArrayObjectInfo)
        {
            $iCurrentID = (int)$objArrayObjectInfo['id'];
            if ( $iCurrentID >= $iID)
            {
                $iID = $iCurrentID+1;
            }    
        }    
        
        $strXPath = self::getXPathByType($pstrType, 1);
        $objArraySimpleXMLElement = $this->m_objSimpleXMLElement->xpath($strXPath); 
        
        $objSimpleXMLElementNew = $objArraySimpleXMLElement[0]->addChild(strtolower($pstrType));
        $objSimpleXMLElementNew->addChild('id', $iID);
        $objSimpleXMLElementNew->addChild('name', $pstrName);        
        
        $this->save();
        
        return $iID;
    }
    
    /**
    * Return id from datastore
    * 
    * @param string $pstrType Type de l'objet
    * @param string $pstrName Nom de l'action
    * 
    * @return int ID
    */
    function selectByName($pstrType, $pstrName)
    {
        $strXPath = self::getXPathByType($pstrType)."[./name='$pstrName']/id";
        $objArraySimpleXMLElement = $this->m_objSimpleXMLElement->xpath($strXPath);    
        if ($objArraySimpleXMLElement !== false && count($objArraySimpleXMLElement) > 0)
        {
            return (int)$objArraySimpleXMLElement[0]; 
        }
        else
        {
            return null;
        }    
    }   
    
    /**
    * Return name from datastore
    * 
    * @param string $pstrType Type de l'objet
    * @param int    $piID     Identificateur de l'action 
    * 
    * @return string Name
    */
    function selectByID($pstrType, $piID)
    {
        $strXPath = self::getXPathByType($pstrType)."[./id='$piID']/name";
        $objArraySimpleXMLElement = $this->m_objSimpleXMLElement->xpath($strXPath);    
        if ($objArraySimpleXMLElement !== false && count($objArraySimpleXMLElement) > 0)
        {
            return (string)$objArraySimpleXMLElement[0]; 
        }
        else
        {
            return null;
        }
    }
    
    /**
    * Return id and name from datastore of all object
    * 
    * @param string $pstrType Type de l'objet
    * 
    * @return ArrayObject
    */
    function selectAll($pstrType)
    {
        $objArrayResult = new ArrayObject(); 
        
        $strXPath = self::getXPathByType($pstrType);
        $objArraySimpleXMLElement = $this->m_objSimpleXMLElement->xpath($strXPath);    
        foreach ($objArraySimpleXMLElement as $objSimpleXMLElementObject)
        {  
            $strID = (string)$objSimpleXMLElementObject->id;
            $strName = (string)$objSimpleXMLElementObject->name;   
            $objArrayResult->append(array('id' => $strID, 'name' => $strName));   
        }
     
        return $objArrayResult;    
    }
    
    /**
    * Insert relation betwen Ressource and Action
    * 
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return void
    */
    function insertRessourceAction(com_borealis_acl_object_Ressource $pobjRessource, com_borealis_acl_object_Action $pobjAction)
    {
        $iID = $pobjRessource->getID();
        $strXPath = "/acl/ressources/ressource[./id='$iID']";
        $objArraySimpleXMLElement = $this->m_objSimpleXMLElement->xpath($strXPath); 
        
        if (count($objArraySimpleXMLElement[0]->children()) == 2)
        {
            $objArraySimpleXMLElement[0]->addChild('actions');       
        }
        $objSimpleXMLElementNew = $objArraySimpleXMLElement[0]->actions->addChild('action', $pobjAction->getID());     
        
        $this->save();        
    }
    
    /**
    * Remove relation betwen Ressource and Action
    * 
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return void
    */
    function deleteRessourceAction(com_borealis_acl_object_Ressource $pobjRessource, com_borealis_acl_object_Action $pobjAction)
    {
        $iRessourceID = $pobjRessource->getID();
        $iActionID = $pobjAction->getID();
        $strXPath = "/acl/ressources/ressource[./id='$iRessourceID']/actions/action[. ='$iActionID']"; 
        $objDomDocument = new DOMDocument();
        $objDomDocument->loadXML($this->m_objSimpleXMLElement->asXML());          
        $objDomXpath = new DOMXpath($objDomDocument); 
        $objDomElements = $objDomXpath->query($strXPath);
        foreach ($objDomElements as $objDomElement)
        {
            $objDomElement->parentNode->removeChild($objDomElement);           
        }
        $this->m_objSimpleXMLElement = simplexml_import_dom($objDomDocument);
        $this->save();        
    }
    
    /**
    * Return id and name of all action linked to the ressource
    * 
    * @param com_borealis_acl_object_Ressource $pobjRessource Target ressource
    * 
    * @return ArrayObject
    */
    function selectRessourceAction(com_borealis_acl_object_Ressource $pobjRessource)
    {
        $iID = $pobjRessource->getID();
        $strXPath = "/acl/ressources/ressource[./id='$iID']/actions/action";
        $objArraySimpleXMLElement = $this->m_objSimpleXMLElement->xpath($strXPath); 
            
        $objArrayResult = new ArrayObject(); 
        if ($objArraySimpleXMLElement !== false)
        {
            foreach ($objArraySimpleXMLElement as $strActionID)
            {
                $strName = $this->selectByID('Action', $strActionID);
                $objArrayResult->append(array('id' => $strActionID, 'name' => $strName));     
            }
        }
        
        return $objArrayResult;    
    }
    
    /**
    * Insert member into role
    * 
    * @param com_borealis_acl_object_Role $pobjRole     Target role
    * @param string                       $pstrUsername Nom d'utilisateur
    * 
    * @return void
    */
    function insertRoleMember(com_borealis_acl_object_Role $pobjRole, $pstrUsername)
    {
        $iID = $pobjRole->getID();
        $strXPath = "/acl/roles/role[./id='$iID']";
        $objArraySimpleXMLElement = $this->m_objSimpleXMLElement->xpath($strXPath); 
        
        if (count($objArraySimpleXMLElement[0]->children()) == 2)
        {
            $objArraySimpleXMLElement[0]->addChild('members');       
        }
        $objSimpleXMLElementNew = $objArraySimpleXMLElement[0]->members->addChild('member', $pstrUsername);     
        
        $this->save();     
    }
    
    /**
    * Delete member into role
    * 
    * @param com_borealis_acl_object_Role $pobjRole     Target role
    * @param string                       $pstrUsername Nom d'utilisateur
    * 
    * @return void
    */
    function deleteRoleMember(com_borealis_acl_object_Role $pobjRole, $pstrUsername)
    {
        $iRoleID = $pobjRole->getID();
        $strXPath = "/acl/roles/role[./id='$iRoleID']/members/member[. ='$pstrUsername']"; 
        $objDomDocument = new DOMDocument(); 
        $objDomDocument->loadXML($this->m_objSimpleXMLElement->asXML());          
        $objDomXpath = new DOMXpath($objDomDocument); 
        $objDomElements = $objDomXpath->query($strXPath);
        foreach ($objDomElements as $objDomElement)
        {
            $objDomElement->parentNode->removeChild($objDomElement);           
        }
        $this->m_objSimpleXMLElement = simplexml_import_dom($objDomDocument);
        $this->save();            
    }
    
    /**
    * Return list of member register in this role
    * 
    * @param com_borealis_acl_object_Role $pobjRole Target role  
    * 
    * @return ArrayObject of Username string
    */
    function selectRoleMember(com_borealis_acl_object_Role $pobjRole)
    {
        $iID = $pobjRole->getID();
        $strXPath = "/acl/roles/role[./id='$iID']/members/member";
        $objArraySimpleXMLElement = $this->m_objSimpleXMLElement->xpath($strXPath); 
              
        $objArrayResult = new ArrayObject(); 
        if ($objArraySimpleXMLElement !== false)
        {
            foreach ($objArraySimpleXMLElement as $strMember)
            {
                $objArrayResult->append($strMember);     
            }
        }
        
        return $objArrayResult;    
    }
    
    /**
    * Insert permision for ressource action into role
    * 
    * @param com_borealis_acl_object_Role      $pobjRole      Role
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return void
    */
    function insertRolePermission(com_borealis_acl_object_Role $pobjRole, com_borealis_acl_object_Ressource $pobjRessource, com_borealis_acl_object_Action $pobjAction)
    {
        $objSimpleXMLElementNew = $this->m_objSimpleXMLElement->permissions->addChild('permission');     
        $objSimpleXMLElementNew->addChild('role', $pobjRole->getID());
        $objSimpleXMLElementNew->addChild('ressource', $pobjRessource->getID()); 
        $objSimpleXMLElementNew->addChild('action', $pobjAction->getID()); 
        $this->save();         
    }
    
    /**
    * Delete permision for ressource action into role
    * 
    * @param com_borealis_acl_object_Role      $pobjRole      Role
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return void
    */
    function deleteRolePermission(com_borealis_acl_object_Role $pobjRole, com_borealis_acl_object_Ressource $pobjRessource, com_borealis_acl_object_Action $pobjAction)
    {
        $iRoleID = $pobjRole->getID();
        $iRessourceID = $pobjRessource->getID();
        $iActionID = $pobjAction->getID(); 
        $strXPath = "/acl/permissions/permission[./role='$iRoleID' and ./ressource='$iRessourceID' and ./action='$iActionID' ]"; 
        $objDomDocument = new DOMDocument(); 
        $objDomDocument->loadXML($this->m_objSimpleXMLElement->asXML());          
        $objDomXpath = new DOMXpath($objDomDocument); 
        $objDomElements = $objDomXpath->query($strXPath);
        foreach ($objDomElements as $objDomElement)
        {
            $objDomElement->parentNode->removeChild($objDomElement);           
        }
        $this->m_objSimpleXMLElement = simplexml_import_dom($objDomDocument);
        $this->save();                
    }
    
    /**
    * Return id and name of all action linked to the role (With duplicate value)
    * 
    * @param com_borealis_acl_object_Role $pobjRole Target role
    * 
    * @return ArrayObject
    */
    function selectRolePermissionAction(com_borealis_acl_object_Role $pobjRole)
    {
        $iID = $pobjRole->getID();
        $strXPath = "/acl/permissions/permission[./role='$iID']/action";
        $objArraySimpleXMLElement = $this->m_objSimpleXMLElement->xpath($strXPath); 
            
        $objArrayResult = new ArrayObject(); 
        if ($objArraySimpleXMLElement !== false)
        {
            foreach ($objArraySimpleXMLElement as $strActionID)
            {
                $strName = $this->selectByID('Action', $strActionID);
                $objArrayResult->append(array('id' => $strActionID, 'name' => $strName));     
            }
        }
        
        return $objArrayResult;        
    }
    
    /**
    * Return id and name of all ressource linked to the role (With duplicate value)
    * 
    * @param com_borealis_acl_object_Role $pobjRole Target role
    * 
    * @return ArrayObject
    */
    function selectRolePermissionRessource(com_borealis_acl_object_Role $pobjRole)
    {
        $iID = $pobjRole->getID();
        $strXPath = "/acl/permissions/permission[./role='$iID']/ressource";
        $objArraySimpleXMLElement = $this->m_objSimpleXMLElement->xpath($strXPath); 
            
        $objArrayResult = new ArrayObject(); 
        if ($objArraySimpleXMLElement !== false)
        {
            foreach ($objArraySimpleXMLElement as $strRessourceID)
            {
                $strName = $this->selectByID('Ressource', $strRessourceID);
                $objArrayResult->append(array('id' => $strRessourceID, 'name' => $strName));     
            }
        }
        
        return $objArrayResult;     
    }
    
    /**
    * Vérifie si l'utilisateur a le doit sur l'action de la ressource demander
    * 
    * @param string                            $pstrUsername  Utilisateur
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return bool true or false
    */
    function isAuthorized($pstrUsername,com_borealis_acl_object_Ressource $pobjRessource,com_borealis_acl_object_Action $pobjAction)
    {
        $iRessourceID = $pobjRessource->getID();
        $iActionID = $pobjAction->getID(); 
        $strXPath = "/acl/permissions/permission[./role = /acl/roles/role[./members/member='$pstrUsername']/id and ./ressource='$iRessourceID' and ./action='$iActionID']";
        $objArraySimpleXMLElement = $this->m_objSimpleXMLElement->xpath($strXPath); 
        if ($objArraySimpleXMLElement !== false && count($objArraySimpleXMLElement) > 0)
        {
            return true;  
        } 
        return false;  
    }
    

    // ###################################################################
    // # Private static
    // ###################################################################        

    /**
    * Return xpath to acces object
    * 
    * @param com_borealis_acl_object_Base $pobjBase Object
    * @param int                          $piType   Xpath type to object 0 to object master 1 
    * 
    * @return string
    */
    private static function getXPathByObject(com_borealis_acl_object_Base $pobjBase, $piType = 0)
    {
        $strType = 'Role';
        if ($pobjBase instanceof com_borealis_acl_object_Action)
        {
            $strType = 'Action';     
        }
        else if ($pobjBase instanceof com_borealis_acl_object_Role)
        {
            $strType = 'Role';     
        } 
        else if ($pobjBase instanceof com_borealis_acl_object_Ressource)
        {
            $strType = 'Ressource';     
        } 
        return self::getXPathByType($strType, $piType);  
    }
    
    /**
    * Return xpath to acces object
    * 
    * @param string $pstrType Type object
    * @param int    $piType   Xpath type to object 0 to object master 1 
    * 
    * @return string
    */
    private static function getXPathByType($pstrType, $piType = 0) 
    {
        $objArrayXPath= Array(
                             array(
                              'Action'  => '/acl/actions/action',             
                              'Role'    => '/acl/roles/role',
                              'Ressource'=> '/acl/ressources/ressource'
                              ),
                              array(
                              'Action'  => '/acl/actions',             
                              'Role'    => '/acl/roles',
                              'Ressource'=> '/acl/ressources'
                              ));  
        return $objArrayXPath[$piType][$pstrType];      
    }
  
    /**
    * Save modif to xml file
    * 
    * @return void
    */
    private function save()
    {
        $this->m_objSimpleXMLElement->asXML($this->m_strConfigFile);   
    }
    
}

?>