<?php
/**
* Récupère les informations d'un serveur ldap
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-authentication
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.boreal-is.com/license BSD License
* @link      http://www.boreal-is.com
*/

/**
* Récupère les informations d'un serveur ldap        
* 
* @category   PHP 
* @package    Borealis-authentication
* @subpackage Checker  
* @author     Pascal Martin 
*/ 
class com_borealis_authentication_checker_LDAPChecker extends com_borealis_authentication_checker_Checker
{
    private $m_strHostname = null; 
    private $m_iPort = null; 
    private $m_strSearchDN = null; 
    private $m_strUsernameColumn = null;
    private $m_objArrayOptions = array();
        
    /**
    * Constructeur de la classe com_borealis_authentication_checker_LDAPChecker
    * 
    * @param com_borealis_authentication_password_PasswordCreator $pobjPasswordCreator Créateur de mot de passe 
    * @param string                                               $pstrHostname        Adresse du serveur ldap
    * @param string                                               $piPort              Port du serveur ldap
    * @param string                                               $pstrSearchDN        Distinguished Name des object a rechercher
    * @param string                                               $pstrUsernameColumn  Nom de l'attribut qui contien le nom d'utilisateur (login name)
    * @param array                                                $pobjArrayOptions    Option suplémentaire pour la connexion ldap
    * 
    * @return com_borealis_authentication_checker_LDAPChecker
    */
    public function __construct(com_borealis_authentication_password_PasswordCreator $pobjPasswordCreator, $pstrHostname, $piPort, $pstrSearchDN, $pstrUsernameColumn, $pobjArrayOptions) 
    {        
        $this->m_strHostname = $pstrHostname;  
        $this->m_iPort = $piPort; 
        $this->m_strSearchDN = $pstrSearchDN; 
        $this->m_strUsernameColumn = $pstrUsernameColumn; 
        $this->m_objArrayOptions = $pobjArrayOptions;
        
        parent::__construct($pobjPasswordCreator);
    }
    
    /**
    * Vérifie le doit d'authentification
    * 
    * @param string $pstrUsername Nom d'utilisateur
    * @param string $pstrPassword Mot de passe
    * 
    * @return boolean Authorized 
    */
    public function isAuthorized($pstrUsername, $pstrPassword)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrUsername);
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrPassword);
        
        $objLDAPConnect=ldap_connect($this->m_strHostname, $this->m_iPort);
              
        foreach ($this->m_objArrayOptions as $piOption => $objValue)
        {
            ldap_set_option($objLDAPConnect, $piOption, $objValue);
        }
        $strSearchFilter = sprintf('%1$s=%2$s', $this->m_strUsernameColumn, $pstrUsername);
        $strBindDN = sprintf('%1$s,%2$s', $strSearchFilter, $this->m_strSearchDN);

        $bBind = false;

        $bBind = @ldap_bind($objLDAPConnect, $strBindDN, $this->getPasswordValue($pstrPassword));
        if (!$bBind)
        {
            $strMessage = ldap_error($objLDAPConnect);                
            if (strstr($strMessage, 'Invalid credentials') === false)
            { 
                ldap_close($objLDAPConnect);
                throw new Exception($strMessage);
            }
        }  

        ldap_close($objLDAPConnect);
        
        return $bBind;             
    }

}

?>