<?php
/**
* Récupère les informations de la base de donnée     
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-authentication
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.boreal-is.com/license BSD License
* @link      http://www.boreal-is.com
*/

/**
* Récupère les informations de la base de donnée
* 
* @category   PHP 
* @package    Borealis-authentication
* @subpackage Checker  
* @author     Pascal Martin 
*/ 
class com_borealis_authentication_checker_PDOChecker extends com_borealis_authentication_checker_Checker
{
    private $m_strDSN = null;
    private $m_strQuery = null;
    
    /**
    * Constructeur de la classe com_borealis_authentication_checker_PDOChecker 
    * 
    * @param com_borealis_authentication_password_PasswordCreator $pobjPasswordCreator Créateur de mot de passe 
    * @param string                                               $pstrDSN             Chaine de connexion à la basse de données
    * @param string                                               $pstrQuery           Requête sql pour la récupération des user doit retouné une ou plusieur ligne si l'authentification réussit
    * 
    * @return com_borealis_authentication_checker_PDOChecker
    */
    public function __construct(com_borealis_authentication_password_PasswordCreator $pobjPasswordCreator, $pstrDSN, $pstrQuery) 
    {        
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrDSN);
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrQuery);
        $this->m_strDSN = $pstrDSN;
        $this->m_strQuery = $pstrQuery;
        parent::__construct($pobjPasswordCreator);
    }
    
    /**
    * Vérifie le doit d'authentification
    * 
    * @param string $pstrUsername Nom d'utilisateur
    * @param string $pstrPassword Mot de passe
    * 
    * @return boolean Authorized 
    */
    public function isAuthorized($pstrUsername, $pstrPassword)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrUsername);
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrPassword);
        
        $objPDO = new PDO($this->m_strDSN);     
        $objStatement = $objPDO->prepare($this->m_strQuery);      
        $objStatement->bindParam(':username', $pstrUsername);
        $objStatement->bindParam(':password', $this->getPasswordValue($pstrPassword));
           
        self::execute($objStatement);

        return $objStatement->rowCount() > 0;                   
    }
     
    
    /**
    * Execute statement and throw exception on error
    * 
    * @param PDOStatement &$pobjStatement PDO Statement already prepare
    * 
    * @throws com_borealis_acl_exception_SQLException
    * 
    * @return void 
    */
    private static function execute(PDOStatement &$pobjStatement)
    {
        if (!$pobjStatement->execute())
        {
            $objArrayErrorInfo = $pobjStatement->errorInfo();
            throw new Exception("SQLSTATE: $objArrayErrorInfo[0] | Driver Error Code: $objArrayErrorInfo[1] | Description: $objArrayErrorInfo[2]");
        }      
    }  
}
  
?>
