<?php
/**
* Fichier php de code général utiliser par tous les sections du module ACL
*   Inclusion et configuration des différente library
*   Fonction utilitaire pour smarty
*   Authentification
*   Vérification des droits d'accés
*
* PHP versions 5
* 
* @category  PHP
* @package   ACL
* @author    Pascal Martin 
* @copyright Boréal Informations Stratégiques Inc.
* @license   http://www.boreal-is.com/license Borealis License
* @link      http://www.boreal-is.com
*/

$strCurrentDir = dirname(__FILE__);

header('Content-type: text/html; charset=UTF-8');
  
session_start();

// --------------------------------------
// Récupération de paramètre
// --------------------------------------

// Lang
if (isset($_REQUEST['lang']))
{
    $_SESSION['ACL_LANGUAGE'] = $_REQUEST['lang'];   
}
else if (!isset($_SESSION['ACL_LANGUAGE']))
{
    $_SESSION['ACL_LANGUAGE'] = "en_US";    
}

require_once "../config.php";
// --------------------------------------
// Set include path
// --------------------------------------
set_include_path(".".PATH_SEPARATOR.$g_strBorealisFoundation.
                     PATH_SEPARATOR.$g_strBorealisAcl.
                     PATH_SEPARATOR.$g_strSmarty.
                     PATH_SEPARATOR.$g_strBorealisUser);

// --------------------------------------
// Require class
// --------------------------------------
require_once 'com/borealis/Autoload.php';   
require_once 'Smarty.class.php';  
  
// --------------------------------------
// Configure i18n - Internationalisation 
// --------------------------------------
$g_strLanguage = $_SESSION['ACL_LANGUAGE'];
$g_strLanguageDir = './languages';
putenv("LANGUAGE=$g_strLanguage"); 
setlocale(LC_ALL, $g_strLanguage.".UTF8");
  
// --------------------------------------
// Configure library
// --------------------------------------

// Set acl setting
com_borealis_acl_SettingImpl::setDataStore('com_borealis_acl_datastore_XMLDataStore');
com_borealis_acl_SettingImpl::setXMLConfigFile("$strCurrentDir/acl.xml");

$objArrayDatastores = array();
//$objArrayDatastores[] = new com_borealis_users_datastore_PDODataStore(com_borealis_acl_SettingImpl::getPDODataStoreDSN(), com_borealis_users_Setting::PDODATASTORE_USERS_QUERY);
com_borealis_users_SettingImpl::setDatastores($objArrayDatastores);

// -----------------------------------------------
// --- Smarty init
// -----------------------------------------------
$g_objSmarty = new Smarty();
$g_objSmarty->register_function('gettext', 'smartyFunctionGettext');
$g_objSmarty->template_dir = dirname(__FILE__).'/smarty/templates';
$g_objSmarty->compile_dir  = dirname(__FILE__).'/smarty/templates_c';
$g_objSmarty->compile_check = true;

if (!is_dir($g_objSmarty->compile_dir))
{
    mkdir($g_objSmarty->compile_dir); 
}

// -----------------------------------------------
// --- Security Authenticate
// ----------------------------------------------- 
//$objPasswordCreator = new com_borealis_security_password_V2PasswordCreator(); 
//com_borealis_security_Authentication::withSmarty($g_objSmarty, 'login.tpl', $objPasswordCreator);
  
// -----------------------------------------------
// --- Check right
// ----------------------------------------------- 
/*
$strUsername = com_borealis_security_manager_SecurityManager::getInstance()->getUser()->getUsername();
$objRessource = com_borealis_acl_object_Ressource::getByName('administration.acl');
$objAction = com_borealis_acl_object_Action::getByName('update');
if (!com_borealis_acl_manager_ACLManager::isAuthorized($strUsername, $objRessource, $objAction))
{
    die('Droit insufisant');
}
*/
   
/**
* Smarty fonction for i18n - Fonction pour internationalisation du texte dans smarty
* Exemple :
*   gettext('sample') = 'sample :param1 :param2 "cool" ';
* 
*     {gettext key=sample}                          => sample :param1 :param2 "cool"
*     {gettext key=sample javascript=true}          => sample :param1 :param2 \"cool\"
*     {gettext key=sample param1=aaa param2=bbb}    => sample aaa bbb "cool" 
* 
* @param array $pobjArrayParams Paramétre de la fonction 
* 
* @return string
*/
function smartyFunctionGettext($pobjArrayParams)
{
    if (count($pobjArrayParams) == 0)
    {
        throw new Exception('Param key is mandatory');  
    }
    
    $strResult = gettext($pobjArrayParams['key']);
    
    // Replace param   
    foreach ($pobjArrayParams as $strParamName => $strValue)
    {
        if ($strParamName != 'key' && $strParamName != 'javascript')
        {
            $strResult = str_replace(":$strParamName", $strValue, $strResult);
        }
    }
    
    // Formatage pour l'échapement des " dans une chaine javascript
    if (isset($pobjArrayParams['javascript']) && $pobjArrayParams['javascript'] == 'true')
    {
        $strResult = str_replace('"', '\"', $strResult);
    } 
    
    return $strResult; 
}
        
?>
