/* 
Copyright (c) 2009-2011 Boreal - Information Strategies, published under the BSD license.
See http://geoprisma.org/license for the full text of the license. 
*/ 

Ext.namespace("GeoExt.ux.form");

GeoExt.ux.form.AttributeFilterPanelBuilder = Ext.extend(Ext.Panel, {

	filters: null,
	
	// TODO: initialize from ctor options (see QueryByRect.js)
	results: [],
	
	/**
	* APIMethod: addResult
	* Add the result to this.results array.
	*
	* Parameters:
	* result - {<OpenLayers.Result>} The result object to add
	*/
	addResult: function(result){
		this.results.push(result);
	},

	/**
	* APIMethod: addResults
	* Add the results to this.results array.
	*
	* Parameters:
	* results - {Array(<OpenLayers.Result>)} The result object to add
	*/
	addResults: function(results){
		for(var i=0, len=results.length; i<len; i++){
			this.addResult(results[i]);
		}
	},
	
	/**
	* Method: resetResults
	* Reset all results content.
	*/
	resetResults: function() {
		for(var i=0, len=this.results.length; i<len; i++){
			this.results[i].resetPanel();
		}
	},
	
	initComponent: function () {
		
		GeoExt.ux.form.AttributeFilterPanelBuilder.superclass.initComponent.call(this);
		
		if (!this.filters) { this.filters = []; }
		
		var i;
		for ( i = 0 ; i < this.filters.length ; i++ ) {
			this.add(this.makePanel(this.filters[i]));
		}
	},
	
	makePanel: function (options) {

		if (!options) { options = {}; }

		var logical_op = this._LOGICAL_OP_TO_OL[options.logical_op] || this._LOGICAL_OP_TO_OL["and"];
		var ignoreEmptyFields = options.ignoreempty || true;
		var noFeaturesOnEmptyForm = options.no_features_on_empty_form || false; // TODO: add in configuration
		var zoomToFeatures = options.zoom_to_features || false; // TODO: add in configuration
		var result = options.result || null;
		
		var that = this;
		
		var items = [], i, field;
		for ( i = 0 ; i < options.fields.length ; i++ ) {
			field = options.fields[i];
			items.push({
				xtype: field.xtype,
				anchor: field.anchor || "90%",
				name: field.name + "__" + field.operator,
				value: field.defaultvalue || "",
				fieldLabel: field.label,
				format: field.format
			});
		}

		// TODO: options.layers is an array of layers and can contain more than 1 element
		// if the resource linked to the widget references more than 1 layer. Make sure
		// to handle this case (how ??) ...
		
		var layer;
		if (options.layers && options.layers.length > 0) {
			layer = options.layers[0];
		}
		
		var panel = new GeoExt.ux.form.AttributeFilterPanel({
			layer: layer,
			title: options.label,
			labelAlign: 'top',
			items: items
		});
		
		panel.addButton({
			// TODO: make buttons' text configurable (i18n)
			text: "Filter",
			handler: function() {
			
				// used to list retrieved (filtered) features in a ResultExtGrid
				var addFeaturesToResult = function() {
					var features = layer.features;
					// FIXME : is it reasonable to use resources[0] ?
					var resource = layer.resources[0];
					if (resource) {
						for (var i = 0, nbFeats = features.length ; i < nbFeats ; i++) {
							features[i].resource = resource;
						}
					}
					that.resetResults();
					// execute each results (using features and htmlResults returned)
					for(var i=0, len=that.results.length; i<len; i++){
						that.results[i].showResult(features, []);
					}
				};
			
				// register handler to zoom to new extent of found features
				var zoomToFeaturesHandler = function(event) {
					var features = layer.features;
					var bounds = null;
					for (var i = 0 ; i < features.length ; i++) {
						if (bounds == null) {
							bounds = new OpenLayers.Bounds();
						}
						var feat = features[i];
						bounds.extend(feat.geometry.bounds);
					}
					if (bounds != null) {
						layer.map.zoomToExtent(bounds, false);
					}
				};
				
				var featuresAddedHandler = function(event) {
					// self-destroy the event handler
					layer.events.un({'featuresadded': featuresAddedHandler});
					
					if (zoomToFeatures) {
						zoomToFeaturesHandler();
					}
					
					if (that.results && that.results.length > 0) {
						addFeaturesToResult();
					}
					
				};
				
				// TODO: this event doesn't seem to do anything if the filter
				// returns 0 results. Is there another way to handle this in this case?
				layer.events.on(
					{
						'featuresadded': featuresAddedHandler
					}
				);
				
				// trigger filter request, the options passed to doAction
				// are passed to the protocol's read method, so one
				// can register a read callback here
				var o = {
					callback: function(response) {
					},
					logicalOp: logical_op,
					ignoreEmptyFields: ignoreEmptyFields,
					noFeaturesOnEmptyForm: noFeaturesOnEmptyForm,
					// TODO: strategy should not be passed to the FilterAction
					// (could FilterAction know itself what to do with the layer's strategies??)
					strategy: layer.strategies[0]
				};
				this.filter(o);
			},
			scope: panel
		});
		panel.addButton({
			// TODO: make buttons' text configurable (i18n)
			text: "Reset",
			handler: function() {
				var o = {
					callback: function(response) {
					},
					formPanel: panel,
					ignoreEmptyFields: ignoreEmptyFields,
					noFeaturesOnEmptyForm: noFeaturesOnEmptyForm
				};
				this.reset(o);
				that.resetResults();
				layer.removeAllFeatures();
			},
			scope: panel
		});
		
		
		// FIXME: ugly hack -- trigger the filter once after the layer loads
		/*
		var initialLayerLoadHandler = function () {
			layer.events.un({'featuresadded': initialLayerLoadHandler});
			panel.filter({
				callback: function(response) {},
				logicalOp: logical_op,
				ignoreEmptyFields: ignoreEmptyFields,
				noFeaturesOnEmptyForm: noFeaturesOnEmptyForm,
				result: result,
				strategy: strat
			});
		};
		*/
		
		// removed 20110125; interfers with first highlighting done by QueryByRect...
		/*
		layer.events.on(
			{
				'featuresadded': initialLayerLoadHandler
			}
		);
		*/
		
		
		return panel;
	},
	
	_LOGICAL_OP_TO_OL: {
		"and": OpenLayers.Filter.Logical.AND,
		"or": OpenLayers.Filter.Logical.OR,
		"not": OpenLayers.Filter.Logical.NOT
	}
});

Ext.reg("gxux_attributefilterpanelbuilder", GeoExt.ux.form.AttributeFilterPanelBuilder);
