<?xml version="1.0" encoding="utf-8"?>
<!-- 
   Copyright (c) 2009- Mapgears Inc., published under the BSD license.
   See http://geoprisma.org/license for the full text of the license.
 --> 
<!--
   NOTE: this widget uses GeoExt.MapPanel.guess() to get the
   GeoExt.MapPanel object and will only work if there is only one
   GeoExt.MapPanel object in the application.
-->
<xsl:stylesheet version="1.0" xmlns:php="http://php.net/xsl"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:dyn="http://exslt.org/dynamic"
    xmlns:geoextprintform="http://geoprisma.org/geoextprintform"
    extension-element-prefixes="dyn">

  <xsl:output method="html" indent="yes" encoding="utf-8" omit-xml-declaration="yes"/>
  <xsl:namespace-alias stylesheet-prefix="php" result-prefix="xsl" />

  <!-- printWidgetSource START  -->
  <xsl:template name="geoextprintform:printWidgetSource">
    <xsl:variable name="WidgetType">
      <xsl:text>geoextprintform</xsl:text>
    </xsl:variable>

    <!-- CSS -->
    <link rel="stylesheet" type="text/css" >
      <xsl:attribute name="href" >
        <xsl:value-of select="$g_widgets_url"/>
        <xsl:text>/geoextprintform/resources/css/geoextprintform.css</xsl:text>
      </xsl:attribute>
    </link>

    <!-- JS -->
    <script>
        <xsl:attribute name="src" >
             <xsl:value-of select="$g_widgets_url"/>
             <xsl:text>/geoextprintform/GeoExtPrintForm.js</xsl:text>
        </xsl:attribute>
    </script>

    <script>
      <xsl:attribute name="src" >
        <xsl:value-of select="$g_widgets_url"/>
        <xsl:text>/geoextprintform/lib/org.GeoPrisma.form.MapScaleLabel.js</xsl:text>
      </xsl:attribute>
    </script>

    <!-- i18n : "fr_CA" -->
    <xsl:if test="$g_locale = 'fr_CA'">
      <script>
        <xsl:attribute name="src" >
          <xsl:value-of select="$g_widgets_url"/>
          <xsl:text>/geoextprintform/resources/lang/fr.js</xsl:text>
        </xsl:attribute>
      </script>
    </xsl:if>

    <script type="text/javascript">
      <!-- some more i18n setup -->
      Ext.namespace("GeoExt.data")
      GeoExt.data.PrintProvider &amp;&amp; Ext.apply(GeoExt.data.PrintProvider.prototype, {
          unsuportedText: <xsl:value-of select="php:functionString('GeoExtPrintForm::getText','i18n_unsuported','JS')" />,
          excludedText: <xsl:value-of select="php:functionString('GeoExtPrintForm::getText','i18n_excluded','JS')" />,
          proceedAnywayText: <xsl:value-of select="php:functionString('GeoExtPrintForm::getText','i18n_proceed_anyway','JS')" />,
          turnLayersOffText: <xsl:value-of select="php:functionString('GeoExtPrintForm::getText','i18n_turn_layers_off','JS')" />,
          tileCacheWMSUnsuportedText: <xsl:value-of select="php:functionString('GeoExtPrintForm::getText','i18n_tilecache_wms_unsuported','JS')" />,
          activeDrawFeatureControlText: <xsl:value-of select="php:functionString('GeoExtPrintForm::getText','i18n_active_draw_feature_control_text','JS')" />
      });

      var objGPWidget<xsl:value-of select="$WidgetType" />GeoExtToolbarOptions;
      var objGPWidget<xsl:value-of select="$WidgetType" />Container = [];
    </script>


   </xsl:template>
  <!-- printWidgetSource END  -->

  <!-- printWidgetExecution START  -->
  <xsl:template name="geoextprintform:printWidgetExecution">
    <xsl:param name="pWidgetName" />
    <xsl:param name="pMapName" />

    <xsl:variable name="WidgetType">
      <xsl:text>geoextprintform</xsl:text>
    </xsl:variable>

    <script type="text/javascript">
        oMap = objGPWidget<xsl:value-of select="$pMapName"/>;
        var objGPWidget<xsl:value-of select="$pWidgetName" />GeoExtToolbarOptions;
        <xsl:variable name="mapfishPrintService">
          <xsl:value-of select="./options/servicename" />
        </xsl:variable>

        var printCapabilities = <xsl:value-of select="php:functionString('geoextprintform::getPrintCapabilitiesWithProxyURL',$mapfishPrintService)"></xsl:value-of>;

        <!-- validation, won't prevent javascript errors, but at least send
             a warning -->
        if (!printCapabilities ||
            !printCapabilities.scales ||
            !printCapabilities.dpis ||
            !printCapabilities.layouts) {
            alert("Warning: your MapFishPrint service returned an incomplete print capabilities, which may cause errors. Please, contact your administrator.");
        }

        <!-- make sure some of the objects returned are arrays -->
        if (printCapabilities.scales.scale) {
            var objScale = printCapabilities.scales.scale;
            printCapabilities.scales = [];
            printCapabilities.scales.push(objScale);
        }
        if (printCapabilities.dpis.dpi) {
            var objDpi = printCapabilities.dpis.dpi;
            printCapabilities.dpis = [];
            printCapabilities.dpis.push(objDpi);
        }
        if (printCapabilities.layouts.layout) {
            var objLayout = printCapabilities.layouts.layout;
            printCapabilities.layouts = [];
            printCapabilities.layouts.push(objLayout);
        }

        <!-- legend -->
        <xsl:if test="./options/legend = 'true'">
            if (printCapabilities.createURL) {
                printCapabilities.createURL = Ext.urlAppend(
                    printCapabilities.createURL,
                    "osmprintlegend=true"
                );
            }
        </xsl:if>

        <!-- PrintProvider -->
        var printProvider = new GeoExt.data.PrintProvider({
            method: "POST",
            capabilities: printCapabilities,
            customParams: {
            <xsl:if test="./options/noMapTitleField = 'true'">
                mapTitle: "Printing Demo"
            </xsl:if>
            }
        });
        <!-- beforeprint validation -->
        printProvider.on("beforeprint", printProvider.beforePrint);
        var printProvider<xsl:value-of select="$pWidgetName" /> = printProvider;

        <!-- defaultResolution -->
        <xsl:if test="./options/defaultResolution">
          var index = printProvider.dpis.find('value', '<xsl:value-of select="./options/defaultResolution" />');
          if (index != -1) {
              printProvider.setDpi(printProvider.dpis.getAt(index));
          }
        </xsl:if>

        <!-- defaultLayout -->
        <xsl:if test="./options/defaultLayout">
          var index = printProvider.layouts.find('name', '<xsl:value-of select="./options/defaultLayout" />');
          if (index != -1) {
              printProvider.setLayout(printProvider.layouts.getAt(index));
          }
        </xsl:if>

        <!-- PrintPage and page layer -->
        var printPage = new GeoExt.data.PrintPage({
            printProvider: printProvider
        });
        var printPage<xsl:value-of select="$pWidgetName" /> = printPage;
    
        var printPageLayer = new OpenLayers.Layer.Vector(
            "Print extent", {displayInLayerSwitcher: false}
        );
        printPageLayer.addFeatures(printPage.feature);
        oMap.addLayer(printPageLayer);
        var printPageLayer<xsl:value-of select="$pWidgetName" /> = printPageLayer;
    
        oMap.events.on({
            "moveend": function(){
                printPage<xsl:value-of select="$pWidgetName" />.fit(this, {mode: "screen"});
            },
            scope: oMap
        });

        <!-- move print page layer to top after "printallwidgetexecutions" -->
        oMap.events.on({
            "printallwidgetexecutions": function(event) {
                this.map.setLayerIndex(this.layer, this.map.layers.length)
            }, scope: {
                layer: printPageLayer,
                map: oMap
            }
        });

        var formPanelItems = [];

        <!-- noMapTitleField -->
        <xsl:choose>
          <xsl:when test="./options/noMapTitleField = 'true'"></xsl:when>
          <xsl:otherwise>
            formPanelItems.push({
                xtype: "textfield",
                name: "mapTitle",
            <!-- defaultComment-->
            <xsl:choose>
              <xsl:when test="./options/defaultMapTitle">
                value: '<xsl:value-of select="./options/defaultMapTitle" />',
              </xsl:when>
              <xsl:otherwise>
                value: "",
              </xsl:otherwise>
            </xsl:choose>
            <xsl:variable name="mapTitleFieldLabel">
              <xsl:variable name="mapTitleFieldLabelOption">
                <xsl:for-each select="./options/mapTitleFieldLabel">
                  <xsl:call-template name="getText" />
                </xsl:for-each>
              </xsl:variable>
              <xsl:choose>
                <xsl:when test="$mapTitleFieldLabelOption != ''">
                  <xsl:value-of select="$mapTitleFieldLabelOption" />
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="php:functionString('GeoExtPrintForm::getText','i18n_mapTitle','JS')" />
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>
                fieldLabel: <xsl:value-of select="$mapTitleFieldLabel" />,
            <xsl:if test="./options/mapTitleMaxLength">
                maxLength: <xsl:value-of select="./options/mapTitleMaxLength" />,
                autoCreate: {tag: 'input', type: 'text', size: '20', autocomplete: 'off', maxlength: '<xsl:value-of select="./options/mapTitleMaxLength" />'},
            </xsl:if>
                plugins: new GeoExt.plugins.PrintProviderField({
                    printProvider: printProvider
                })
            });
          </xsl:otherwise>
        </xsl:choose>

        <!-- noCommentField -->
        <xsl:choose>
          <xsl:when test="./options/noCommentField = 'true'"></xsl:when>
          <xsl:otherwise>
            formPanelItems.push({
                xtype: "textarea",
                name: "comment",
            <!-- defaultComment-->
            <xsl:choose>
              <xsl:when test="./options/defaultComment">
                value: '<xsl:value-of select="./options/defaultComment" />',
              </xsl:when>
              <xsl:otherwise>
                value: "",
              </xsl:otherwise>
            </xsl:choose>
                fieldLabel: <xsl:value-of select="php:functionString('GeoExtPrintForm::getText','i18n_comment','JS')" />,
                plugins: new GeoExt.plugins.PrintPageField({
                    printPage: printPage
                })
            });
          </xsl:otherwise>
        </xsl:choose>

        <!-- noLayoutField -->
        <xsl:choose>
          <xsl:when test="./options/noLayoutField = 'true'"></xsl:when>
          <xsl:otherwise>
            formPanelItems.push({
                xtype: "combo",
                store: printProvider.layouts,
                displayField: "name",
                fieldLabel: <xsl:value-of select="php:functionString('GeoExtPrintForm::getText','i18n_layout','JS')" />,
                typeAhead: true,
                mode: "local",
                triggerAction: "all",
                plugins: new GeoExt.plugins.PrintProviderField({
                    printProvider: printProvider
                })
            });
          </xsl:otherwise>
        </xsl:choose>

        <!-- noResolutionField -->
        <xsl:choose>
          <xsl:when test="./options/noResolutionField = 'true'"></xsl:when>
          <xsl:otherwise>
            formPanelItems.push({
                xtype: "combo",
                store: printProvider.dpis,
                displayField: "name",
                fieldLabel: <xsl:value-of select="php:functionString('GeoExtPrintForm::getText','i18n_resolution','JS')" />,
                tpl: '<tpl for="."><div class="x-combo-list-item">{name} dpi</div></tpl>',
                typeAhead: true,
                mode: "local",
                triggerAction: "all",
                plugins: new GeoExt.plugins.PrintProviderField({
                    printProvider: printProvider
                }),
                // the plugin will work even if we modify a combo value
                setValue: function(v) {
                    v = parseInt(v) + " dpi";
                    Ext.form.ComboBox.prototype.setValue.apply(this, arguments);
                }
            });
          </xsl:otherwise>
        </xsl:choose>

        <!-- noScaleField -->
        <xsl:choose>
          <xsl:when test="./options/noScaleField = 'true'"></xsl:when>
          <xsl:otherwise>
            formPanelItems.push({
                xtype: "combo",
                store: printProvider.scales,
                displayField: "name",
                fieldLabel: <xsl:value-of select="php:functionString('GeoExtPrintForm::getText','i18n_scale','JS')" />,
                typeAhead: true,
                mode: "local",
                triggerAction: "all",
                plugins: new GeoExt.plugins.PrintPageField({
                    printPage: printPage
                })
            });
            formPanelItems.push(new org.GeoPrisma.form.MapScaleLabel({
                map: objGPWidget<xsl:value-of select="$pMapName"/>,
                printPage: printPage
            }));
          </xsl:otherwise>
        </xsl:choose>

        <!-- noRotationField -->
        <xsl:choose>
          <xsl:when test="./options/noRotationField = 'true'"></xsl:when>
          <xsl:otherwise>
            formPanelItems.push({
                xtype: "textfield",
                name: "rotation",
                fieldLabel: <xsl:value-of select="php:functionString('GeoExtPrintForm::getText','i18n_rotation','JS')" />,
                plugins: new GeoExt.plugins.PrintPageField({
                    printPage: printPage
                })
            });
          </xsl:otherwise>
        </xsl:choose>

        <!-- print formPanel -->
        var formPanel = new Ext.Panel({
        <xsl:choose>
          <xsl:when test="./options/title">
            title: <xsl:for-each select="./options/title"><xsl:call-template name="getText" /></xsl:for-each>,
          </xsl:when>
          <xsl:otherwise>
            title: <xsl:value-of select="php:functionString('GeoExtPrintForm::getText','i18n_panel_title','JS')" />,
          </xsl:otherwise>
        </xsl:choose>
            border: false,
            bodyStyle: "padding:5px",
            labelAlign: "top",
            defaults: {anchor: "100%"},
            items: formPanelItems,
            layout: 'form',
            buttons: [{
                text: <xsl:value-of select="php:functionString('GeoExtPrintForm::getText','i18n_create_pdf','JS')" />,
                handler: function() {
                    printProvider<xsl:value-of select="$pWidgetName" />.print(GeoExt.MapPanel.guess(), printPage<xsl:value-of select="$pWidgetName" />);
                }
            }]
        });

        // show/hide page layer on panel expand/collapse
        formPanel.addListener(
            "expand",
            function(){
                printPageLayer<xsl:value-of select="$pWidgetName" />.setVisibility(true);
            }
        );
        formPanel.addListener(
            "collapse",
            function(){
                printPageLayer<xsl:value-of select="$pWidgetName" />.setVisibility(false);
            }
        );

        <!-- mask while printing -->
        printProvider.on("beforeprint", function() {
            this.el.mask(
                <xsl:value-of select="php:functionString('GeoExtPrintForm::getText','i18n_generating_pdf_please_wait_text','JS')" />,
                'x-mask-loading'
            );
        }, formPanel);
        printProvider.on("print", function() {
            this.el.unmask();
        }, formPanel);
        printProvider.on("printexception", function() {
            this.el.unmask();
        }, formPanel);

        <!-- IE open -->
        <xsl:if test="./options/ieWindowOpen = 'true'">
        Ext.isIE &amp;&amp; printProvider.on("beforedownload", function(printProvider, url) {
            // Make sure that InternetExplorer don't replace the content tab
            // with the pdf
            window.open(url);
            return false;
        });
        </xsl:if>

        <!-- if WMSmap_resolution if isMapServer -->
        printProvider.on("encodelayer", function(provider, layer, encLayer) {
            if (layer instanceof  OpenLayers.Layer.WMS &amp;&amp;
                layer.vendor === "mapserver"
            ) {
                encLayer.customParams = encLayer.customParams || {};
                encLayer.customParams.map_resolution = provider.dpi.get("value");
            }
        }, printProvider);
    
        objGPWidget<xsl:value-of select="$WidgetType" />Container.push(formPanel);
    </script>

  </xsl:template>
  <!-- printWidgetExecution END  -->

  <!-- drawWidgets START  -->
  <xsl:template name="geoextprintform:drawWidgets">
    <xsl:param name="pContainer" />

    <xsl:variable name="WidgetType">
      <xsl:text>geoextprintform</xsl:text>
    </xsl:variable>
	if (typeof(objGPWidget<xsl:value-of select="$WidgetType" />Container) != 'undefined' &amp;&amp; typeof(<xsl:value-of select="$pContainer" />) != 'undefined')
    {
        for (var i=0, len=objGPWidget<xsl:value-of select="$WidgetType" />Container.length; i&lt;len; i++) {
            if(<xsl:value-of select="$pContainer" />['ctype']){
                <xsl:value-of select="$pContainer" />.add(
                    objGPWidget<xsl:value-of select="$WidgetType" />Container[i]
                );
            } else {
                <xsl:value-of select="$pContainer" />.items.push(
                    objGPWidget<xsl:value-of select="$WidgetType" />Container[i]
                )
            }
        }
    }
  </xsl:template>
  <!-- drawWidgets END  -->

</xsl:stylesheet>
