<?xml version="1.0" encoding="utf-8"?>
<!-- 
   Copyright (c) 2009- Boreal - Information Strategies, published under the BSD license.
   See http://geoprisma.org/license for the full text of the license.
 --> 

<xsl:stylesheet version="1.0" xmlns:php="http://php.net/xsl"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:dyn="http://exslt.org/dynamic"
    xmlns:mapfishlayertree="http://geoprisma.org/mapfishlayertree" 
    extension-element-prefixes="dyn">
    
  <xsl:output method="html" indent="yes" encoding="utf-8" omit-xml-declaration="yes"/> 
  <xsl:namespace-alias stylesheet-prefix="php" result-prefix="xsl" />
  
<!-- 
     Function : printWidgetSource

     Print the source code for this widget
-->
  <!-- Call for each widget type-->
  <xsl:template name="mapfishlayertree:printWidgetSource">
    <script>
        <xsl:attribute name="src" >
             <xsl:value-of select="$g_widgets_url"/>
             <xsl:text>/mapfishlayertree/MapFishLayerTree.js</xsl:text>
        </xsl:attribute>
    </script>
  </xsl:template>


<!-- 
     Function : printWidgetExecution

     Print the execution code for this widget
-->
  <!-- Call for each widget name-->
  <xsl:template name="mapfishlayertree:printWidgetExecution">
    <xsl:param name="pWidgetName" />
    <xsl:param name="pResourceName" />
    <xsl:param name="pMapName" />

    <xsl:variable name="sXpathMap">
      <xsl:text>/geoprisma/widgets/widget[./name='</xsl:text>
      <xsl:value-of select="$pWidgetName"/>
      <xsl:text>']</xsl:text>
    </xsl:variable>

    <xsl:for-each select="dyn:evaluate($sXpathMap)">

      <xsl:for-each select="./options/nodes">
        <script type="text/javascript">

          aModel<xsl:value-of select="$pWidgetName"/> = [

          <xsl:call-template name="mapfishlayertree:printLayerTreeNode">
            <xsl:with-param name="pWidgetName" select="$pWidgetName" />
          </xsl:call-template>

          ];

          
          var div = document.createElement("div");
          div.setAttribute("id", "osmDiv_<xsl:value-of select='$pWidgetName'/>");
          document.body.appendChild(div);

          var objGPWidget<xsl:value-of select="$pWidgetName"/> = new mapfish.widgets.LayerTree({
              map: oMap,
              el: 'osmDiv_<xsl:value-of select="$pWidgetName"/>',
              model: aModel<xsl:value-of select="$pWidgetName"/>,
              title: <xsl:value-of select="php:functionString('MapFishLayerTree::getText','i18n_layer_tree_title','JS')" />,
              border: false,
              plugins: [
                  mapfish.widgets.LayerTree.createDisableLayersPlugin(72)
              ]
          });

          var layerTree = objGPWidget<xsl:value-of select="$pWidgetName"/>;

          <!--
              The visibility of Vector layers can sometime be changed automatically (outside of the
              layertree).  The following fix ensure that the layer tree node gets checked/unchecked
              according to theses events.
            -->
          for(var key in layerTree.layerNameToLayer){
              <!-- Get the layer object -->
              layer = layerTree.layerNameToLayer[key];

              <!-- Does the layer has sublayer nodes ? --> 
              layerWithSublayers = (layerTree.layersWithSublayers[key]);

              if(layerWithSublayers) {
                  layer.map.events.on({
                      "changelayer": layerTree.toggleSubLayerNode,
                      scope: {'layerTree': layerTree}
                  });
              } else {
                  layer.events.on({
                      "visibilitychanged": layerTree.toggleLayerNode,
                      scope: {'layer': layer, 'layerTree': layerTree}
                  });
              }
          }

        </script>
      </xsl:for-each>

    </xsl:for-each>
  </xsl:template>

  <xsl:template name="mapfishlayertree:printLayerTreeNode">
    <xsl:param name="pWidgetName" />

    <xsl:for-each select="node">

      <xsl:variable name="pServiceName">
        <xsl:call-template name="getDataStoreInfoFromWidget">
          <xsl:with-param name="pWidgetName" select="$pWidgetName" />
          <xsl:with-param name="pServiceType" select="'wms'" />
          <xsl:with-param name="pNodeName"><xsl:text>service</xsl:text></xsl:with-param>
          <xsl:with-param name="pResourceName" select="./resourcename"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:variable name="pLayerName">
        <xsl:call-template name="getDataStoreInfoFromWidget">
          <xsl:with-param name="pWidgetName" select="$pWidgetName" />
          <xsl:with-param name="pServiceType" select="'wms'" />
          <xsl:with-param name="pNodeName"><xsl:text>params/layers</xsl:text></xsl:with-param>
          <xsl:with-param name="pResourceName" select="./resourcename"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:if test="position()>1">,</xsl:if>
      {
          <xsl:if test="./textkey">
            text: <xsl:for-each select="./textkey" ><xsl:call-template name="getText" /></xsl:for-each>,   
          </xsl:if>          
      <xsl:choose>
        <xsl:when test="./checked = 'true'">
          checked: true,
        </xsl:when>
        <xsl:when test="./checked = 'undefined'">
          checked: undefined,
        </xsl:when>
        <xsl:otherwise>
          checked: false,
        </xsl:otherwise>
      </xsl:choose>
      <xsl:choose>
        <xsl:when test="./layername">

          <![CDATA[
            layerName: "]]><xsl:value-of select='./layername' /><![CDATA["]]>

        <xsl:if test="./resourcename"><![CDATA[
          +(oMap.getLayersByName("]]><xsl:value-of select='./layername' /><![CDATA[")[0].CLASS_NAME == "OpenLayers.Layer.WMS" && typeof oMap.getLayersByName("]]><xsl:value-of select='./layername' /><![CDATA[")[0].params.LAYERS != "string" &&  oMap.getLayersByName("]]><xsl:value-of select='./layername' /><![CDATA[")[0].params.LAYERS.length >= 2 ? ":]]><xsl:value-of select='$pLayerName' /><![CDATA[":"")]]>
        </xsl:if>,

        </xsl:when> <!-- end of when 'layername'  -->
        <xsl:otherwise>
          <!-- If no 'layername' node was set, get the layer name by using
               the resourcename and service type nodes (if they are set)
            -->
          <xsl:if test="./servicetype"><xsl:if test="./resourcename">
            layerName: "" +

            (
                oMap.getLayersByResource(
                    "<xsl:value-of select='./resourcename' />", 
                    "<xsl:value-of select='./servicetype' />"
                ).length == 1
            
                ? oMap.getLayersByResource(
                    "<xsl:value-of select='./resourcename' />", 
                    "<xsl:value-of select='./servicetype' />"
                )[0]['name'] +
                (
                    oMap.getLayersByResource(
                        "<xsl:value-of select='./resourcename' />", 
                        "<xsl:value-of select='./servicetype' />"
                    )[0]['serviceType'] == "wms"
                    &amp;&amp; typeof oMap.getLayersByResource(
                        "<xsl:value-of select='./resourcename' />", 
                        "<xsl:value-of select='./servicetype' />"
                    )[0].params.LAYERS != "string"
                    &amp;&amp; oMap.getLayersByResource(
                        "<xsl:value-of select='./resourcename' />", 
                        "<xsl:value-of select='./servicetype' />"
                    )[0]['params'].LAYERS.length &gt;= 2
                        ? ":<xsl:value-of select='$pLayerName' />"
                        : ""
                )
                : ""
            )
          ,
          </xsl:if></xsl:if>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:if test="./icon">
        icon: "<xsl:value-of select='./icon' />",
      </xsl:if>

      <xsl:if test="./maxScale">
        maxScale: "<xsl:value-of select='./maxScale' />",
      </xsl:if>
      <xsl:if test="./minScale">
        minScale: "<xsl:value-of select='./minScale' />",
      </xsl:if>
      
      <xsl:if test="./nodes">
        <xsl:choose>
          <xsl:when test="./expanded = 'false'">
            expanded: false,
          </xsl:when>
          <xsl:otherwise>
            expanded: true,
          </xsl:otherwise>
        </xsl:choose>
        children: [
        <xsl:for-each select="./nodes">
          <xsl:call-template name="mapfishlayertree:printLayerTreeNode">
            <xsl:with-param name="pWidgetName" select="$pWidgetName"/>
          </xsl:call-template>
        </xsl:for-each>
        ],
      </xsl:if>
        last: ''
      }
    </xsl:for-each>

  </xsl:template>


<!-- 
     Function : drawWidget

     Output the widget depending on the render type
-->
  <!-- Call for each widget name -->
  <xsl:template name="mapfishlayertree:drawWidget">
    <xsl:param name="pWidgetName" />

      <!-- <![CDATA[items: [objGPWidget]]><xsl:value-of select="$pWidgetName" />] -->
      <![CDATA[objGPWidget]]><xsl:value-of select="$pWidgetName" />

  </xsl:template>


<!--
    Function: getWidgetDepedencies

    Return the list of dependencies for this widget
-->
  <!-- Call for each widget type-->
  <xsl:template name="mapfishlayertree:getWidgetDepedencies">

  </xsl:template>

  
</xsl:stylesheet>
