<?xml version="1.0" encoding="utf-8"?>
<!-- 
   Copyright (c) 2009- Boreal - Information Strategies, published under the BSD license.
   See http://geoprisma.org/license for the full text of the license.
 --> 
<xsl:stylesheet version="1.0" xmlns:php="http://php.net/xsl"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:dyn="http://exslt.org/dynamic"
    xmlns:mapfishrecenter="http://geoprisma.org/mapfishrecenter"
    extension-element-prefixes="dyn">

  <xsl:output method="html" indent="yes" encoding="utf-8" omit-xml-declaration="yes"/>
  <xsl:namespace-alias stylesheet-prefix="php" result-prefix="xsl" />

<!-- 
     Function : printWidgetSourcef

     Print the source code for this widget
-->
  <!-- Call for each widget type-->
  <xsl:template name="mapfishrecenter:printWidgetSource"><link rel="stylesheet" type="text/css" ></link>
    <script>
        <xsl:attribute name="src" >
             <xsl:value-of select="$g_widgets_url"/>
             <xsl:text>/mapfishrecenter/MapFishRecenter.js</xsl:text>
        </xsl:attribute>    
    </script>

    <script>            
        Ext.apply(mapfish.widgets.recenter.DataField.prototype, {
            i18n_filter_field_label : <xsl:value-of select="php:functionString('mapfishrecenter::getText','i18n_filter_field_label','JS')" />,
            i18n_filter_box_label_start : <xsl:value-of select="php:functionString('mapfishrecenter::getText','i18n_filter_box_label_start','JS')" />,
            i18n_filter_box_label_anywhere : <xsl:value-of select="php:functionString('mapfishrecenter::getText','i18n_filter_box_label_anywhere','JS')" />,
            i18n_filter_box_label_end : <xsl:value-of select="php:functionString('mapfishrecenter::getText','i18n_filter_box_label_end','JS')" />,
            i18n_min_char_label : <xsl:value-of select="php:functionString('mapfishrecenter::getText','i18n_min_char_label','JS')" />,
            i18n_show_center_label : <xsl:value-of select="php:functionString('mapfishrecenter::getText','i18n_show_center_label','JS')" />       
        });   
    </script>
    
   </xsl:template>

<!-- 
     Function : printWidgetExecution

     Print the execution code for this widget
-->
  <!-- Call for each widget name-->
  <xsl:template name="mapfishrecenter:printWidgetExecution">
    <xsl:param name="pWidgetName" />
    <xsl:param name="pResourceName" />
    <xsl:param name="pMapName" />

    <!-- get draw mode  -->
    <xsl:variable name="pDrawMode">
      <xsl:value-of select="$drawmode" />
    </xsl:variable>
    
    <!-- resource name : this widget supports only 1 resource,
         so only the first is selected  -->
    <xsl:variable name="resourceName">
      <xsl:for-each select="./resources">
        <xsl:value-of select="./resource[1]" />
      </xsl:for-each>
    </xsl:variable>
    
    <!-- this widget supports only featureserver -->
    <xsl:variable name="pServiceType">
      <xsl:text>featureserver</xsl:text>  
    </xsl:variable>
    
    <!-- get proxy url -->
    <xsl:variable name="pProxyURL">
      <xsl:value-of select="$proxyurl" />
    </xsl:variable>
    
    <!-- get defaultZoom  -->
    <xsl:variable name="pDefaultZoom">
      <xsl:call-template name="getWidgetOptionByName">
        <xsl:with-param name="pWidgetName" select="$pWidgetName" />        
        <xsl:with-param name="pOptionName">
            <xsl:text>defaultzoom</xsl:text>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:variable>
    
    <!-- get forceFilters  -->
    <xsl:variable name="pForceFilters">
      <xsl:call-template name="getWidgetOptionByName">
        <xsl:with-param name="pWidgetName" select="$pWidgetName" />        
        <xsl:with-param name="pOptionName">
            <xsl:text>forcefilters</xsl:text>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:variable>

    <!-- get displayField  -->
    <xsl:variable name="pDisplayField">
      <xsl:for-each select="options/fields/field"><xsl:value-of select="displayfield"/>;</xsl:for-each>
    </xsl:variable>
    
    <!-- get label  -->
    <xsl:variable name="pLabel">
      <xsl:for-each select="options/fields/field"><xsl:for-each select="label" ><xsl:call-template name="getText" /></xsl:for-each>,</xsl:for-each>
    </xsl:variable>
    
    <!-- get title  -->
    <xsl:variable name="pTitle">
      <xsl:call-template name="getWidgetOptionByName">
        <xsl:with-param name="pWidgetName" select="$pWidgetName" />        
        <xsl:with-param name="pOptionName">
            <xsl:text>title</xsl:text>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:variable>

    <!-- get queryParam  -->
    <xsl:variable name="pQueryParam">
      <xsl:for-each select="options/fields/field"><xsl:value-of select="queryparam"/>;</xsl:for-each>
    </xsl:variable>
    
    <!-- get filterType  -->
    <xsl:variable name="pFilterType">
      <xsl:for-each select="options/fields/field"><xsl:value-of select="filter"/>;</xsl:for-each>
    </xsl:variable>
    
    <!-- get service Name -->
    <xsl:variable name="pServiceName">
      <xsl:call-template name="getDataStoreInfoFromWidget">
        <xsl:with-param name="pWidgetName" select="$pWidgetName" />
        <xsl:with-param name="pServiceType" select="$pServiceType" />
        <xsl:with-param name="pNodeName"><xsl:text>service</xsl:text></xsl:with-param>
        <xsl:with-param name="pResourceName" select="$resourceName" />
      </xsl:call-template>
    </xsl:variable>

    <!-- get layer name -->
    <xsl:variable name="pLayerName">
      <xsl:call-template name="getDataStoreInfoFromWidget">
        <xsl:with-param name="pWidgetName" select="$pWidgetName" />
        <xsl:with-param name="pServiceType" select="$pServiceType" />
        <xsl:with-param name="pNodeName"><xsl:text>params/layers</xsl:text></xsl:with-param>
        <xsl:with-param name="pResourceName" select="$resourceName" />
      </xsl:call-template>
    </xsl:variable>

    <!-- javascript START -->
	<script type="text/javascript">

	<![CDATA[

		var objGPWidget]]><xsl:value-of select="$pWidgetName"/><![CDATA[ = {
              xtype: 'datafieldrecenter',
			  name: ']]><xsl:value-of select="$pWidgetName" /><![CDATA[',
              title: ]]><xsl:value-of select="php:functionString('MapFishRecenter::getText','i18n_recenter_title','JS')" /><![CDATA[,
              bodyStyle: 'padding: 7px;',
              ]]><xsl:if test="$pDefaultZoom != ''">
	        defaultZoom: <xsl:value-of select="$pDefaultZoom" />,
              </xsl:if><![CDATA[
              url: ']]><xsl:value-of select="$pProxyURL" /><![CDATA[/]]><xsl:value-of select="$pLayerName" /><![CDATA[?osmresource[]=]]><xsl:value-of select="$resourceName" /><![CDATA[&osmservice=]]><xsl:value-of select="$pServiceName" /><![CDATA[',
              map: oMap,
              border: false,
              ]]><xsl:if test='./options/minchars'><![CDATA[
              minChars: ]]><xsl:value-of select='./options/minchars'/><![CDATA[,
              ]]></xsl:if>
              <xsl:if test='not(./options/minchars)'><![CDATA[
              minChars: 0,
              ]]></xsl:if><![CDATA[        
              ]]><xsl:if test='./options/maxchars'><![CDATA[
              maxChars: ]]><xsl:value-of select='./options/maxchars'/><![CDATA[,
              ]]></xsl:if>
              <xsl:if test='not(./options/maxchars)'><![CDATA[
              maxChars: 10,
              ]]></xsl:if><![CDATA[
              fieldLabel: new Array(]]><xsl:value-of select="$pLabel" /><![CDATA[""),
              queryParam: ']]><xsl:value-of select="$pQueryParam" /><![CDATA[',
              forceFilters: ']]><xsl:value-of select="$pForceFilters" /><![CDATA[',
              filterType: ']]><xsl:value-of select="$pFilterType" /><![CDATA[',
              displayField: ']]><xsl:value-of select="$pDisplayField" /><![CDATA['
            };
	]]>
	</script>    
    <!-- javascript END -->
  </xsl:template>

<!-- 
     Function : drawWidget

     Output the widget depending on the render type
-->
  <!-- Call for each widget name -->
  <xsl:template name="mapfishrecenter:drawWidget">
    <xsl:param name="pWidgetName" />
      objGPWidget<xsl:value-of select="$pWidgetName" />
  </xsl:template>


<!--
    Function: getWidgetDepedencies

    Return the list of dependencies for this widget
-->
  <!-- Call for each widget type-->
  <xsl:template name="mapfishrecenter:getWidgetDepedencies">

  </xsl:template>

</xsl:stylesheet>
