<?xml version="1.0" encoding="utf-8"?>
<!-- 
   Copyright (c) 2010- Mapgears Inc., published under the BSD license.
   See http://geoprisma.org/license for the full text of the license.
 --> 
<xsl:stylesheet version="1.0" xmlns:php="http://php.net/xsl"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:dyn="http://exslt.org/dynamic"
    xmlns:mappanel="http://geoprisma.org/mappanel"
    extension-element-prefixes="dyn">

  <xsl:output method="html" indent="yes" encoding="utf-8" omit-xml-declaration="yes"/>
  <xsl:namespace-alias stylesheet-prefix="php" result-prefix="xsl" />

  <!-- printWidgetSource START  -->
  <xsl:template name="mappanel:printWidgetSource">
    <xsl:variable name="WidgetType">
      <xsl:text>mappanel</xsl:text>
    </xsl:variable>

    <script>
        <xsl:attribute name="src" >
             <xsl:value-of select="$g_widgets_url"/>
             <xsl:text>/mappanel/Map.js</xsl:text>
        </xsl:attribute>    
    </script>

    <script>
        <xsl:attribute name="src" >
             <xsl:value-of select="$g_widgets_url"/>
             <xsl:text>/mappanel/OpenLayers/Protocol/HTTP.js</xsl:text>
        </xsl:attribute>    
    </script>

    <script type="text/javascript">
      var objGPWidget<xsl:value-of select="$WidgetType" />Container = [];
      var objGPWidget<xsl:value-of select="$WidgetType" />DefaultOptions = {
        border: false,
        region: 'center'
      };
    </script>

   </xsl:template>
  <!-- printWidgetSource END  -->

  <!-- printWidgetExecution START  -->
  <xsl:template name="mappanel:printWidgetExecution">
    <xsl:param name="pWidgetName" />
    <xsl:param name="pMapName" />

    <xsl:variable name="WidgetType">
      <xsl:text>mappanel</xsl:text>
    </xsl:variable>

    <script type="text/javascript">
      var objGPWidget<xsl:value-of select="$pWidgetName" />;

      //Ext.onReady(function() {
          <!-- Map -->
          <xsl:call-template name="getOpenLayersOptions" >
            <xsl:with-param name="pWidgetName">
              <xsl:value-of select="$pWidgetName" />
            </xsl:with-param>
          </xsl:call-template>
          objOptions.controls = [
              new OpenLayers.Control.PanPanel(),
              new OpenLayers.Control.ZoomPanel(),
            <xsl:choose>
              <xsl:when test="./options/zoomBoxNavigation = 'true'">new OpenLayers.Control.Navigation()</xsl:when>
              <xsl:otherwise>new OpenLayers.Control.Navigation({zoomBoxEnabled: false})</xsl:otherwise>
            </xsl:choose>
          ];
          
          objGPWidget<xsl:value-of select="$pWidgetName" /> =
              new OpenLayers.Map( objOptions );

          <!-- MapPanel -->
          var objMapPanelOptions = Ext.applyIf(
            <xsl:for-each select="./options">
              <xsl:call-template name="getObjectFromNode">
                <xsl:with-param name="pNode" select="." />
              </xsl:call-template>
            </xsl:for-each>,
            objGPWidget<xsl:value-of select="$WidgetType" />DefaultOptions
          );

          <!-- center and extent options management -->
          <xsl:if test="./options/extentString">
            objMapPanelOptions["extent"] = new OpenLayers.Bounds(<xsl:value-of select="./options/extentString" />);
            delete objMapPanelOptions["extentString"];
          </xsl:if>
          <xsl:if test="./options/centerString">
            objMapPanelOptions["center"] = new OpenLayers.LonLat(<xsl:value-of select="./options/centerString" />);
            delete objMapPanelOptions["centerString"];
          </xsl:if>

          <!-- i18n support for 'title' option -->
          <xsl:if test="./options/title != ''">
            objMapPanelOptions.title = <xsl:for-each select="./options/title"><xsl:call-template name="getText" /></xsl:for-each>;
          </xsl:if>

          objMapPanelOptions['map'] = objGPWidget<xsl:value-of select="$pWidgetName" />;
          <!-- GeoExtToolbarSingleton adding as MapPanel 'tbar' property -->
          if (typeof org.GeoPrisma.GeoExtToolbarSingleton !== "undefined") {
              objMapPanelOptions.tbar = org.GeoPrisma.GeoExtToolbarSingleton;
          }

          objGPWidgetMapPanel<xsl:value-of select="$pWidgetName" /> = 
              new GeoExt.MapPanel(objMapPanelOptions);

          <!-- setMapPanel in API widget -->
          GeoPrisma.setMapPanel(
              objGPWidgetMapPanel<xsl:value-of select="$pWidgetName" />
          );

          <!-- Push to container in order to draw later -->
          objGPWidget<xsl:value-of select="$WidgetType" />Container.push(
              objGPWidgetMapPanel<xsl:value-of select="$pWidgetName" />
          );
      //});
    </script>

  </xsl:template>
  <!-- printWidgetExecution END  -->

  <!-- drawWidgets START  -->
  <xsl:template name="mappanel:drawWidgets">
    <xsl:param name="pContainer" />

    <xsl:variable name="WidgetType">
      <xsl:text>mappanel</xsl:text>
    </xsl:variable>
	if (typeof(objGPWidget<xsl:value-of select="$WidgetType" />Container) != 'undefined' &amp;&amp; typeof(<xsl:value-of select="$pContainer" />) != 'undefined')
    {
        for (var i=0, len=objGPWidget<xsl:value-of select="$WidgetType" />Container.length; i&lt;len; i++) {
            if(<xsl:value-of select="$pContainer" />['ctype']){
                <xsl:value-of select="$pContainer" />.add(
                    objGPWidget<xsl:value-of select="$WidgetType" />Container[i]
                );
            } else {
                <xsl:value-of select="$pContainer" />.items.push(
                    objGPWidget<xsl:value-of select="$WidgetType" />Container[i]
                )
            }
        }
    }
  </xsl:template>
  <!-- drawWidgets END  -->

  <!--
     API Method: printPageTitle
     This method can be used as a replacement of the 'title' tag in your main
     template page.  Its purpose it to get the mappanel widget 'pageTitle'
     option available and print a 'title' tag out of it.  If none was present,
     the value is 'GeoPrisma'.
    -->
  <xsl:template name="mappanel:printPageTitle">
    <xsl:variable name="widgetType">
      <xsl:text>mappanel</xsl:text>
    </xsl:variable>

    <xsl:variable name="pageTitle">
      <xsl:value-of select="/geoprisma/widgets/widget[./type = $widgetType]/options[./pageTitle != '']/pageTitle" />
    </xsl:variable>

    <title>
      <xsl:choose>
        <xsl:when test="$pageTitle != ''">
          <xsl:for-each select="/geoprisma/widgets/widget[./type = $widgetType]/options[./pageTitle != '']/pageTitle">
            <xsl:call-template name="getText">
              <xsl:with-param name="pType">
                <xsl:text>HTML</xsl:text>
              </xsl:with-param>
            </xsl:call-template>
          </xsl:for-each>
        </xsl:when>
        <xsl:otherwise>GeoPrisma</xsl:otherwise>
      </xsl:choose>
    </title>
  </xsl:template>

</xsl:stylesheet>
