/**
 * Copyright (c) 2009-2011 Mapgears Inc.
 * 
 * Published under the BSD license.
 * See http://geoprisma.org/license for the full text of the license. 
 * 
 * The code of this widget was borrowed from the GXP library at the following
 * version, which was published under the BSD license at that time :
 * 
 * Commit : a56125b1ff578110bc39f4ac76312eab0fc46073
 * Author : Bart Van den Eijnden
 * Date : Wed Mar 23 17:11:55 2011 +0100
 */

/** api: (define)
 *  module = org.GeoPrisma
 *  class = MeasureTool
 */

/** api: (extends)
 *  SplitButton.js
 */
Ext.namespace("org.GeoPrisma");

/** api: constructor
 *  .. class:: MeasureTool(config)
 *
 *    Provides two actions for measuring length and area in metric and english
 *    units.
 */
org.GeoPrisma.MeasureTool = Ext.extend(Ext.SplitButton, {

    /* i18n */

    /** api: config[tooltip]
     *  ``String``
     *  Text for button tooltip (i18n).
     */
    tooltip: "Measure Tools",

    /** api: config[lengthMenuText]
     *  ``String``
     *  Text for measure length menu item (i18n).
     */
    lengthMenuText: "Length",

    /** api: config[areaMenuText]
     *  ``String``
     *  Text for measure area menu item (i18n).
     */
    areaMenuText: "Area and Perimeter",

    /** api: config[lengthTooltip]
     *  ``String``
     *  Text for measure length action tooltip (i18n).
     */
    lengthTooltip: "Measure length",

    /** api: config[areaTooltip]
     *  ``String``
     *  Text for measure area action tooltip (i18n).
     */
    areaTooltip: "Measure area",

    /** api: config[perimeterTooltip]
     *  ``String``
     *  Text for measure perimeter action tooltip (i18n).
     */
    perimeterTooltip: "Measure perimeter",
    
    /** api: config[lastSegmentTooltip]
     *  ``String``
     *  Text for measure the last segement action tooltip (i18n).
     */
    lastSegmentTooltip: "Last segment Measure",

    /** api: config[measureTooltip]
     *  ``String``
     *  Text for measure action tooltip (i18n).
     */
    measureTooltip: "Measure",
    
    /**
     * api: config[unitsText]
     * ``Object``
     * texts for units
     */
    unitsText: {dd:'dd',mi:'mi', ft:'ft', 'in':"in", km:'km', m:'m'},

    /* API Properties (Mandatory) */ 

    /** api: config[mapPanel]
     *  :class:`GeoExt.MapPanel`
     *  Used to access the :class:`OpenLayers.Map` and calculate the tooltip
     *  position.
     */
    mapPanel: null,

    /** api: config[measureWindow]
     *  :class:`Ext.Window`
     *  Used contain the informative window.
     */
    measureWindow: null,

    /* API Properties (Optional, with default value) */

    /** api: property[geodesic]
     *  ``Boolean``
     *  Sets the Measure control 'geodesic' property.
     */
    geodesic: false,

    /** api: property[persist]
     *  ``Boolean``
     *  Sets the Measure control 'persist' property.
     */
    persist: true,

    /** api: property[immediate]
     *  ``Boolean``
     *  Sets the Measure control 'immediate' property.
     */
    immediate: true,

    /** api: property[hectare]
     *  ``Boolean``
     *  Whether to add the hectare units or not.
     */
    hectare: false,

    /** api: property[iconCls]
     *  ``String``
     *  Default value of the 'iconCls' property.
     */
    iconCls: "gp-measuretool-icon-measure-length",

    /** api: property[enableToggle]
     *  ``Boolean``
     *  Default value of the 'enableToggle' property.
     */
    enableToggle: true,

    /** api: property[allowDepress]
     *  ``Boolean``
     *  Default value of the 'allowDepress' property.
     */
    allowDepress: false,
    
    /** api: property[showLastSegmentMeasure]
     *  ``Boolean``
     *  Default value of the 'showLastSegmentMeasure' property.
     */
    showLastSegmentMeasure: false,

    /** api: property[tooltipAtCursor]
     *  ``Boolean``
     *  Sets if the tooltip box follow the cursor.
     */
    tooltipAtCursor: true,

    /* Private Properties */

    /** api: property[activeIndex]
     *  ``Integer``
     *  Is set to the current active control index.
     */
    activeIndex: null,

    /** private: method[constructor]
     */
    constructor: function(config) {
        Ext.apply(this, config);
        this.addActions();
        org.GeoPrisma.MeasureTool.superclass.constructor.apply(this, arguments);
        this.activeIndex = 0;
        this.mapPanel = this.mapPanel || GeoExt.MapPanel.guess();
    },

    /** private: method[initMyContent]
     *  Create and add the items of this widget : a Ext.ux.form.FileUploadField.
     */
    addActions: function() {
        Ext.apply(this, {
            listeners: {
                toggle: function(button, pressed) {
                    // toggleGroup should handle this
                    if(!pressed) {
                        button.menu.items.each(function(i) {
                            i.setChecked(false);
                        });
                    } else {
                        button.menu.get(this.activeIndex).setChecked(true);
                    }
                },
                render: function(button) {
                    // toggleGroup should handle this
                    Ext.ButtonToggleMgr.register(button);
                }
            },
            menu: new Ext.menu.Menu({
                items: [
                    new Ext.menu.CheckItem(
                        new GeoExt.Action({
                            text: this.lengthMenuText,
                            iconCls: "gp-measuretool-icon-measure-length",
                            toggleGroup: this.toggleGroup,
                            group: this.toggleGroup,
                            listeners: {
                                checkchange: function(item, checked) {
                                    this.activeIndex = 0;
                                    this.toggle(checked);
                                    this.measureWindow.init(this);
                                    if (checked) {
                                        this.setIconClass(item.iconCls);
                                    }
                                },
                                scope: this
                            },
                            map: this.mapPanel.map,
                            control: this.createMeasureControl(
                                OpenLayers.Handler.Path, this.lengthTooltip
                            )
                        })
                    ),
                    new Ext.menu.CheckItem(
                        new GeoExt.Action({
                            text: this.areaMenuText,
                            iconCls: "gp-measuretool-icon-measure-area",
                            toggleGroup: this.toggleGroup,
                            group: this.toggleGroup,
                            allowDepress: false,
                            listeners: {
                                checkchange: function(item, checked) {
                                    this.activeIndex = 1;
                                    this.toggle(checked);
                                    this.measureWindow.init(this);
                                    if (checked) {
                                        this.setIconClass(item.iconCls);
                                    }
                                },
                                scope: this
                            },
                            map: this.mapPanel.map,
                            control: this.createMeasureControl(
                                OpenLayers.Handler.Polygon, this.areaTooltip
                            )
                        })
                    )
                ]
            })
        });
    },

    /** private: method[createMeasureControl]
     * :param: handlerType: the :class:`OpenLayers.Handler` for the measurement
     *     operation
     * :param: title: the string label to display alongside results
     *
     * Convenience method for creating a :class:`OpenLayers.Control.Measure`
     * control
     */
    createMeasureControl: function(handlerType, title) {

        var styleMap = this.getNewStyleMapObject();
        var cleanup = function() {
            if (this.measureWindow) {
                this.measureWindow.hide();
            }  
        };
    
        var makeString = function(metricData, This) {
            var content = [];

            // Title of the first measure if it's a 'length' or 'area'
            content.push("<b>" + title + "</b><br />");
            
            // manage first measure, whether it's a 'length' or 'area'
            var metric = metricData.measure;
            var metricUnit = metricData.units;

            measureControl.displaySystem = "english";

            var englishData = metricData.geometry.CLASS_NAME.indexOf("LineString") > -1 ?
            measureControl.getBestLength(metricData.geometry) :
            measureControl.getBestArea(metricData.geometry);

            var english = englishData[0];
            var englishUnit = englishData[1];

            measureControl.displaySystem = "metric";
            var dim = metricData.order == 2 ?
            '<sup>2</sup>' :
            '';

            content.push(metric.toFixed(2) + " " + This.unitsText[metricUnit] + dim + "<br>" +
                english.toFixed(2) + " " + This.unitsText[englishUnit] + dim);

            // hectare, only calculate if area measure (order == 2)
            if (This.hectare && metricData.order == 2) {
                var toHectareRatio = Math.pow(
                    OpenLayers.INCHES_PER_UNIT[metricUnit] /
                    OpenLayers.INCHES_PER_UNIT["Hectometer"], 2);
                var hectare = metric * toHectareRatio;
                var hectareUnit = "ha";
                content.push("<br>" + hectare.toFixed(2) + " " + hectareUnit);
            }
            
             //Show if last segment mesure is asked and if there is a segment and if it's a line measure
            if (This.showLastSegmentMeasure && metricData.geometry.components.length > 1 && metricData.geometry.CLASS_NAME.indexOf("LineString") > -1) {
                
                
                //Keep only the last segment in a clone
                lastSegment = metricData.geometry.clone();
                while(lastSegment.components.length > 2){
                    lastSegment.removeComponent(lastSegment.components[0])
                }
                
                //Calculate measures with the clone
                metric = measureControl.getBestLength(lastSegment);
                metricUnit = metric[1];
                metric = metric[0];

                measureControl.displaySystem = "english";
                englishData = measureControl.getBestLength(lastSegment);
                english = englishData[0];
                englishUnit = englishData[1];

                measureControl.displaySystem = "metric";

                content.push("<br /><br />");
                content.push("<b>" + This.lastSegmentTooltip + "</b><br />");
                content.push(metric.toFixed(2) + " " + This.unitsText[metricUnit] + 
                    "<br />" + english.toFixed(2) + " " + This.unitsText[englishUnit]);
            }
            
            // manage a second custom measure if 'area' : perimeter
            if (metricData.geometry.CLASS_NAME.indexOf("LineString") == -1) {
                metric = measureControl.getBestLength(metricData.geometry);
                metric = metric[0];
                metricUnit = metricData.units;

                measureControl.displaySystem = "english";
                englishData = measureControl.getBestLength(metricData.geometry);
                english = englishData[0];
                englishUnit = englishData[1];

                measureControl.displaySystem = "metric";

                content.push("<br /><br />");
                content.push("<b>" + This.perimeterTooltip + "</b><br />");
                content.push(metric.toFixed(2) + " " + This.unitsText[metricUnit] + 
                    "<br />" + english.toFixed(2) + " " + This.unitsText[englishUnit]);
            }

            return content.join("");
        };

        this.measureWindow = new Ext.Window({
            target: Ext.getBody(),
            html: '',
            autoHide: false,
            closable: true, 
            closeAction: 'hide',
            draggable: true,
            border: false,
            resizable: false,
            constrain: true,
            width: 200, 
            showDelay: 1,
            monitorResize: true,
            listeners: {
                hide: function() {
                    measureControl.cancel();
                    cleanup();
                }
            },
            init: function(measureTool) {
                // Display tooltip at right top corner
                if(measureTool.tooltipAtCursor == false) {
                    var p0 = measureTool.mapPanel.getPosition();
                    var s0 = measureTool.mapPanel.getSize();
                    
                    this.setPosition([p0[0]+s0.width-this.width-5, p0[1]+36]);
                }
            },
            renderTo: this.mapPanel.getEl()
        });
        
        var measureControl = new OpenLayers.Control.Measure(handlerType, {
            geodesic: this.geodesic,
            persist: this.persist,
            immediate: this.immediate,
            handlerOptions: {layerOptions: {styleMap: styleMap}},
            eventListeners: {
                measurepartial: function(event) {
                    if(event.measure > 0) {
                        this.measureWindow.show();
                        
                        // Update window text
                        this.measureWindow.update(makeString(event, this));
                        
                        // Display tooltip at cursor
                        if(this.tooltipAtCursor == true) {
                            var px = measureControl.handler.lastUp;
                            var p0 = this.mapPanel.getPosition();
                        
                            this.measureWindow.setPosition([p0[0]+px.x-50, p0[1]+px.y+40]);
                        }
                    }
                    else {
                        this.measureWindow.hide();
                    }
                },
                measure: function(event) {
                    // Update window text
                    this.measureWindow.update(makeString(event, this));
                    
                    // Display tooltip at cursor
                    if (this.tooltipAtCursor == true) {
                        Ext.getBody().on("mousemove", function(e) {
                            var px = e.getXY()
                            var p0 = this.mapPanel.getPosition();
                            this.measureWindow.setPosition([px[0], px[1]]);
                        }, this, {single: true});
                    }
                },
                deactivate: cleanup,
                scope: this
            }
        });

        return measureControl;
    },

    /** private: method[getNewStyleMapObject]
     * :return: :class:`OpenLayers.StyleMap` Used by the measure control layer.
     */
    getNewStyleMapObject: function() {
        return new OpenLayers.StyleMap({
            "default": new OpenLayers.Style(null, {
                rules: [new OpenLayers.Rule({
                    symbolizer: {
                        "Point": {
                            pointRadius: 4,
                            graphicName: "square",
                            fillColor: "white",
                            fillOpacity: 1,
                            strokeWidth: 1,
                            strokeOpacity: 1,
                            strokeColor: "#333333"
                        },
                        "Line": {
                            strokeWidth: 3,
                            strokeOpacity: 1,
                            strokeColor: "#666666",
                            strokeDashstyle: "dash"
                        },
                        "Polygon": {
                            strokeWidth: 2,
                            strokeOpacity: 1,
                            strokeColor: "#666666",
                            fillColor: "white",
                            fillOpacity: 0.3
                        }
                    }
                })]
            })
        });
    }
});