/**
 * @requires OpenLayers/Control.js
 * @requires mapfish/Protocol/MapFish.js
 */

/**
 * Class: OpenLayers.Control.PDFPrint
 *
 *
 * Inherits From:
 *  - <OpenLayers.Control>
 */
OpenLayers.Control.PDFPrint = OpenLayers.Class(OpenLayers.Control, {
	/*
     * i18n text for language pack
     */
	i18n_tab_template_title: 'Template',
	i18n_tab_form_title: 'Form',
											   
    /**
     * Property: formPanel
     * {Ext.Panel} The form panel saved so it can be recalled
     */
    formPanel: null,
												
    /**
     * Property: map
     * {OpenLayers.Map} The map object to be called
     */
    map: null,
	
    /**
     * Property: oldwindow
     * {Ext.Window} The window saved so it can be recalled
     */
    oldwindow: null,
	
    /**
     * Property: selectedTemplate
     * {Object} The currently selected template
     */
    selectedTemplate: 0,
	
    /**
     * Property: templates
     * {Object array} The array containing all templates
     */
    templates: null,
	
    /**
     * Property: type
     * {String} The type of <OpenLayers.Control> -- When added to a 
     *     <Control.Panel>, 'type' is used by the panel to determine how to 
     *     handle our events.
     */
    type: OpenLayers.Control.TYPE_BUTTON,
    
    isArray: function(a) {
    return Object.prototype.toString.apply(a) === '[object Array]';
    },
											   
	/*
     * Method: generatePDF
     * Generate the PDF with the specified template
     */
	generatePDF: function(button, e) {
		var template = this.selectedTemplate;
		var spec = {
						layout: template.model,
						title: template.model,
						srs: this.map.projection.projCode,
						units: 'm',
						layers: [],
						pages: [
							{
								center: [this.map.center.lon, this.map.center.lat],
								scale: (this.map.scales)?this.map.scales[this.map.zoom]:Math.round(OpenLayers.Util.getScaleFromResolution(this.map.resolution, "m")),
								resolution: (this.map.resolutions)?this.map.resolutions[this.map.zoom]:Math.round(OpenLayers.Util.getResolutionFromScale(this.map.scales[this.map.zoom], "m")),
								dpi: template.dpi ? template.dpi : 96
							}
						]
					};
					
					
		// For each layers
		for (var i=0; i<this.map.layers.length; i++)
		{
			if (this.map.layers[i].getVisibility())
			{
				switch(this.map.layers[i].serviceType)
				{
					case "wms":
						spec.layers.push({
											type: "wms",
											format: this.map.layers[i].params.FORMAT,
											layers: this.map.layers[i].params.LAYERS,
											baseURL: this.isArray(this.map.layers[i].url) ? this.map.layers[i].url[0] : this.map.layers[i].url,
											opacity: (typeof this.map.layers[i].opacity == "number")?this.map.layers[i].opacity:1,
											customParams: {TRANSPARENT: this.map.layers[i].params.TRANSPARENT}
										 });
						break;
					case "tilecache":
						spec.layers.push({
											type: "wms",
											format: this.map.layers[i].params.FORMAT,
											layers: (typeof this.map.layers[i].params.LAYERS == "object")?this.map.layers[i].params.LAYERS:[this.map.layers[i].params.LAYERS],
											baseURL: this.isArray(this.map.layers[i].url) ? this.map.layers[i].url[0] : this.map.layers[i].url,
											opacity: (typeof this.map.layers[i].opacity == "number")?this.map.layers[i].opacity:1,
											customParams: {TRANSPARENT: this.map.layers[i].params.TRANSPARENT}
										 });
						break;
				}
			}
		}
		
		// For each fields
		for (i=0; i<this.formPanel.items.items.length; i++)
		{
			spec.pages[0][this.formPanel.items.items[i].getName()] = this.formPanel.items.items[i].getValue();
		}
		
		// Additionnal infos
		var currentDate = new Date();
		spec.pages[0].year = currentDate.getFullYear();
		spec.pages[0].month = currentDate.getMonth()+1;
		spec.pages[0].day = currentDate.getDate();
		
		// For each page variables
		if (template.nbPages > 0)
		{
			for (i=0; i<template.nbPages; i++)
			{
				// Except the first page, copy all infos
				if (i>0)
				{
					spec.pages[i] = new Object();
					for (key in spec.pages[i-1])
					{
						spec.pages[i][key] = spec.pages[i-1][key];
					}
				}
				
				// For each page fields
				for (key in template.pages)
				{
					spec.pages[i][key] = template.pages[key][i];
				}
			}
		}
		// Sends the specs to the print applet
		var form = document.createElement("form");
		form.action = "print_proxy.php";
		form.method = "POST";
		form.target = "PDFPrint";
		var hiddenForm = document.createElement("input");
		hiddenForm.type = "hidden";
		hiddenForm.name = "spec";
		hiddenForm.value = escape("{"+this.stringify(spec)+"}");
		document.body.insertBefore(form, null);
		form.insertBefore(hiddenForm, null);
		window.open("","PDFPrint","location=0,status=0,scrollbars=0,resizable=1");
		form.submit();
		document.body.removeChild(form);
		
	},
	
	/*
     * Method: initializeForm
     * Initialize the form with the corresponding fields
     */
	initializeForm: function(template, node) {	
		var template = (template != null)? template:this.templates[node];
		
		this.formPanel.removeAll();
		this.selectedTemplate = template;
		
		// If there's only one field
		if (!template.fields.length)
		{
			template.fields = Array(template.fields.field);
		}
		
		for (var i=0; i<template.fields.length; i++)
		{
			var field = template.fields[i];
			switch (field.type)
			{
				case "hidden":
					this.formPanel.add(new Ext.form.Hidden({fieldLabel: field.label, name: field.name, value: field.value}));
					break;
				case "text":
					this.formPanel.add(new Ext.form.TextField({fieldLabel: field.label, name: field.name, value: field.value}));
					break;
				case "textarea":
					this.formPanel.add(new Ext.form.TextArea({fieldLabel: field.label, name: field.name, value: field.value, autoWidth: true, style:{color: '#000000'}}));
					break;
			}
		}
		
		if (!this.formPanel.buttons)
		{
			this.formPanel.addButton({text: 'PDF'}, this.generatePDF ,this);
		}
		this.formPanel.doLayout();
	},
	
	/*
     * Method: stringify
     * Transform JSON in string
     */
	stringify: function(json) {
		var stringified = "";
		for (var key in json)
		{
			if (json["remove"])
			{
				switch(typeof json[key])
				{
					case "boolean":
						stringified += json[key] + ',';
						break;
					case "number":
						stringified += json[key] + ',';
						break;
					case "string":
						stringified += '"' + json[key].replace(/\"/g,'\\"') + '",';
						break;
					case "object":
						if (json[key].remove)
						{
							stringified += "[" + this.stringify(json[key]) + "],";
						}else{
							stringified += "{" + this.stringify(json[key]) + "},";
						}
						break;
				}
			}else{
				switch(typeof json[key])
				{
					case "boolean":
						stringified += key + ': ' + json[key] + ',';
						break;
					case "number":
						stringified += key + ': ' + json[key] + ',';
						break;
					case "string":
						stringified += key + ': "' + json[key].replace(/\"/g,'\\"') + '",';
						break;
					case "object":
						if (json[key].remove)
						{
							stringified += key + ": [" + this.stringify(json[key]) + "],";
						}else{
							
							stringified += key + ": {" + this.stringify(json[key]) + "},";
						}
						break;
				}
			}
		}
		
		// Remove last comma if not empty
		if (stringified.length>0)
		{
			stringified = stringified.substr(0,stringified.length-1);
		}
		return stringified;
	},
	
	/*
     * Method: trigger
     * Open the window.
     */
	trigger: function(template) {
		if (template == null){template = null;} // If a template is not pushed
		
		if (!this.oldwindow)
		{
			this.store = new Ext.data.JsonStore({
				data: this.templates,
				fields: ['name', 'thumb']
			});
			
			this.tpl = new Ext.XTemplate(
				'<tpl for=".">',
					'<div class="thumb-wrap" id="{name}">',
					'<div class="thumb"><img src="{thumb}" title="{name}"></div>',
					'<span class="x-editable">{name}</span></div>',
				'</tpl>',
				'<div class="x-clear"></div>'
			);

			templateChooser = new Ext.DataView({
				store: this.store,
				tpl: this.tpl,
				autoWidth: true,
				autoHeight: true,
				maxHeight: 400,
				multiSelect: false,
				singleSelect: true,
				overClass:'x-view-over',
				itemSelector:'div.thumb-wrap',
				emptyText: '',
				ownerWidget: this,
				
				listeners: {
					click: {
						fn: function(dv,nodes){
							this.ownerWidget.initializeForm(null, nodes);
						}
					}
				}
			});
			
			templatePanel = new Ext.Panel({
				id:'images-view',
				frame:true,
				autoWidth: true,
				autoHeight: true,
				maxHeight: 400,
				title: this.i18n_tab_template_title,
				autoScroll: true,
				items: [templateChooser]
			});
			
			this.formPanel = new Ext.FormPanel({
				id:'formPDF',
				frame:true,
				autoWidth: true,
				autoHeight: true,
				minHeight: 200,
				title: this.i18n_tab_form_title,
				autoScroll: true,
				items: []
			});
			this.initializeForm(template, 0);

			
			this.oldwindow = new Ext.Window({
				id: 'PDFPrint',
				title: 'PDF',
				closeAction: 'hide',
				resizable: true,
				items: [templatePanel, this.formPanel],
				width: 450
			});
			templateChooser.select(0);
		}
		if (template)
		{
			this.initializeForm(template, 0);
			this.generatePDF(null, null);
		}else{
			this.oldwindow.show();
		}
	},
	
    CLASS_NAME: "OpenLayers.Control.PDFPrint"					 
});
