<?php
/* 
   Copyright (c) 2011- Mapgears Inc., published under the BSD license.
   See http://geoprisma.org/license for the full text of the license.
*/

class QueryOnClickWizard extends org_geoprisma_widget_WidgetBase
{
    private static $m_objWidgetQueryOnClick = null;

    private static $m_objWidgetVectorLayer = null;

    private static $m_objWidgetResultVectorLayer = null;

    static public function getText($pstrMsg, $pstrType = null)
    {
        return parent::getText($pstrMsg, self::getType(), $pstrType);
    }

    static public function getAction()
    {
        return 'read';
    }    
     
    static public function getType()
    {
        return 'queryonclickwizard';
    }

    static public function getServiceType()
    {
        return 'wms';
    }

    static public function needLayer() 
    {
        return true;
    }

    static public function getMandatoryResourceOptionList()
    {
        return array("queryable");
    }

    static public function getOptionalResourceOptionList()
    {
        return null;
    }

    static public function mustClone()
    {
        return false;
    }

    static public function linkResource(
        $pobjResource, $pobjWidget, $pobjConfig
    ) {
        if ($pobjResource == null || $pobjWidget == null) 
        {
            return;
        }

        // create the widgets if it's the first time this method is called
        if (!self::getQueryOnClickWidget()) 
        {
            // create the widgets
            self::setVectorLayerWidget($pobjWidget);
            self::setResultVectorLayerWidget(self::getVectorLayerWidget());
            self::setQueryOnClickWidget(
                $pobjWidget, self::getResultVectorLayerWidget()
            );

            // add the widgets to the config
            $objArrayWidget = $pobjConfig->getWidgets();
            $objArrayWidget[self::getVectorLayerWidget()->getName()] =
                self::getVectorLayerWidget();
            $objArrayWidget[self::getResultVectorLayerWidget()->getName()] =
                self::getResultVectorLayerWidget();
            $objArrayWidget[self::getQueryOnClickWidget()->getName()] =
                self::getQueryOnClickWidget();

            // find the 'geoexttoolbar' widget and append the queryonclick to
            // 'widgets' option in order to have the button added to the toolbar
            foreach ($objArrayWidget as $objWidget)
            {
                if ($objWidget->getType() == "geoexttoolbar") 
                {
                    // try to add inside the 'widgets' option
                    try
                    {
                        $objOption = $objWidget->getOption('widgets');
                        $objOption->setValue(
                            array_merge(
                                array(
                                    self::getQueryOnClickWidget()->getName(),
                                    "__separator__"
                                ),
                                $objOption->getValue()
                            )
                        );
                    }
                    // 'widgets' option wasn't found, try 'widget' instead and
                    // transform it into a 'widgets' option since we'll have
                    // more than one widgets from now on
                    catch (Exception $objError) {
                        try
                        {
                            $objOption = $objWidget->getOption('widget');
                            $objOption->setValue(array(
                                $objOption->getValue(),
                                "__separator__",
                                self::getQueryOnClickWidget()->getName()
                            ));
                            $objOption->setName('widgets');
                        }
                        catch (Exception $objError) {}
                    }
                    break;
                }
            }
        }

        // link the resource and created widgets together
        $pobjResource->addWidget(self::getQueryOnClickWidget());
        $pobjResource->addWidget(self::getResultVectorLayerWidget());
    }

    /**
    * Create a new instance of a org_geoprisma_widget_GenericWidget with the
    *     "vectorlayer" type and set it to this widget properties.
    *
    * If this widget has a "layerName" option set, it's used to set the
    *     VectorLayer "title" widget option.
    * 
    * @param org_geoprisma_widget_GenericWidget $pobjWidgetWizard This widget
    * 
    * @return void
    */
    static public function setVectorLayerWidget($pobjWidgetWizard)
    {
        $iVectorLayer = "W_MyVectorLayer";
        $objArrayOptions = array("displayInLayerSwitcher" => "false");

        try
        {
            $objArrayOptions["title"]
                = $pobjWidgetWizard->getOption("layerName")->getValue();
        }
        catch (Exception $objError) {}

        self::$m_objWidgetVectorLayer = new org_geoprisma_widget_GenericWidget(
            "vectorlayer",
            array("name" => $iVectorLayer, "options" => $objArrayOptions)
        );
    }

    /**
    * Create a new instance of a org_geoprisma_widget_GenericWidget with the
    *     "resultvectorlayer" type and set it to this widget properties.
    * 
    * @param org_geoprisma_widget_GenericWidget $pobjWidgetVectorLayer
    *     The VectorLayer to use when creating the ResultVectorLayer widget.
    * 
    * @return void
    */
    static public function setResultVectorLayerWidget($pobjWidgetVectorLayer)
    {
        $iResultVectorLayer = "W_MyResultVectorLayer";
        self::$m_objWidgetResultVectorLayer
            = new org_geoprisma_widget_GenericWidget(
                "resultvectorlayer",
                array("name" => $iResultVectorLayer, "options" => array(
                    "vectorlayer" => $pobjWidgetVectorLayer->getName(),
                    "singleMode" => "true"
                ))
            );
    }

    /**
    * Create a new instance of a org_geoprisma_widget_GenericWidget with the
    *     "queryonclick" type and set it to this widget properties.
    *
    * This widget "iconCls" option is added to the QueryOnClick widget options
    *     if set.
    * 
    * @param org_geoprisma_widget_GenericWidget $pobjWidgetWizard This widget
    * @param org_geoprisma_widget_GenericWidget $pobjWidgetResultVectorLayer
    *     The ResultVectorLayer widget required to add to the created
    *     QueryOnClick widget.
    * 
    * @return void
    */
    static public function setQueryOnClickWidget(
        $pobjWidgetWizard, $pobjWidgetResultVectorLayer
    ) {
        $iQueryOnClickID = "W_MyQueryOnClick";
        $objArrayOptions = array(
            "dropDownList" => "false",
            "multipleKey" => "shiftKey",
            "noMarker" => "true",
            "results" => array($pobjWidgetResultVectorLayer->getName()),
            "toggle" => "true",
            "displayClass" => "olControlQueryOnClickWizard"
        );

        try
        {
            $objArrayOptions["iconCls"] 
                = $pobjWidgetWizard->getOption("iconCls")->getValue();
        }
        catch (Exception $objError)
        {
            $objArrayOptions["iconCls"] = "queryonclickwizard";
        }

        self::$m_objWidgetQueryOnClick = new org_geoprisma_widget_GenericWidget(
            "queryonclick",
            array("name" => $iQueryOnClickID, "options" => $objArrayOptions)
        );
    }

    /**
    * Returns the VectorLayer widget.
    * 
    * @return org_geoprisma_widget_GenericWidget 
    */
    static public function getVectorLayerWidget()
    {
        return self::$m_objWidgetVectorLayer;
    }

    /**
    * Returns the ResultVectorLayer widget.
    * 
    * @return org_geoprisma_widget_GenericWidget 
    */
    static public function getResultVectorLayerWidget()
    {
        return self::$m_objWidgetResultVectorLayer;
    }

    /**
    * Returns the QuetyOnClick widget.
    * 
    * @return org_geoprisma_widget_GenericWidget 
    */
    static public function getQueryOnClickWidget()
    {
        return self::$m_objWidgetQueryOnClick;
    }
}
  
?>
