<?xml version="1.0" encoding="utf-8"?>
<!-- 
   Copyright (c) 2010- Nippour Geomatik, published under the BSD license.
   See http://geoprisma.org/license for the full text of the license.
 -->
<xsl:stylesheet version="1.0" xmlns:php="http://php.net/xsl"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:dyn="http://exslt.org/dynamic"
    xmlns:wmslayeradder="http://geoprisma.org/wmslayeradder"
    extension-element-prefixes="dyn">

  <xsl:output method="html" indent="yes" encoding="utf-8" omit-xml-declaration="yes"/>
  <xsl:namespace-alias stylesheet-prefix="php" result-prefix="xsl" />

<!-- 
     Function : printWidgetSource

     Print the source code for this widget
-->
  <!-- Call for each widget type-->
  <xsl:template name="wmslayeradder:printWidgetSource">
    <xsl:variable name="WidgetType">
      <xsl:text>wmslayeradder</xsl:text>
    </xsl:variable>

  	<link rel="stylesheet" type="text/css" >
  		<xsl:attribute name="href" >
            <xsl:value-of select="$g_widgets_url"/>
            <xsl:text>/wmslayeradder/WMSLayerAdder.css</xsl:text>
        </xsl:attribute>
    </link>   
    
    <script>
        <xsl:attribute name="src" >
             <xsl:value-of select="$g_widgets_url"/>
             <xsl:text>/wmslayeradder/WMSLayerAdder.js</xsl:text>
        </xsl:attribute>    
    </script>
    <script>
        <xsl:attribute name="src" >
             <xsl:value-of select="$g_widgets_url"/>
             <xsl:text>/wmslayeradder/ItemSelector.js</xsl:text>
        </xsl:attribute>    
    </script>
   </xsl:template>

<!-- 
     Function : printWidgetExecution

     Print the execution code for this widget
-->
  <!-- Call for each widget name-->
  <xsl:template name="wmslayeradder:printWidgetExecution">
    <xsl:param name="pWidgetName" />
    <xsl:param name="pMapName" />

    <!-- get draw mode  -->
    <xsl:variable name="pDrawMode">
      <xsl:value-of select="$drawmode" />
    </xsl:variable>

    <!-- javascript START -->
    <script type="text/javascript">
		oMap = objGPWidget<xsl:value-of select="$pMapName" />;
		
                var mapfishlayertree;
		<xsl:for-each select="//widget[./type = 'mapfishlayertree']">
                  mapfishlayertree = objGPWidget<xsl:value-of select="./name"/>;
	  	</xsl:for-each>
		
		var dispolayers = {};
		<xsl:if test="./options/layers">
		dispolayers = 
			<xsl:for-each select="./options/layers">
				<xsl:call-template name="getObjectFromNode">
				  <xsl:with-param name="pNode" select="./layer" />
				</xsl:call-template>
			</xsl:for-each>;
		</xsl:if>
		
		// If there is only one layer
		if (dispolayers.layer){
			dispolayers = Array(dispolayers.layer);
		}
		
		// Find the widget service and resource
		var service, resource;
		<xsl:for-each select="//datastores/datastore/params[./widgettype='wmslayeradder']">
			service = "<xsl:value-of select="../service" />";
			<xsl:variable name="datastore"><xsl:value-of select="../name" /></xsl:variable> 
			<xsl:for-each select="//resources/resource/datastores[./datastore=$datastore]">
				resource = "<xsl:value-of select="../name" />";
			</xsl:for-each>
		</xsl:for-each>
		
		objGPWidget<xsl:value-of select="$pWidgetName" /> = new OpenLayers.Control.WMSLayerAdder({
			i18n_alert_error_title:					"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_alert_error_title','JS')" />",
			i18n_alert_empty_url:					"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_alert_empty_url','JS')" />",
			i18n_alert_empty_name:					"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_alert_empty_name','JS')" />",
			i18n_alert_invalid_wms_getcapabilities:	"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_alert_invalid_wms_getcapabilities','JS')" />",
			i18n_alert_no_selected_server:			"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_alert_no_selected_server','JS')" />",
			i18n_btn_external_connect:				"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_btn_external_connect','JS')" />",
			i18n_btn_external_delete:				"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_btn_external_delete','JS')" />",
			i18n_btn_external_addserver:			"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_btn_external_addserver','JS')" />",
			i18n_field_external_url:				"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_field_external_url','JS')" />",
			i18n_field_external_user:				"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_field_external_user','JS')" />",
			i18n_field_external_pass:				"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_field_external_pass','JS')" />",
			i18n_form_btn_add:						"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_form_btn_add','JS')" />",
			i18n_form_btn_cancel:					"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_form_btn_cancel','JS')" />",
			i18n_form_itemselect_add:				"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_form_itemselect_add','JS')" />",
			i18n_form_itemselect_available:			"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_form_itemselect_available','JS')" />",
			i18n_form_server_list:					"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_form_server_list','JS')" />",
			i18n_prompt_servername_title:			"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_prompt_servername_title','JS')" />",
			i18n_prompt_servername_msg:				"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_prompt_servername_msg','JS')" />",
			i18n_tab_local_server:					"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_tab_local_server','JS')" />",
			i18n_tab_external_server:				"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_tab_external_server','JS')" />",
			i18n_wmslayeradder_tooltip:				"<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_wmslayeradder_tooltip','JS')" />",

			dispolayers: dispolayers,
			map: oMap,
			service: service,
			resource: resource,
			mapfishlayertree: mapfishlayertree
		});
		
		objGPWidget<xsl:value-of select="$pWidgetName" />GeoExtToolbarOptions = new GeoExt.Action({
            iconCls: 'wmslayeradder',
            control: objGPWidget<xsl:value-of select="$pWidgetName" />,
			tooltip: '<xsl:value-of select="php:functionString('WMSLayerAdder::getText','i18n_wmslayeradder_tooltip','JS')" />'
          });
		
		// Load new capabilities
		objGPWidget<xsl:value-of select="$pWidgetName" />.loadCapabilities();
    </script>
    <!-- javascript END -->
  </xsl:template>

<!-- 
     Function : drawWidget

     Output the widget depending on the render type
-->
  <!-- Call for each widget name -->
  <xsl:template name="wmslayeradder:drawWidget">
    <xsl:param name="pWidgetName" />
   	objGPWidget<xsl:value-of select="$pWidgetName" />
  </xsl:template>


<!--
    Function: getWidgetDepedencies

    Return the list of dependencies for this widget
-->
  <!-- Call for each widget type-->
  <xsl:template name="wmslayeradder:getWidgetDepedencies">

  </xsl:template>

</xsl:stylesheet>
