<?php
/**
* Setter et getter pour la configuration
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Setter et getter pour la configuration
* 
* @category PHP 
* @package  GeoPrisma
* @author   Pascal Martin 
*/ 
class org_geoprisma_SettingImpl extends org_geoprisma_Setting
{
    
    private static $s_strConfig  = self::CONFIG;
    
    private static $s_strXMLConfigFile  = self::XML_CONFIG_FILE; 

    private static $s_strProxyURL  = self::PROXY_URL;
    private static $s_objArrayProxyURLAliases = array();

    private static $s_strURLSeparator = self::URL_SEPARATOR;
    
    private static $s_strTemplateDir = self::TEMPLATE_DIR;
    
    private static $s_strWidgetDir = self::WIDGET_DIR;   
    
    private static $s_strLocaleDir = self::LOCALE_DIR;   
    
    private static $s_strACLClass = self::ACL_CLASS;
    
    private static $s_strLocaleClass = self::LOCALE_CLASS;
    private static $s_strLocaleLanguageDefault = self::LOCALE_LANGUAGE_DEFAULT;
    private static $s_strLocaleLanguageRequest = self::LOCALE_LANGUAGE_REQUEST;
     
    private static $s_strLibDir = self::LIB_DIR; 
    
    private static $s_strTemplateUrl = self::TEMPLATE_URL;      
    private static $s_strWidgetUrl = self::WIDGET_URL;      
    private static $s_strLibUrl = self::LIB_URL;      

    private static $s_bTileCacheProxyCaching = false;
    private static $s_iTileCacheProxyCachingExpires = 1209600;
    private static $s_bWMSProxyCaching = false;
    private static $s_iWMSProxyCachingExpires = 1209600;
    private static $s_bTMSProxyCaching = false;
    private static $s_iTMSProxyCachingExpires = 1209600;

    // --------------------------
    // Getter
    // --------------------------
    
    /**
    * Return TemplateUrl
    * 
    * @return string
    */
    public static function getTemplateUrl()
    {
         return self::$s_strTemplateUrl;
    }
    
    /**
    * Return WidgetUrl
    * 
    * @return string
    */
    public static function getWidgetUrl()
    {
         return self::$s_strWidgetUrl;
    }
    
    /**
    * Return LibUrl
    * 
    * @return string
    */
    public static function getLibUrl()
    {
         return self::$s_strLibUrl;
    }
    
    /**
    * Return LibDir
    * 
    * @return string
    */
    public static function getLibDir()
    {
         return self::$s_strLibDir;
    }
    
    /**
    * Return LocaleLanguageRequest
    * 
    * @return string
    */
    public static function getLocaleLanguageRequest()
    {
         return self::$s_strLocaleLanguageRequest;
    }
    
    /**
    * Return LocaleLanguageDefault
    * 
    * @return string
    */
    public static function getLocaleLanguageDefault()
    {
         return self::$s_strLocaleLanguageDefault;
    }
    
    /**
    * Return LocaleClass
    * 
    * @return string
    */
    public static function getLocaleClass()
    {
         return self::$s_strLocaleClass;
    }
    
    /**
    * Return ACLClass
    * 
    * @return string
    */
    public static function getACLClass()
    {
         return self::$s_strACLClass;
    }
    
    /**
    * Return WidgetDir
    * 
    * @return string
    */
    public static function getWidgetDir()
    {
         return self::$s_strWidgetDir;
    }
    
    /**
    * Return TemplateDir
    * 
    * @return string
    */
    public static function getTemplateDir()
    {
         return self::$s_strTemplateDir;
    }
    
    
    /**
    * Return LocaleDir
    * 
    * @return string
    */
    public static function getLocaleDir()
    {
         return self::$s_strLocaleDir;
    }
    
    /**
    * Return Config class name
    * 
    * @return string
    */
    public static function getConfig()
    {
         return self::$s_strConfig;
    }
    
    /**
    * Return Xml config file path
    * 
    * @return string
    */
    public static function getXMLConfigFile()
    {
         return self::$s_strXMLConfigFile;
    }
    
    /**
    * Return Proxy URL
    * 
    * @return string
    */
    public static function getProxyURL()
    {
         return self::$s_strProxyURL;
    }    

    /**
    * Return Proxy URLs as a string separated by " "
    * 
    * @return string
    */
    public static function getProxyURLAliases()
    {
        return implode(self::getURLSeparator(), self::$s_objArrayProxyURLAliases);
    }

    /**
    * Return Proxy URLs array
    * 
    * @return array
    */
    public static function getProxyURLAliasesArray()
    {
        return self::$s_objArrayProxyURLAliases;
    }

    /**
    * Return URL Separator
    * 
    * @return string
    */
    public static function getURLSeparator()
    {
         return self::$s_strURLSeparator;
    }

    /**
    * Get TileCache Proxy Caching
    *
    * @return boolean
    */
    public static function getTileCacheProxyCaching()
    {
        return self::$s_bTileCacheProxyCaching;
    }

    /**
    * Get TileCache Proxy Caching Expires
    *
    * @return integer
    */
    public static function getTileCacheProxyCachingExpires()
    {
        return self::$s_iTileCacheProxyCachingExpires;
    }

    /**
    * Get WMS Proxy Caching
    *
    * @return boolean
    */
    public static function getWMSProxyCaching()
    {
        return self::$s_bWMSProxyCaching;
    }

    /**
    * Get WMS Proxy Caching Expires
    *
    * @return integer
    */
    public static function getWMSProxyCachingExpires()
    {
        return self::$s_iWMSProxyCachingExpires;
    }

    /**
    * Get TMS Proxy Caching
    *
    * @return boolean
    */
    public static function getTMSProxyCaching()
    {
        return self::$s_bTMSProxyCaching;
    }

    /**
    * Get TMS Proxy Caching Expires
    *
    * @return integer
    */
    public static function getTMSProxyCachingExpires()
    {
        return self::$s_iTMSProxyCachingExpires;
    } 

    // --------------------------
    // Setter
    // --------------------------
    
    /**
    * Set LibUrl 
    * 
    * @param string $pstrValue Value
    * 
    * @return void
    */
    public static function setLibUrl($pstrValue)
    {
         com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrValue);
         
         self::$s_strLibUrl = $pstrValue;
    }
    
    /**
    * Set WidgetUrl 
    * 
    * @param string $pstrValue Value
    * 
    * @return void
    */
    public static function setWidgetUrl($pstrValue)
    {
         com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrValue);
         
         self::$s_strWidgetUrl = $pstrValue;
    }
    
    /**
    * Set TemplateUrl 
    * 
    * @param string $pstrValue Value
    * 
    * @return void
    */
    public static function setTemplateUrl($pstrValue)
    {
         com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrValue);
         
         self::$s_strTemplateUrl = $pstrValue;
    }
    
    /**
    * Set LibDir 
    * 
    * @param string $pstrValue Value
    * 
    * @return void
    */
    public static function setLibDir($pstrValue)
    {
         com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrValue);
         
         self::$s_strLibDir = $pstrValue;
    }
     
    /**
    * Set LocaleClass 
    * 
    * @param string $pstrValue Value
    * 
    * @return void
    */
    public static function setLocaleClass($pstrValue)
    {
         com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrValue);
         
         self::$s_strLocaleClass = $pstrValue;
    }
    
    /**
    * Set ACLClass 
    * 
    * @param string $pstrValue Value
    * 
    * @return void
    */
    public static function setACLClass($pstrValue)
    {
         com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrValue);
         
         self::$s_strACLClass = $pstrValue;
    } 
    
    /**
    * Set WidgetsDir
    * 
    * @param string $pstrValue Value
    * 
    * @return void
    */
    public static function setWidgetDir($pstrValue)
    {
         com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrValue);
         
         self::$s_strWidgetDir = $pstrValue;
    } 
    
    /**
    * Set TemplateDir
    * 
    * @param string $pstrValue Value
    * 
    * @return void
    */
    public static function setTemplateDir($pstrValue)
    {
         com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrValue);
         
         self::$s_strTemplateDir = $pstrValue;
    } 
    
    /**
    * Set LocaleDir
    * 
    * @param string $pstrValue Value
    * 
    * @return void
    */
    public static function setLocaleDir($pstrValue)
    {
         com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrValue);
         
         self::$s_strLocaleDir = $pstrValue;
    } 

    /**
    * set LocaleLanguageDefault
    *
    * @param string $pstrValue Value
    * 
    * @return string
    */
    public static function setLocaleLanguageDefault($pstrValue)
    {
         com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrValue);
         self::$s_strLocaleLanguageDefault = $pstrValue;
    }
    
    /**
    * Set Config class name 
    * 
    * @param string $pstrValue Value
    * 
    * @return void
    */
    public static function setConfig($pstrValue)
    {
         com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrValue);
         
         self::$s_strConfig = $pstrValue;
    } 
    
    /**
    * Set Xml config file path  
    * 
    * @param string $pstrValue Value
    * 
    * @return void
    */
    public static function setXMLConfigFile($pstrValue)
    {
         com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrValue);
         
         self::$s_strXMLConfigFile = $pstrValue;
    } 
    
    /**
    * Set Proxy URL
    * 
    * @param string $pstrValue Value
    * 
    * @return void 
    */
    public static function setProxyURL($pstrValue)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrValue);  
        
        self::$s_strProxyURL = $pstrValue;
    }  

    /**
    * Set Proxy URL Aliases with the session name and id of the original session.
    *
    * @param array $pobjArrayProxyURLAliases List of url strings
    *
    * @return void
    */
    public static function setProxyURLAliases($pobjArrayProxyURLAliases)
    {
        $iCount = count($pobjArrayProxyURLAliases);
        for ($iIter=0; $iIter<$iCount; $iIter++)
        {
            $iPos = strpos($pobjArrayProxyURLAliases[$iIter], "?");
            if ($iPos === false) 
            {
                $pobjArrayProxyURLAliases[$iIter] .= "?";
            }
            else if ($iPos != (strlen($pobjArrayProxyURLAliases[$iIter])-1)) 
            {
                $pobjArrayProxyURLAliases[$iIter] .= "&";
            }
            $pobjArrayProxyURLAliases[$iIter] .= session_name()."=".session_id()."&";
        }

        self::$s_objArrayProxyURLAliases = $pobjArrayProxyURLAliases;
    }

    /**
    * Set URL Separator
    * 
    * @param string $pstrValue Value
    * 
    * @return void 
    */
    public static function setURLSeparator($pstrValue)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrValue);  
        
        self::$s_strURLSeparator = $pstrValue;
    }  
    
    /**
    * Set Proxy Caching
    *
    * @param boolean $pbValue true or false
    *
    * @return void
    */
    public static function setProxyCaching($pbValue)
    {
        self::setTileCacheProxyCaching($pbValue);
        self::setWMSProxyCaching($pbValue);
        self::setTMSProxyCaching($pbValue);
    }

    /**
    * Set Proxy Caching Expires
    *
    * @param integer $piValue Value
    *
    * @return void
    */
    public static function setProxyCachingExpires($piValue)
    {
        self::setTileCacheProxyCachingExpires($piValue);
        self::setWMSProxyCachingExpires($piValue);
        self::setTMSProxyCachingExpires($piValue);
    }

    /**
    * Set TileCache Proxy Caching
    *
    * @param boolean $pbValue true or false
    *
    * @return void
    */
    public static function setTileCacheProxyCaching($pbValue)
    {
        self::$s_bTileCacheProxyCaching = $pbValue;
    }

    /**
    * Set TileCache Proxy Caching Expires
    *
    * @param integer $piValue Value
    *
    * @return void
    */
    public static function setTileCacheProxyCachingExpires($piValue)
    {
        self::$s_iTileCacheProxyCachingExpires = $piValue;
    }

    /**
    * Set WMS Proxy Caching
    *
    * @param boolean $pbValue true or false
    *
    * @return void
    */
    public static function setWMSProxyCaching($pbValue)
    {
        self::$s_bWMSProxyCaching = $pbValue;
    }

    /**
    * Set WMS Proxy Caching Expires
    *
    * @param integer $piValue Value
    *
    * @return void
    */
    public static function setWMSProxyCachingExpires($piValue)
    {
        self::$s_iWMSProxyCachingExpires = $piValue;
    }

    /**
    * Set TMS Proxy Caching
    *
    * @param boolean $pbValue true or false
    *
    * @return void
    */
    public static function setTMSProxyCaching($pbValue)
    {
        self::$s_bTMSProxyCaching = $pbValue;
    }

    /**
    * Set TMS Proxy Caching Expires
    *
    * @param integer $piValue Value
    *
    * @return void
    */
    public static function setTMSProxyCachingExpires($piValue)
    {
        self::$s_iTMSProxyCachingExpires = $piValue;
    }
}
  
?>
