<?php
/**
* Récupère les informations de configuration d'un fichier xml    
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Récupère les informations de configuration d'un fichier xml
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Config  
* @author     Pascal Martin 
*/ 
class org_geoprisma_config_XMLConfig extends org_geoprisma_config_Config
{
    /**
    * @var org_geoprisma_config_XMLConfig
    */
    private static $s_objInstance = null; 
    
    protected $m_objArrayService = null;
    protected $m_objArrayLayout = null;
    protected $m_objArrayDatastore = null;
    protected $m_objArrayWidget = null;
    protected $m_objArrayAccessFilter = null;
    protected $m_objArrayResource = null;   

    /**
    * @var string
    */
    protected $m_strConfigFile = null;          
    
    /**
    * Construit une nouvelle instance de org_geoprisma_config_XMLConfig
    * Force l'utilisation de getInstance (Singleton)
    * 
    * @throws org_geoprisma_exception_BadXMLConfigFile
    * 
    * @return org_geoprisma_config_XMLConfig
    */
    private function __construct()
    {    
        $this->load();           
    }
    
    /**
    * return instanceof org_geoprisma_config_XMLConfig
    * return instanceof org_geoprisma_config_Config 
    * return instanceof com_borealis_foundation_util_Singleton
    * 
    * @return org_geoprisma_config_XMLConfig 
    */
    public static function getInstance()
    {
        if (self::$s_objInstance === null) 
        {
            self::$s_objInstance = new org_geoprisma_config_XMLConfig();
        }
        self::$s_objInstance->checkReload();
        return self::$s_objInstance;
    } 
    
    /**
    * Charge la config
    * 
    * @return void
    */
    protected function load()
    {
        try
        {
            $this->m_strConfigFile = org_geoprisma_SettingImpl::getXMLConfigFile();
            $objDomDoc = new DOMDocument();
            $objDomDoc->load($this->m_strConfigFile);
            $objSimpleXMLElement = simplexml_import_dom($objDomDoc);
            $this->checkMapFishLayerTree($objSimpleXMLElement);
            $objSimpleXMLElement = $this->checkLayers($objSimpleXMLElement);
            $this->loadService($objSimpleXMLElement); 
            $this->loadLayout($objSimpleXMLElement);  
            $this->loadDatastore($objSimpleXMLElement);
            $this->loadWidget($objSimpleXMLElement);
            $this->loadAccessFilter($objSimpleXMLElement);
            $this->loadResource($objSimpleXMLElement);
        }
        catch(Exception $objError)
        {
            throw new org_geoprisma_exception_BadXMLConfigFileException(org_geoprisma_SettingImpl::getXMLConfigFile(), $objError->__toString());       
        }  
    }
    
    /**
    * Charge le tableau d'object org_geoprisma_service_Service 
    * a partir du document xml de configuration
    * 
    * @param SimpleXMLElement &$pobjSimpleXMLElement SimpleXMLElement du fichier de config
    * 
    * @return void
    */
    protected function loadService(SimpleXMLElement &$pobjSimpleXMLElement)
    {
        $this->m_objArrayService = new ArrayObject();
                            
        $objArraySimpleXMLElement = $pobjSimpleXMLElement->xpath("/geoprisma/services/*");
        foreach ($objArraySimpleXMLElement as $objSimpleXMLElementService)
        {
            $objArrayService = $this->simpleXMLToArray($objSimpleXMLElementService);
            $strType = $objSimpleXMLElementService->getName(); 
            $this->m_objArrayService->append(org_geoprisma_service_ServiceFactory::getService($strType, $objArrayService));  
        }
    }
    
    /**
    * Charge le tableau d'object org_geoprisma_layout_Layout 
    * a partir du document xml de configuration
    * 
    * @param SimpleXMLElement &$pobjSimpleXMLElement SimpleXMLElement du fichier de config
    * 
    * @return void
    */
    protected function loadLayout(SimpleXMLElement &$pobjSimpleXMLElement)
    {
        $this->m_objArrayLayout = new ArrayObject();
                    
        $objArraySimpleXMLElement = $pobjSimpleXMLElement->xpath("/geoprisma/layouts/layout");
        foreach ($objArraySimpleXMLElement as $objSimpleXMLElementService)
        {
            $strName = (string)$objSimpleXMLElementService->name;
            $strFile = (string)$objSimpleXMLElementService->file;
            $strDrawMode = (string)$objSimpleXMLElementService->drawmode;
            $this->m_objArrayLayout->append(new org_geoprisma_layout_Layout($strName, $strFile, $strDrawMode));
        }   
    }
   
    /**
    * Charge le tableau d'object org_geoprisma_datastore_Datastore 
    * a partir du document xml de configuration
    * 
    * @param SimpleXMLElement &$pobjSimpleXMLElement SimpleXMLElement du fichier de config
    * 
    * @return void
    */
    private function loadDatastore(SimpleXMLElement &$pobjSimpleXMLElement)
    {
        $this->m_objArrayDatastore = new ArrayObject();
                    
        $objArraySimpleXMLElement = $pobjSimpleXMLElement->xpath("/geoprisma/datastores/datastore");
        foreach ($objArraySimpleXMLElement as $objSimpleXMLElementDatastore)
        {
            $objArrayDataStore = $this->simpleXMLToArray(
                $objSimpleXMLElementDatastore
            );

            // name
            $strName = $objArrayDataStore['name'];

            // service
            $objService = $this->getService($objArrayDataStore['service']);

            // layers and options
            $strLayers = null;
            $objArrayDataStoreOptions = Array();
            if (isset($objArrayDataStore['params'])) 
            {
                if (isset($objArrayDataStore['params']['layers'])) 
                {
                    $strLayers = $objArrayDataStore['params']['layers'];
                    unset($objArrayDataStore['params']['layers']);
                }

                $objArrayDataStoreOptions = $objArrayDataStore['params'];
            }
            else
            {
                if (isset($objArrayDataStore['layers'])) 
                {
                    $strLayers = $objArrayDataStore['layers'];
                }

                if (isset($objArrayDataStore['options'])) 
                {
                    $objArrayDataStoreOptions = $objArrayDataStore['options'];
                }
            }

            $this->m_objArrayDatastore->append(
                new org_geoprisma_datastore_Datastore(
                    $strName,
                    $objService,
                    $strLayers,
                    $objArrayDataStoreOptions
                )
            );
        } 
    }
    
    /**
    * Charge le tableau d'object org_geoprisma_widget_Widget 
    * a partir du document xml de configuration
    * 
    * @param SimpleXMLElement &$pobjSimpleXMLElement SimpleXMLElement du fichier de config
    * 
    * @return void
    */
    protected function loadWidget(SimpleXMLElement &$pobjSimpleXMLElement)
    {
        $this->m_objArrayWidget = new ArrayObject();
        $this->m_objArrayExclusiveUnboundWidgetName = array();
                    
        $objArraySimpleXMLElement = $pobjSimpleXMLElement->xpath("/geoprisma/widgets/*");
        foreach ($objArraySimpleXMLElement as $objSimpleXMLElementWidget)
        {
            $objArrayWidget = $this->simpleXMLToArray(
                $objSimpleXMLElementWidget
            );
            $strType = $objSimpleXMLElementWidget->getName();
            $this->m_objArrayWidget->append(org_geoprisma_widget_WidgetFactory::getWidget($strType, $objArrayWidget));
        }
    }
    
    /**
    * Charge le tableau d'object org_geoprisma_accessfilter_AccessFilter 
    * a partir du document xml de configuration
    * 
    * @param SimpleXMLElement &$pobjSimpleXMLElement SimpleXMLElement du fichier de config
    * 
    * @return void
    */
    protected function loadAccessFilter(SimpleXMLElement &$pobjSimpleXMLElement)
    {
        $this->m_objArrayAccessFilter = new ArrayObject();
                    
        $objArraySimpleXMLElement = $pobjSimpleXMLElement->xpath("/geoprisma/accessfilters/*");
        foreach ($objArraySimpleXMLElement as $objSimpleXMLElementAccessFilter)
        {
            $objArrayAccessFilter = $this->simpleXMLToArray(
                $objSimpleXMLElementAccessFilter
            );

            $strName = $objArrayAccessFilter['name'];

            $objArrayConditions = array();
            if (isset($objArrayAccessFilter['condition'])
                && $objArrayAccessFilter['condition']
            ) 
            {
                $objArrayConditions[] = $objArrayAccessFilter['condition'];
            }
            else if (isset($objArrayAccessFilter['conditions'])
                && is_array($objArrayAccessFilter['conditions'])
            ) 
            {
                $objArrayConditions = $objArrayAccessFilter['conditions'];
            }

            $this->m_objArrayAccessFilter->append(
                new org_geoprisma_accessfilter_AccessFilter(
                    $strName,
                    $objArrayConditions
                )
            );
        }
    }
    
    /**
    * Charge le tableau d'object org_geoprisma_resource_Resource 
    * a partir du document xml de configuration
    * 
    * @param SimpleXMLElement &$pobjSimpleXMLElement SimpleXMLElement du fichier de config
    * 
    * @return void
    */
    protected function loadResource(SimpleXMLElement &$pobjSimpleXMLElement)
    {
        $this->m_objArrayResource = new ArrayObject();
                    
        $objArraySimpleXMLElement = $pobjSimpleXMLElement->xpath("/geoprisma/resources/resource");
        foreach ($objArraySimpleXMLElement as $objSimpleXMLElementResource)
        {
            $objArrayResource = $this->simpleXMLToArray(
                $objSimpleXMLElementResource
            );

            $strName = (string)$objSimpleXMLElementResource->name;
            $strTitle = (string)$objSimpleXMLElementResource->title;

            $objArrayDatastore = new ArrayObject(); 
            $objArrayWidget = new ArrayObject();
            $objArrayAccessFilter = new ArrayObject();
            $objArrayOption = new ArrayObject();
            $objArrayField = new ArrayObject();
            
            foreach ($objSimpleXMLElementResource->datastores->datastore as $strDatastoreName)
            {
                $objArrayDatastore->append($this->getDatastore((string)$strDatastoreName));   
            }
            foreach ($objSimpleXMLElementResource->xpath('widgets/widget') as $strWidgetName)
            {
                $objArrayWidget->append($this->getWidget((string)$strWidgetName));   
            }
            foreach ($objSimpleXMLElementResource->xpath('accessfilters/accessfilter') as $strAccessFilterName)
            {
                $objArrayAccessFilter->append($this->getAccessFilter((string)$strAccessFilterName));   
            }
            foreach ($objSimpleXMLElementResource->xpath('options/*') as $objSimpleXMLElementOption)
            {
                $objArrayOption->append(
                    new org_geoprisma_option_Option(
                        $objSimpleXMLElementOption->getName(),
                        $this->simpleXMLToArray(
                            $objSimpleXMLElementOption
                        )
                    )
                );                          
            }

            if ($objSimpleXMLElementResource->fields) 
            {
                foreach ($objSimpleXMLElementResource->fields->field as $objField)
                {
                    if ((string)$objField->title) 
                    {
                        $strFieldTitle = (string)$objField->title;
                    }
                    else
                    {
                        $strFieldTitle = (string)$objField->name;
                    }

                    $objArrayField->append(
                        new org_geoprisma_field_Field(
                            (string)$objField->name,
                            (string)$objField->name,
                            $this->simpleXMLToArray(
                                $objField
                            ),
                            $strFieldTitle
                        )
                    );
                }
            }
            
            $this->m_objArrayResource->append(
                new org_geoprisma_resource_Resource(
                    $strName,
                    $objArrayDatastore,
                    $objArrayWidget,
                    $objArrayAccessFilter,
                    $objArrayOption,
                    $strTitle,
                    $objArrayField
                )
            );
        } 
    }
    
    /**
    * Vérifie si la config doit être reloader, car le fichier de configuration a changé
    * 
    * @return void 
    */
    public function checkReload()
    {
        if ($this->m_strConfigFile != org_geoprisma_SettingImpl::getXMLConfigFile()) 
        {
            $this->reload();
        }    
    }
    
    /**
    * Force le rechargement de la config
    * 
    * @return void
    */
    public function reload()
    {
        $this->load();      
    }
    
    // -------------------------------------------------------------------
    // Implement abstarct methode of org_geoprisma_config_Config
    // -------------------------------------------------------------------
    
    /**
    * Retourne un object service
    * 
    * @param string $pstrServiceName Service Name
    * 
    * @return org_geoprisma_service_Service 
    */
    public function getService($pstrServiceName)
    { 
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrServiceName);
        
        foreach ($this->m_objArrayService as $objService)
        {
            if ($objService->getName() == $pstrServiceName) 
            {
                return $objService; 
            }        
        }
        throw new org_geoprisma_exception_ServiceNotFoundException($pstrServiceName);
    }
    
    /**
    * Retourne la liste de service disponible service
    * 
    * @return ArrayObject of org_geoprisma_service_Service 
    */
    public function getServices()
    {
        return $this->m_objArrayService;
    } 
    
    /**
    * Retorrne un object layout
    * 
    * @param string $pstrLayoutName Layout name
    * 
    * @return org_geoprisma_layout_Layout
    */
    public function getLayout($pstrLayoutName)
    {   
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrLayoutName);
        
        foreach ($this->m_objArrayLayout as $objLayout)
        {
            if ($objLayout->getName() == $pstrLayoutName) 
            {
                return $objLayout; 
            }        
        }
        throw new org_geoprisma_exception_LayoutNotFoundException($pstrLayoutName);
    }
    
    /**
    * Retorrne la liste des layouts disponible
    * 
    * @return ArrayObject of org_geoprisma_layout_Layout
    */
    public function getLayouts()
    {
        return $this->m_objArrayLayout;
    }   
    
    
    /**
    * Retorrne un object datastore
    * 
    * @param string $pstrDatastoreName Datastore name
    * 
    * @return org_geoprisma_datastore_Datastore
    */
    public function getDatastore($pstrDatastoreName)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrDatastoreName);
        
        foreach ($this->m_objArrayDatastore as $objDatastore)
        {
            if ($objDatastore->getName() == $pstrDatastoreName) 
            {
                return $objDatastore; 
            }        
        }
        throw new org_geoprisma_exception_DatastoreNotFoundException($pstrDatastoreName);
    }
    
    /**
    * Retorrne la liste des datastores disponible
    * 
    * @return ArrayObject of org_geoprisma_datastore_Datastore
    */
    public function getDatastores()
    {
        return $this->m_objArrayDatastore;
    }

    /**
    * Retorrne un object widget
    * 
    * @param string  $pstrWidgetName Widget name
    * @param boolean $pbKey          Returns the key or the value.  'value' by
    *                                default.
    * 
    * @return org_geoprisma_widget_Widget
    */
    public function getWidget($pstrWidgetName, $pbKey=false)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrWidgetName); 
        
        foreach ($this->m_objArrayWidget as $iKey => $objWidget)
        {
            if ($objWidget->getName() == $pstrWidgetName) 
            {
                return ($pbKey) ? $iKey : $objWidget ;
            }        
        }
        throw new org_geoprisma_exception_WidgetNotFoundException($pstrWidgetName);
    }
    
    /**
    * Retorrne la liste des widget disponible
    * 
    * @return ArrayObject of org_geoprisma_widget_Widget
    */
    public function getWidgets()
    {
        return $this->m_objArrayWidget;   
    }

    /**
    * Remove a widget from the widgets array using its name.
    * 
    * @param string $pstrWidgetName Widget name
    *
    * @return void
    */
    public function removeWidget($pstrWidgetName)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrWidgetName);
        $iKey = $this->getWidget($pstrWidgetName, true);
        
        unset($this->m_objArrayWidget[$iKey]);
    }

    /**
    * Retorrne un object accessfilter
    * 
    * @param string  $pstrAccessFilterName AccessFilter name
    * @param boolean $pbKey                Returns the key or the value.
    *                                      Defaults to 'value'.
    * 
    * @return org_geoprisma_accessfilter_AccessFilter
    */
    public function getAccessFilter($pstrAccessFilterName, $pbKey=false)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrAccessFilterName); 
        
        foreach ($this->m_objArrayAccessFilter as $iKey => $objAccessFilter)
        {
            if ($objAccessFilter->getName() == $pstrAccessFilterName) 
            {
                return ($pbKey) ? $iKey : $objAccessFilter ;
            }        
        }
        throw new org_geoprisma_exception_OptionNotFoundException($pstrAccessFilterName);
    }
    
    /**
    * Retorrne la liste des accessfilter disponible
    * 
    * @return ArrayObject of org_geoprisma_accessfilter_AccessFilter
    */
    public function getAccessFilters()
    {
        return $this->m_objArrayAccessFilter;   
    }
    
    /**
    * Retorrne un object resource
    * 
    * @param string $pstrResourceName Resource name
    * 
    * @return org_geoprisma_resource_Resource
    */
    public function getResource($pstrResourceName)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrResourceName);
        
        foreach ($this->m_objArrayResource as $objResource)
        {
            if ($objResource->getName() == $pstrResourceName) 
            {
                return $objResource; 
            }        
        }
        throw new org_geoprisma_exception_ResourceNotFoundException($pstrResourceName);        
    }
    
    /**
    * Retorrne la liste des resources disponible
    * 
    * @return ArrayObject of org_geoprisma_resource_Resource
    */
    public function getResources()
    {
        return $this->m_objArrayResource;  
    }

    /**
    * Converts a simpleXML element into an array. Preserves attributes and 
    *     everything.  You can choose to get your elements either flattened,
    *     or stored in a custom index that you define.
    *
    *     For example, for a given element
    *     <field name="someName" type="someType"/>
    *     if you choose to flatten attributes, you would get:
    *     $array['field']['name'] = 'someName';
    *     $array['field']['type'] = 'someType';
    *     If you choose not to flatten, you get:
    *     $array['field']['@attributes']['name'] = 'someName';
    * 
    *     Repeating fields are stored in indexed arrays. so for a markup such
    *     as :
    *     <parent>
    *       <child>a</child>
    *       <child>b</child>
    *       <child>c</child>
    *     </parent>
    *     you array would be:
    *     $array['parent']['child'][0] = 'a';
    *     $array['parent']['child'][1] = 'b';
    *     ...And so on.
    *
    * @param simpleXMLElement $pstrXML             the XML to convert
    * @param boolean          $pbFlattenValues     Choose wether to flatten
    *                                              values or to set them under a
    *                                              particular index.
    *                                              Defaults to true;
    * @param boolean          $pbFlattenAttributes Choose wether to flatten 
    *                                              attributes or to set them
    *                                              under a particular index.  
    *                                              Defaults to true;
    * @param boolean          $pbFlattenChildren   Choose wether to flatten
    *                                              children or to set them under
    *                                              a particular index.
    *                                              Defaults to true;
    * @param string           $pstrValueKey        index for values, in case
    *                                              $pbFlattenValues was set to
    *                                              false. Defaults to "@value"
    * @param string           $pstrAttributesKey   index for attributes, in case
    *                                              $pbFlattenAttributes was set
    *                                              to false. Defaults to 
    *                                              "@attributes"
    * @param string           $pstrChildrenKey     index for children, in case 
    *                                              $pbFlattenChildren was set to
    *                                              false. Defaults to 
    *                                              "@children"
    *
    * @return array the resulting array.
    */
    public function simpleXMLToArray(
        $pstrXML,
        $pbFlattenValues=true,
        $pbFlattenAttributes = true,
        $pbFlattenChildren=true,
        $pstrValueKey='@value',
        $pstrAttributesKey='@attributes',
        $pstrChildrenKey='@children'
    ) {
        $objArrayReturn = array();

        if (!($pstrXML instanceof SimpleXMLElement)) 
        {
            return $objArrayReturn;
        }

        $strName = $pstrXML->getName();
        $strValue = trim((string)$pstrXML);

        if (strlen($strValue) == 0) 
        {
            $strValue = null;
        }

        if ($strValue !== null) 
        {
            if (!$pbFlattenValues) 
            {
                $objArrayReturn[$pstrValueKey] = $strValue;
            }
            else
            {
                $objArrayReturn = $strValue;
            }
        }

        $objArrayChildren = array();
        $bFirst = true;
        foreach ($pstrXML->children() as $strElementName => $objChild) 
        {
            $objArrayValue = $this->simpleXMLToArray(
                $objChild,
                $pbFlattenValues,
                $pbFlattenAttributes,
                $pbFlattenChildren,
                $pstrValueKey,
                $pstrAttributesKey,
                $pstrChildrenKey
            );

            if (substr($strName, 0, -1) == $strElementName) 
            {
                array_push($objArrayChildren, $objArrayValue);
            }
            else
            {
                if (isset($objArrayChildren[$strElementName])) 
                {
                    if ($bFirst) 
                    {
                        $objTemp = $objArrayChildren[$strElementName];
                        unset($objArrayChildren[$strElementName]);

                        $objArrayChildren[$strElementName][] = $objTemp;
                        $bFirst=false;
                    }
                    $objArrayChildren[$strElementName][] = $objArrayValue;
                }
                else
                {
                    $objArrayChildren[$strElementName] = $objArrayValue;
                }
            }
        }

        if (count($objArrayChildren) > 0) 
        {
            if (!$pbFlattenChildren) 
            {
                $objArrayReturn[$pstrChildrenKey] = $objArrayChildren;
            }
            else
            {
                $objArrayReturn = array_merge(
                    $objArrayReturn, $objArrayChildren
                );
            }
        }

        $objArrayAttributes = array();
        foreach ($pstrXML->attributes() as $strName=>$objArrayValue) 
        {
            $objArrayAttributes[$strName] = trim($objArrayValue);
        }

        if (count($objArrayAttributes) > 0) 
        {
            if (!$pbFlattenAttributes) 
            {
                $objArrayReturn[$pstrAttributesKey] = $objArrayAttributes;
            }
            else
            {
                $objArrayReturn = array_merge(
                    $objArrayReturn, $objArrayAttributes
                );
            }
        }
       
        return $objArrayReturn;
    }

    /**
    * Checks if there is a org_geoprisma_widget_MapFishLayerTreeWidget and see
    * that it has 'nodes' nodes instead of 'children' and 'model' nodes.
    * 
    * @param SimpleXMLElement &$pobjSimpleXMLElement SimpleXMLElement of the
    *                                                config file.
    *
    * @return void
    */
    protected function checkMapFishLayerTree(
        SimpleXMLElement &$pobjSimpleXMLElement
    ) {
        if (isset($pobjSimpleXMLElement->widgets->mapfishlayertree)
            && isset($pobjSimpleXMLElement->widgets->mapfishlayertree->options->model)
        ) 
        {
            $strXML = $pobjSimpleXMLElement->widgets->mapfishlayertree->asXML();
            $strXML = str_replace('<model>', '<nodes>', $strXML);
            $strXML = str_replace('</model>', '</nodes>', $strXML);
            $strXML = str_replace('<children>', '<nodes>', $strXML);
            $strXML = str_replace('</children>', '</nodes>', $strXML);
            $objNewSimpleXMLElement = simplexml_load_string($strXML);

            unset($pobjSimpleXMLElement->widgets->mapfishlayertree);

            $this->simplexml_append(
                $pobjSimpleXMLElement->widgets,
                $objNewSimpleXMLElement
            );            
        }
    }

    /**
    * Appends a SimpleXMLElement to an other SimpleXMLElement.
    * 
    * @param SimpleXMLElement $pobjParent   Parent element
    * @param SimpleXMLElement $pobjNewChild New child element to append to the
    *                                       parent.
    *
    * @return void
    */
    public function simplexml_append(
        SimpleXMLElement $pobjParent, SimpleXMLElement $pobjNewChild
    ) {
        $objNode1 = dom_import_simplexml($pobjParent);
        $objDomSxe = dom_import_simplexml($pobjNewChild);
        $objNode2 = $objNode1->ownerDocument->importNode($objDomSxe, true);
        $objNode1->appendChild($objNode2);
    }

    /**
    * Checks inside the map tag if there is any 'attributegualfunction' or
    *     'fidegualfunction' nodes and replace them for 'function' nodes.
    * 
    * @param SimpleXMLElement &$pobjSimpleXMLElement SimpleXMLElement of the
    *                                                config file.
    *
    * @return SimpleXMLElement
    */
    protected function checkLayers(
        SimpleXMLElement &$pobjSimpleXMLElement
    ) {

        $objArrayElements = array_merge(
            $pobjSimpleXMLElement->xpath("//attributegualfunction"),
            $pobjSimpleXMLElement->xpath("//fidegualfunction")
        );

        if (count($objArrayElements) > 0) 
        {
            $strXML = $pobjSimpleXMLElement->asXML();
            $strXML = str_replace(
                '<attributegualfunction>', '<function>', $strXML
            );
            $strXML = str_replace(
                '</attributegualfunction>', '</function>', $strXML
            );
            $strXML = str_replace(
                '<fidegualfunction>', '<function>', $strXML
            );
            $strXML = str_replace(
                '</fidegualfunction>', '</function>', $strXML
            );
            $pobjSimpleXMLElement = simplexml_load_string($strXML);

        }

        return $pobjSimpleXMLElement;
    }
}


?>
