<?php
/**
* Classe de base des layer, Lier aux fichier de configuration
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Classe de base des layer, Lier aux fichier de configuration  
* 
*       <layer>
*          <resourcename>ResourceVMap0</resourcename>
*          <servicetype>wms</servicetype>
*          <options>
*            <isbaselayer>true</isbaselayer>
*            <projection>EPSG:4326</projection>
*          </options>
*       </layer>
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Datastore 
* @author     Pascal Martin 
*/ 
class org_geoprisma_layer_Layer extends org_geoprisma_ConfigElement
{
    private $m_objArrayResourceNames = null;
    private $m_strServiceType = null;
    private $m_strOptions = null;
    
    /**
    * Construit un instance de org_geoprisma_layer_Layer
    * 
    * @param ArrayObject $pobjArrayResourceNames Nom de la ressource du layer
    * @param string      $pstrServiceType        Type de service du layer
    * @param string      $pstrOptions            String XMl avec les options du layer
    * 
    * @return org_geoprisma_layer_Layer
    */
    public function __construct(
        ArrayObject $pobjArrayResourceNames,
        $pstrServiceType, 
        $pstrOptions
    ) {    
        //com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrServiceType);
        //com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrOptions); 
        
        $this->m_objArrayResourceNames = $pobjArrayResourceNames;
        $this->m_strServiceType = $pstrServiceType;
        $this->m_strOptions = $pstrOptions; 
    }

    /**
    * Layer ResourceName
    *
    * @return ArrayObject
    */
    public function getResourceNames()
    {
        return $this->m_objArrayResourceNames;   
    }
    
    /**
    * Layer ServiceType
    *
    * @return string
    */
    public function getServiceType()
    {
        return $this->m_strServiceType;   
    }
    
    /**
    * Layer Options
    *
    * @return string
    */
    public function getOptions()
    {
        return $this->m_strOptions;   
    }

    /**
    *  Ajout le layer courant (this) dans le xml      
    * 
    * @param DOMElement  &$pobjDomElementLayers Noeud xml ou le layer doit être ajouter
    * @param DOMDocument &$pobjDomDocument      Document XMl ou le noeud xml datastore est ajouter, utiliser pour construire les noeuds
    * 
    * @return void
    */
    public function insertIntoXML(
        DOMElement &$pobjDomElementLayers, DOMDocument &$pobjDomDocument
    ) {
        // create single <layer> node and append to <layers>
        $objDomElementLayer = $pobjDomDocument->createElement('layer');
        $pobjDomElementLayers->appendChild($objDomElementLayer);

        // create <resourcenames>, <servicetype> and <options> nodes
        // with their childs and append to <layer>
        $objDomElementLayer->appendChild(
            $this->getDomElements(
                'resourcenames', 
                $this->m_objArrayResourceNames, 
                $pobjDomDocument
            )
        );
        $objDomElementLayer->appendChild(
            $this->getDomElements(
                'servicetype', $this->m_strServiceType, $pobjDomDocument
            )
        );
        $objDomElementLayer->appendChild(
            $this->getDomElements(
                'options', $this->m_strOptions, $pobjDomDocument
            )
        );
        
    }
}

?>
