<?php
/**
* Return text based on language and domain name
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Return text based on language and domain name
*  
* @category   PHP 
* @package    GeoPrisma
* @subpackage UI
* @author     Pascal Martin
*/      
  
abstract class org_geoprisma_locale_Locale implements com_borealis_foundation_util_Singleton
{
    private $m_strLanguage = 'en_US';
    
    /**
    * Return the current use langage for traduction (e.g. en_US , fr_CA) 
    * 
    * @return string     
    */
    public function getLanguage()
    {
        return $this->m_strLanguage;   
    }

    /**
    * Set the current use langage for traduction (e.g. en_US , fr_CA)
    * 
    * @param string $pstrLanguage Language
    * 
    * @return void
    */
    public function setLanguage($pstrLanguage)
    {
        $this->m_strLanguage = $pstrLanguage; 
    }
    
    /**
    * Retourne le language par défaut
    * 
    * @return string 
    */
    public function getDefaultLanguage()
    {
        if (!session_id()) 
        {
            session_start();
        }
        $strRequest = org_geoprisma_SettingImpl::getLocaleLanguageRequest();
        if (isset($_REQUEST) && isset($_REQUEST[$strRequest])) 
        {
            $_SESSION[$strRequest] = $_REQUEST[$strRequest];            
            return $_REQUEST[$strRequest]; 
        }
        else if (isset($_SESSION) && isset($_SESSION[$strRequest])) 
        {
            return $_SESSION[$strRequest]; 
        }   
        else
        {
            return org_geoprisma_SettingImpl::getLocaleLanguageDefault();
        }           
    }
    
    /**
    * Return text based on domainName for right language
    * 
    * @param string $pstrMsg        Message Key
    * @param string $pstrDomainName Name of the 
    *
    * @return string 
    */  
    abstract public function getTranslation($pstrMsg, $pstrDomainName);
    
    /**
    * Return text based on domainName for right language
    * 
    * @param string $pstrMsg        Message Key
    * @param string $pstrDomainName Name of the 
    * @param string $pstrType       Type encoding "JS" "HTML"
    *
    * @return string 
    */  
    static public function getText($pstrMsg, $pstrDomainName, $pstrType = null) 
    {
        $strResult = org_geoprisma_locale_LocaleFactory::getLocale()->getTranslation($pstrMsg, $pstrDomainName);
        
        switch (strtolower($pstrType)) 
        {
            case 'js':
                $strResult = self::jsEncode($strResult);
                break;
            case 'html':
                $strResult = self::htmlEncode($strResult);
                break;   
        }
        
        return $strResult;    
    }
    
    /**
    * Encode la string recu pour être imprimer dans du code Javascritp
    * 
    * @param string $pstrText Chaine de texte
    * 
    * @return string
    */
    static public function jsEncode($pstrText)
    {
        return json_encode($pstrText);     
    }
    
    /**
    * Encode la string recu pour être imprimer dans du code Html
    * 
    * @param string $pstrText Chaine de texte
    * 
    * @return string
    */
    static private function htmlEncode($pstrText)
    {
        return htmlspecialchars($pstrText, ENT_QUOTES);     
    }
}
?>
