<?php
/**
* Factory de proxy   
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Factory de proxy
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Proxy 
* @author     Pascal Martin 
*/ 
class org_geoprisma_proxy_ProxyFactory
{
    /**
    * Return the good proxy depending of osmservice get parameter
    * 
    * @return org_geoprisma_proxy_Proxy
    */
    static public function getProxy()
    {
        // ############################################################
        // Check param
        // ############################################################    
        if (!isset($_REQUEST['osmservice'])) 
        {
            throw new Exception("osmservice param is missing");
        }

        $objConfig = org_geoprisma_config_ConfigFactory::getConfig(); 
                
        // New way to call a WidgetProxy object without having to use a service
        // and a resource
        if ($_REQUEST['osmservice'] == org_geoprisma_service_Service::BLANK_SERVICE) 
        {
            if (isset($_REQUEST['osmprocess'])) 
            {
                // widget process
                if (isset($_REQUEST['osmwidgettype'])) 
                {
                    $objArrayWidgets = $objConfig->getWidgets();
                    $objRequiredWidget = null;
                    foreach ($objArrayWidgets as $objWidget)
                    {
                        if ($objWidget->getType() == $_REQUEST['osmwidgettype']) 
                        {
                            $objRequiredWidget = $objWidget;
                            if (!in_array($_REQUEST['osmprocess'], $objRequiredWidget->getProcessList())) 
                            {
                                throw new Exception("osmprocess param is missing/invalid");
                            }
                            $strMethod = $_REQUEST['osmprocess'];
                            break;
                        }
                    }

                    if ($objRequiredWidget == null) 
                    {
                        throw new Exception("The osmwidgetype '".$_REQUEST['osmwidgettype']."' was not found in this config");
                    }

                    $objProxy = new org_geoprisma_proxy_widget_WidgetProxy(
                        null, 
                        null, 
                        $objRequiredWidget, 
                        $strMethod
                    );

                    return $objProxy;
                }
                // config process
                else
                {
                    $objProxy = new org_geoprisma_proxy_config_ConfigProxy(
                        $objConfig,
                        $_REQUEST['osmprocess']
                    );
        
                    return $objProxy;
                }
            }
            else
            {
                throw new Exception("osmprocess param is mandatory when using osmservice=".org_geoprisma_service_Service::BLANK_SERVICE);
            }
        }

        // ############################################################
        // Get main service from request
        // ############################################################
        $objService = $objConfig->getService($_REQUEST['osmservice']);
        
        // ############################################################
        // Get the correct proxy depending of service 
        // ############################################################
        $objProxy = null;
        if ($objService instanceof org_geoprisma_service_WMSService) 
        {   
            $objProxy = new org_geoprisma_proxy_wms_WMSProxy($objService);              
        }
        else if ($objService instanceof org_geoprisma_service_TileCacheService) 
        {   
            $objProxy = new org_geoprisma_proxy_tilecache_TileCacheProxy($objService);              
        }
        else if ($objService instanceof org_geoprisma_service_FeatureServerService) 
        {
            $objProxy = org_geoprisma_proxy_featureserver_FeatureServerProxyFactory::getFeatureServerProxy($objService);   
        }
        else if ($objService instanceof org_geoprisma_service_MapFishPrintService) 
        {
            $objProxy = org_geoprisma_proxy_mapfishprint_MapFishPrintProxyFactory::getMapFishPrintProxy($objService);
        }
        else if ($objService instanceof org_geoprisma_service_FileService) 
        {
            $objProxy = org_geoprisma_proxy_file_FileProxyFactory::getFileProxy($objService);   
        }
        else if ($objService instanceof org_geoprisma_service_WFSService) 
        {
            $objProxy = org_geoprisma_proxy_wfs_WFSProxyFactory::getWFSProxy($objService);
        }
        else if ($objService instanceof org_geoprisma_service_WPSService) 
        {
            $objProxy = new org_geoprisma_proxy_wps_WPSProxy($objService);   
        }
        else if ($objService instanceof org_geoprisma_service_HttpRequestService) 
        {
            $objProxy = new org_geoprisma_proxy_httprequest_HttpRequestProxy($objService);
        }
        else if ($objService instanceof org_geoprisma_service_TMSService) 
        {
            $objProxy = new org_geoprisma_proxy_tms_TMSProxy($objService);
        }
        // the 'WidgetService' should no longer require a resource to work
        // If your widget no longer works because of this, see the following 
        // ticket : http://trac.osgeo.org/geoprisma/ticket/162
        /*
        else if ($objService instanceof org_geoprisma_service_WidgetService) 
        {            
            $objProxy = new org_geoprisma_proxy_widget_WidgetProxy($objService, $objArrayResource->offsetGet(0));   
        }
        */

        // ############################################################
        // Get resource objects from request, validate them, validate
        // the layers from the request and see that all resources are
        // authorized by the ACL
        // ############################################################
        $objProxy->setResources($objProxy->getResourcesFromRequest($objConfig));
        $objProxy->validateResourcesFromRequest();
        $objProxy->validateLayersFromRequest();
        $objProxy->areResourcesFromRequestAuthorized();
        $objProxy->applyAccessFilter($objService);
        
        return $objProxy;
    }
}
  
?>
