<?php
/**
* Build the right type of file proxy based on the request
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Julien-Samuel Lacroix
* @copyright Mapgears
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Construit le bon type de file proxy selon le type de requete
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Proxy 
* @author     Julien-Samuel Lacroix
*/  
class org_geoprisma_proxy_file_FileProxyFactory
{
    /**
    * Return the good file proxy
    * 
    * @param org_geoprisma_service_FileService $pobjService Service
    * 
    * @return org_geoprisma_proxy_file_FileProxy
    */
    public static function getFileProxy(org_geoprisma_service_FileService $pobjService)
    {
        // Set default to get
        $objFileProxy = new org_geoprisma_proxy_file_FileGetProxy($pobjService);

        if (self::isDownload()) 
        {    
            $objFileProxy = new org_geoprisma_proxy_file_FileDownloadProxy($pobjService);
        }
        else if (self::isView()) 
        {
            $objFileProxy = new org_geoprisma_proxy_file_FileViewProxy($pobjService);
        }
        else if (self::isUpload()) 
        {
            $objFileProxy = new org_geoprisma_proxy_file_FileUploadProxy($pobjService);
        }
        else if (self::isNewDir()) 
        {
            $objFileProxy = new org_geoprisma_proxy_file_FileNewDirProxy($pobjService);
        }
   
        return $objFileProxy;
    }
    
    /**
    * Check if query is the type of download | cmd = download
    * 
    * @return bool
    */
    private static function isDownload()
    {
        return (isset($_REQUEST['cmd']) && $_REQUEST['cmd'] == 'download' && 
                isset($_REQUEST['path']) && $_REQUEST['path'] != '');
    }
    
    /**
    * Check if query is the type of view | cmd = view
    * 
    * @return bool
    */
    private static function isView()
    {
        return (isset($_REQUEST['cmd']) && $_REQUEST['cmd'] == 'view' && 
                isset($_REQUEST['path']) && $_REQUEST['path'] != '');
    }

    /**
    * Check if query is the type of upload | cmd = upload
    * 
    * @return bool
    */
    private static function isUpload()
    {
        return (isset($_REQUEST['cmd']) && $_REQUEST['cmd'] == 'upload' && 
                isset($_REQUEST['path']) && $_REQUEST['path'] != '' &&
                isset($_FILES) && isset($_FILES['x-filename']) &&
                isset($_REQUEST['dir']) && $_REQUEST['dir'] != '');
    }

    /**
    * Check if query is the type of new directory | cmd = newdir
    * 
    * @return bool
    */
    private static function isNewDir()
    {
        return (isset($_REQUEST['cmd']) && $_REQUEST['cmd'] == 'newdir' && 
                isset($_REQUEST['dir']) && $_REQUEST['dir'] != '');
    }
}
?>
