<?php
/**
* Proxy of MapFishPrint type
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Alexandre Dube
* @copyright 2011, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Proxy of MapFishPrint type
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Proxy 
* @author     Alexadnre Dube
*/     
class org_geoprisma_proxy_mapfishprint_MapFishPrintGetFileProxy extends org_geoprisma_proxy_Proxy
{
    /**
    * @var String
    */
    private $m_strPDFURL = null;

    /**
    * Get the action this proxy does.
    *                                   
    * @return string
    */
    public function getAction()
    {
        return self::CRUD_READ;    
    }

    /**
    * Forward the call end return the result
    * 
    * @return void
    */
    public function process()
    {
        $strPDFURL = $this->getPDFURL();
        $strFile = "map.pdf";
        header("Content-Transfer-Encoding: Binary");
        header('Content-type: application/pdf');
        
        // work around IE6/7/8 bug with download of PDF files over https
        if (! (strstr($_SERVER["HTTP_USER_AGENT"], "MSIE")==false)) 
        {
            header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
            header("Pragma: public");
        }
        else
        {
            header('Content-Disposition: attachment; filename="'.$strFile.'"');
        }

        readfile($strPDFURL);
    }
    
    /**
    * Not used by this proxy.
    * 
    * @return void
    */
    public function getLayers()
    {
        return;
    }

    /**
    * Get the resources from the 'spec' object.  At the same time, get all 
    *     the requested 'layers' as well and the resources according 'services'
    *     that are going to be used to render the images to print.
    *
    * @param org_geoprisma_config_Config &$pobjConfig The config object.
    *
    * @return array of org_geoprisma_resource_Resource objects
    */
    public function getResourcesFromRequest(&$pobjConfig)
    {
        $objArrayResources = Array();

        if (!session_id()) 
        {
            session_start();
        }

        if (!isset($_REQUEST)) 
        {
            throw new com_borealis_foundation_exception_AssertException(
                "Request is not set"
            );
        }
        else if (!isset($_REQUEST['osmfile'])) 
        {
            throw new com_borealis_foundation_exception_AssertException(
                "'file' param is not set"
            );
        }
        else if (!isset($_SESSION)) 
        {
            throw new com_borealis_foundation_exception_AssertException(
                "Session is not set"
            );
        }
        else if (!isset($_SESSION[$_REQUEST['osmfile']])) 
        {
            throw new com_borealis_foundation_exception_AssertException(
                "Specified file not found in session"
            );
        }

        $this->setPDFURL($_SESSION[$_REQUEST['osmfile']]['url']);
        $objArrayResourceNames = $_SESSION[$_REQUEST['osmfile']]['resources'];

        foreach ($objArrayResourceNames as $strRessourceName) 
        {
            array_push(
                $objArrayResources,
                $pobjConfig->getResource($strRessourceName)
            );
        }

        return $objArrayResources;
    }

    /**
    * Not used by this proxy.
    *
    * @return void
    */
    public function validateResourcesFromRequest() 
    {
        return;
    }

    /**
    * Not used by this proxy.
    *
    * @return void
    */
    public function validateLayersFromRequest()
    {
        return;
    }

    /**
    * Set pdf file url
    *
    * @param string $pstrURL url of the pdf
    *
    * @return void
    */
    public function setPDFURL($pstrURL)
    {
        $this->m_PDFURL = $pstrURL;
    }

    /**
    * Get pdf file url
    *
    * @return String
    */
    public function getPDFURL()
    {
        return $this->m_PDFURL;
    }
} 

?>
