<?php
/**
* Class Proxy of TMS type
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma
* @author    Alexandre Dube
* @copyright 2009-2011, Mapgears Inc.
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Class Proxy of TMS type
* 
* @category   PHP
* @package    GeoPrisma
* @subpackage Proxy
* @author     Alexandre Dube
*/     
class org_geoprisma_proxy_tms_TMSProxy extends org_geoprisma_proxy_Proxy
{
    /**
    * Returns the action of this proxy.
    *                                   
    * @return string
    */
    public function getAction()
    {
        return self::CRUD_READ;    
    }

    /**
    * Process the request and return its result.
    * 
    * @return void
    */
    public function process()
    {
        ob_start();

        $objCurl = curl_init(urldecode(str_replace("?TMSRequest=", "", $this->addParam($this->getService()->getSource()))));

        curl_exec($objCurl);
        $objArrayInfo = curl_getinfo($objCurl);
        curl_close($objCurl);

        header("Content-type: $objArrayInfo[content_type]");
        if ($this->getCaching() === true) 
        {
            header("Pragma: public");
            header("Cache-Control: maxage=".$this->getExpires());
            header('Expires: ' . gmdate('D, d M Y H:i:s', time()+$this->getExpires()) . ' GMT');
        }

        ob_end_flush();
    }
    
    /**
    * Returns an array of layers being accessed by the request.
    * 
    * @return ArrayObject
    */
    public function getLayers()
    {
        $objArrayLayer = new ArrayObject();

        // TMSRequest property and value example : 
        //   TMSRequest=1.0.0/osm@g/4/4/10.png
        //   (0) -> 1.0.0 -> version
        //   (1) -> osm@g -> layername
        if (isset($_GET['TMSRequest'])) 
        {
            $objArrayPathInfo = explode('/', $_GET['TMSRequest']);
            // (1) get layername
            $objArrayLayer->append($objArrayPathInfo[1]);
        }

        return $objArrayLayer;         
    }

    /**
    * Return true if caching is enabled
    *
    * @return boolean
    */
    public function getCaching()
    {
        return org_geoprisma_SettingImpl::getTMSProxyCaching();
    }

    /**
    * Return expires
    *
    * @return integer
    */
    public function getExpires()
    {
        return org_geoprisma_SettingImpl::getTMSProxyCachingExpires();
    }
}
  
?>
