<?php
/**
* WFS Proxy for read requests
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Etienne Dube
* @copyright 2011, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* WFS Proxy for read requests
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Proxy 
* @author     Etienne Dube
*/   
class org_geoprisma_proxy_wfs_WFSReadProxy extends org_geoprisma_proxy_wfs_WFSProxy
{
    /**
    * Foward the call and return the result
    * 
    * @return void
    */
    public function process()
    {

        if ( $this->m_iRequestType == self::REQUEST_TYPE_POSTXML )
        {
            ob_start();
            $strPostRequest = $this->m_strPostRequest;

            $strContentType = $this->m_strContentType;
            $strPathInfo = self::getPathInfo();  
            $objCurl = curl_init($this->addParam($this->getService()->getSource()));
            curl_setopt($objCurl, CURLOPT_POST, 1);
            curl_setopt($objCurl, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($objCurl, CURLOPT_HTTPHEADER, array("Content-Type: $strContentType"));
            curl_setopt($objCurl, CURLOPT_POSTFIELDS, "$strPostRequest");
            $objResult = curl_exec($objCurl);
            $objArrayInfo = curl_getinfo($objCurl);
            curl_close($objCurl);
          
            header("Content-type: $objArrayInfo[content_type]");

            // set file name in case of a text/csv file returned
            if (isset($objArrayInfo['content_type'])
                && $objArrayInfo['content_type'] == "text/csv"
            )
            {
                $strFileName = "records.csv";
                header(
                    'Content-Disposition: attachment; filename="'.
                    $strFileName.'"'
                );
                
                try
                {
                    //Get the service option
                    $strNewlineReplaceChar = $this->getService()->getOption('csvNewlineReplaceChar')->getValue();
                    
                    //Get the optionnal separator value chararter or set it to "," if not given
                    $objCsvSeperatorChar = $this->getService()->getOption('csvSeperatorChar', true);
                    if ($objCsvSeperatorChar)
                    {
                        $strCsvSeperatorChar= $objCsvSeperatorChar->getValue();
                    }
                    else
                    {
                        $strCsvSeperatorChar= ',';
                    }
                    
                    if (!empty($strNewlineReplaceChar))
                    {
                        //Splitting all the columns and cells
                        //The regex string look like that: '(\"[^\"]*\";?|[^;]*;?)' Assuming that ";" is the seperator
                        $iFoundMatches = preg_match_all('(\"[^\"]*\"'.$strCsvSeperatorChar.'?|[^'.$strCsvSeperatorChar.']*'.$strCsvSeperatorChar.'?)', $objResult, &$objArrayPatterns);
                        if ($iFoundMatches)
                        {
                            //preg_match_all put the results in a sub array.. skip the first array
                            $objArrayPatterns = $objArrayPatterns[0];
                            $iPatternsLength = count($objArrayPatterns);
                            
                            //Go through all celles
                            for ($iIndex = 0;$iIndex < $iPatternsLength; $iIndex++)
                            {
                                //Get the data of each cell
                                $strPattern = $objArrayPatterns[$iIndex];
                                
                                //Check if it's a quoted string
                                //The regex string look like that: '(\"[^\"]*\";?)' Assuming that ";" is the seperator
                                $iQuotedString = preg_match('(\"[^\"]*\"'.$strCsvSeperatorChar.'?)', $strPattern);
                                if ($iQuotedString)
                                {
                                    $objArrayPatterns[$iIndex] = str_replace("\n", $strNewlineReplaceChar, $strPattern);
                                }
                            }
                            $objResult = implode('', $objArrayPatterns);
                        }
                    }
                }
                catch (Exception $objError) 
                {
                }
                // if service option 'utf8Decode' is set, convert result to
                // iso-8859-1 using utf8_decode method.
                try
                {
                    if ($this->getService()->getOption('utf8Decode')->getValue() === "true")
                    {
                        $objResult = utf8_decode($objResult);
                    }
                }
                catch (Exception $objError) 
                {
                }
            }

            echo $objResult;            
            ob_end_flush();
        }

        else if ($this->m_iRequestType = org_geoprisma_proxy_wfs_WFSProxy::REQUEST_TYPE_GET)
        {
            ob_start();
            $strPathInfo = self::getPathInfo();  
            $objCurl = curl_init($this->addParam($this->getService()->getSource().$strPathInfo));                
            curl_exec($objCurl);
            $objArrayInfo = curl_getinfo($objCurl);
            curl_close($objCurl);

            header("Content-type: $objArrayInfo[content_type]");

            ob_end_flush(); 
        }
    }
    
    /**
    * Returns the name of the action done by this proxy (Create / Read / Update / Delete)
    *                                   
    * @return string
    */
    public function getAction()
    {
        return self::CRUD_READ;    
    }
}

?>
