<?php
/**
* Service de type tilecache     
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Service de type tilecache 
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Service 
* @author     Pascal Martin 
*/   
class org_geoprisma_service_TileCacheService extends org_geoprisma_service_Service
{
    const TILECACHE_SERVICE = 'tilecache';
    const TYPE = 'tilecache';
    const TYPE_ID = 2;
    
    private $m_strFormat = null;
    private $m_strSRS = null;
    
    /**
    * Construit un instance de org_geoprisma_service_TileCacheService
    * 
    * @param array $pobjArray Config array
    * 
    * @return org_geoprisma_service_TileCacheService
    */
    public function __construct($pobjArray) 
    {
        if (isset($pobjArray['id'])) 
        {
            $strName = (string)$pobjArray['id'];
        }
        else
        {
            $strName = (string)$pobjArray['name'];
        }

        $strSource = (string)$pobjArray['source'];

        if (isset($pobjArray['format'])) 
        {
            $this->m_strFormat = (string)$pobjArray['format'];
        }

        if (isset($pobjArray['srs'])) 
        {
            $this->m_strSRS = (string)$pobjArray['srs'];
        }

        parent::__construct($strName, $strSource, self::TYPE, $pobjArray);
    }
    
    /**
    * Return the supported format for the tilecache service
    * 
    * @return string 
    */
    public function getFormat()
    {
        return $this->m_strFormat;   
    }
    
    /**
    * Return the srs
    * 
    * @return string 
    */
    public function getSRS()
    {
        return $this->m_strSRS;
    }

    /**
    * Return the OpenLayers.Layer default options using this type of service.
    * 
    * @param org_geoprisma_config_Config &$pobjConfig Config object
    *
    * @return array 
    */
    public function getDefaultLayerOptions(&$pobjConfig)
    {
        return array(
            'buffer' => "0"
        );
    }
    
    /**
    *  Ajout le service courant (this) dans le xml     
    * 
    * @param DOMElement  &$pobjDomElementServices Noeud xml ou le service doit être ajouter
    * @param DOMDocument &$pobjDomDocument        Document XMl ou le noeud xml datastore est ajouter, utiliser pour construire les noeuds
    * 
    * @return void
    */
    public function insertIntoXML(DOMElement &$pobjDomElementServices, DOMDocument &$pobjDomDocument)
    {
        $objDomElementService = $pobjDomDocument->createElement('service'); 
        $pobjDomElementServices->appendChild($objDomElementService);        
        $objDomElementType = $pobjDomDocument->createElement('type', self::TILECACHE_SERVICE);
        $objDomElementName = $pobjDomDocument->createElement('name', $this->getName());
        $objDomElementSrs = $pobjDomDocument->createElement('srs', $this->getSRS());
        $objDomElementFormat = $pobjDomDocument->createElement('format', $this->getFormat());
        $objDomElementService->appendChild($objDomElementType);
        $objDomElementService->appendChild($objDomElementName); 
        $objDomElementService->appendChild($objDomElementSrs);
        $objDomElementService->appendChild($objDomElementFormat);

        // options node
        $objDomElementService->appendChild(
            $this->getDomElements(
                'options', $this->getOptions(), $pobjDomDocument
            )
        );
    }
}
  
?>
