<?php
/**
* Service de type wms     
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Service de type wms 
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Service 
* @author     Pascal Martin 
*/   
class org_geoprisma_service_WMSService extends org_geoprisma_service_Service
{
    const WMS_SERVICE = 'wms';
    const TYPE = 'wms';
    const TYPE_ID = 1;

    private $m_strFormat = null;
    private $m_strSRS = null;
    
    /**
    * Construit un instance de org_geoprisma_service_WMSService
    * 
    * @param array $pobjArray Config array
    * 
    * @return org_geoprisma_service_WMSService
    */
    public function __construct($pobjArray) 
    {
        if (isset($pobjArray['id'])) 
        {
            $strName = (string)$pobjArray['id'];
        }
        else
        {
            $strName = (string)$pobjArray['name'];
        }

        $strSource = (string)$pobjArray['source'];

        if (isset($pobjArray['format'])) 
        {
            $this->m_strFormat = (string)$pobjArray['format'];
        }

        if (isset($pobjArray['srs'])) 
        {
            $this->m_strSRS = (string)$pobjArray['srs'];
        }
        
        parent::__construct($strName, $strSource, self::TYPE, $pobjArray);
    }
    
    /**
    * Return the supported format for the wms service
    * 
    * @return string 
    */
    public function getFormat()
    {
        return $this->m_strFormat;   
    }
    
    /**
    * Return the srs
    * 
    * @return string 
    */
    public function getSRS()
    {
        return $this->m_strSRS;
    }

    /**
    * Return the OpenLayers.Layer default options using this type of service.
    * 
    * @param org_geoprisma_config_Config &$pobjConfig Config object
    *
    * @return array 
    */
    public function getDefaultLayerOptions(&$pobjConfig)
    {
        $objArrayDefaultLayerOptions = array();

        if ($pobjConfig->isUsingLegacyWidgets()) 
        {
            $objArrayDefaultLayerOptions = array(
                'isbaselayer' => "false",
                'transparent' => "true",
                'singletile' => "true"
            );
        }
        else
        {
            $objArrayDefaultLayerOptions = array(
                'isBaseLayer' => "false",
                'transparent' => "true",
                'singleTile' => "true"
            );
        }

        return $objArrayDefaultLayerOptions;
    }
    
    /**
    *  Ajout le service courant (this) dans le xml     
    * 
    * @param DOMElement  &$pobjDomElementServices Noeud xml ou le service doit être ajouter
    * @param DOMDocument &$pobjDomDocument        Document XMl ou le noeud xml datastore est ajouter, utiliser pour construire les noeuds
    * 
    * @return void
    */
    public function insertIntoXML(DOMElement &$pobjDomElementServices, DOMDocument &$pobjDomDocument)
    {
        $objDomElementService = $pobjDomDocument->createElement('service'); 
        $pobjDomElementServices->appendChild($objDomElementService);        
        $objDomElementType = $pobjDomDocument->createElement('type', self::WMS_SERVICE);
        $objDomElementName = $pobjDomDocument->createElement('name', $this->getName());
        $objDomElementSrs = $pobjDomDocument->createElement('srs', $this->getSRS());
        $objDomElementFormat = $pobjDomDocument->createElement('format', $this->getFormat());
        $objDomElementService->appendChild($objDomElementType);
        $objDomElementService->appendChild($objDomElementName); 
        $objDomElementService->appendChild($objDomElementSrs);
        $objDomElementService->appendChild($objDomElementFormat);

        // options node
        $objDomElementService->appendChild(
            $this->getDomElements(
                'options', $this->getOptions(), $pobjDomDocument
            )
        );
    }

    /**
    * Build and execute a WMS GetStyles request given a specific layer name.
    * 
    * @param string $pstrLayerName The layer name to use in the request.
    * 
    * @return string
    */
    public function getStyles($pstrLayerName)
    {
        $objArrayParams = array(
            "SERVICE" => "WMS",
            "REQUEST" => "GetStyles",
            "VERSION" => "1.1.1",
            "LAYERS" => $pstrLayerName
        );

        // attempt to get service "version" option value
        $objVersionOption = $this->getOption("version", true);
        if ($objVersionOption) 
        {
            $objArrayParams["VERSION"] = $objVersionOption->getValue();
        }

        $objCurl = curl_init(
            $this->urlAppend($this->getSource(), $objArrayParams)
        );
        curl_setopt($objCurl, CURLOPT_RETURNTRANSFER, 1);
        $strContents = curl_exec($objCurl);
        curl_close($objCurl);
        ob_clean();

        return $strContents;
    }

    /**
    * Build and return WMS GetLegendGraphic request url for the specified 
    * resource and parameters.
    * 
    * @param org_geoprisma_resource_Resource $pobjResource    The resource
    * @param array                           $pobjArrayParams Additional request
    *                                                         parameters.
    * 
    * @return string
    */
    public function getLegendGraphicURL(
        $pobjResource,
        $pobjArrayParams=array()
    ) {
        // default parameters
        $objArrayDefaultParams = array(
            "EXCEPTIONS" => "application/vnd.ogc.se_xml",
            "FORMAT" => "image/png",
            // LAYER param automatically set using wms datastore
            "LAYER" => $pobjResource->getDataStore($this)->getLayerString(),
            "REQUEST" => "GetLegendGraphic",
            "SERVICE" => "WMS",
            "VERSION" => "1.1.1"
        );

        // merge default parameters <- additional parameters
        $objArrayParams = array_merge(
            $objArrayDefaultParams,
            $pobjArrayParams
        );

        return $this->urlAppend($this->getSource(), $objArrayParams);
    }
}
  
?>
