<?php
/**
* Widget de générique
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Widget de générique  
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Widget 
* @author     Pascal Martin
* @license    http://www.geoprisma.org/license BSD License
* @link       http://www.geoprisma.org
*/  
class org_geoprisma_widget_GenericWidget extends org_geoprisma_widget_Widget
{
    private $m_strType = null; 
 
    /**
    * Construit un instance de org_geoprisma_widget_GenericWidget
    *
    * @param string $pstrType  Widget type
    * @param array  $pobjArray Config array
    * 
    * @return org_geoprisma_widget_GenericWidget
    */
    public function __construct($pstrType, $pobjArray) 
    {
        if (isset($pobjArray['id'])) 
        {
            $strName = (string)$pobjArray['id'];
        }
        else
        {
            $strName = (string)$pobjArray['name'];
        }

        parent::__construct($strName);
        
        $this->m_strType = $pstrType;
        $this->loadOptions($pobjArray); 
    }
    
    /**
    * Charge la liste des otions
    * 
    * @param array &$pobjArray Config array
    * 
    * @return void
    */
    private function loadOptions(&$pobjArray)
    {
        $this->m_objArrayOptions = new ArrayObject();
        //$objArrayOptions = $this->getConfigElements($pobjArray['options']);
        $objArrayOptions = $pobjArray['options'];

        foreach ($objArrayOptions as $strOption => $strValue) 
        {
            $this->m_objArrayOptions->append(
                new org_geoprisma_option_Option($strOption, $strValue)
            );
        }
    }
    
    /**
    * Retourne le type du widget
    * 
    * @return string
    */
    public function getType()
    {
        return $this->m_strType;   
    }
    
    /**
    *  Ajout le widget courant (this) dans le xml      
    * 
    * @param DOMElement  &$pobjDomElementWidgets Noeud xml ou le widget doit
    *                                            être ajouter
    * @param DOMDocument &$pobjDomDocument       Document XMl ou le noeud xml
    *                                            datastore est ajouter, utiliser
    *                                            pour construire les noeuds
    * 
    * @return void
    */
    public function insertIntoXML(
        DOMElement &$pobjDomElementWidgets, DOMDocument &$pobjDomDocument
    ) {
        $objDomElementWidget = $pobjDomDocument->createElement('widget');
        $pobjDomElementWidgets->appendChild($objDomElementWidget);
           
        $objDomElementType = $pobjDomDocument->createElement(
            'type', $this->getType()
        );
        $objDomElementName = $pobjDomDocument->createElement(
            'name', $this->getName()
        );
        $objDomElementWidget->appendChild($objDomElementType);
        $objDomElementWidget->appendChild($objDomElementName);       
            
        $objDomElementOptions = $pobjDomDocument->createElement('options');
        $objDomElementWidget->appendChild($objDomElementOptions);        
        foreach ($this->getOptions() as $objOption) 
        {
            $objOption->insertIntoXML($objDomElementOptions, $pobjDomDocument);
        }    
        
        return $objDomElementWidget;   
    }
}
  
?>
