<?php
/**
* Widget de type mapfishlayertree
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Widget de type mapfishlayertree  
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Widget 
* @author     Pascal Martin 
*/  
class org_geoprisma_widget_MapFishLayerTreeWidget extends org_geoprisma_widget_Widget
{
    const MAP_FISH_LAYER_TREE_WIDGET = 'mapfishlayertree';

    const PATH_DELIMITER = '/';

    const DOMAIN_KEY_LOOKUP_PATTERN = '#@([^@:]*):([^@:]*)@#';
    
    /**
    * @var DOMDocument
    */
    private $m_objDomDocumentOptions = null; 
    
    /**
    * Construit un instance de org_geoprisma_widget_MapWidget
    *
    * @param array &$pobjArray Config node
    * 
    * @return org_geoprisma_widget_MapWidget
    */
    public function __construct(&$pobjArray) 
    {
        if (isset($pobjArray['id'])) 
        {
            $strName = (string)$pobjArray['id'];
        }
        else
        {
            $strName = (string)$pobjArray['name'];
        }

        $this->m_objArrayOptions = $pobjArray['options'];

        if (!isset($this->m_objArrayOptions['nodes'])) 
        {
            $this->m_objArrayOptions['nodes'] = array();
        }

        parent::__construct($strName);
    }

    /**
    * Given an array of nodes, remove each ones that are leaves and that have
    *     non-authorized or inexistant resources.  Nodes that are not leaves 
    *     (that contain other nodes) should have its childs dealt with also.
    * 
    * @param array $pobjArrayNodes Array of nodes
    * 
    * @return array
    */
    private function removeNodesWithNoRight($pobjArrayNodes)
    {
        foreach ($pobjArrayNodes as $strNodeKey => $objArrayNode)
        {
            // if current node has children
            if (isset($objArrayNode['nodes'])) 
            {
                $pobjArrayNodes[$strNodeKey]['nodes'] 
                    = $this->removeNodesWithNoRight($objArrayNode['nodes']);

                // if current node had all its children removed, remove it
                if (count($pobjArrayNodes[$strNodeKey]['nodes']) == 0) 
                {
                    unset($pobjArrayNodes[$strNodeKey]);
                }
            }
            // if current node is a leaf
            else
            {
                if (isset($objArrayNode['resourcename'])) 
                {
                    $strResource = $objArrayNode['resourcename'];
                    $strAction = $this->getAction();

                    try
                    {
                        $objResource = org_geoprisma_config_ConfigFactory::getConfig()->getResource($strResource);
                        if (!org_geoprisma_acl_ACLFactory::getACL()->isAuthorized($objResource, $strAction)) 
                        {                       
                            unset($pobjArrayNodes[$strNodeKey]);
                        }                 
                    }
                    catch(org_geoprisma_exception_ResourceNotFoundException $objError)
                    {
                        unset($pobjArrayNodes[$strNodeKey]);
                    }
                }
            }
        }

        return $pobjArrayNodes;
    }

    /**
    * Remove a specific leaf using its resourcename and servicetype values from
    *     given nodes.  If the parent node becomes empty, it's removed as well.
    * 
    * @param array  &$pobjArrayNodes  Array of nodes
    * @param string $pstrResourceName The name of the resourcename of the node
    *                                 to remove
    * @param string $pstrServiceType  The servicetype of the node to remove
    * 
    * @return void
    */
    public function removeLeaf(
        &$pobjArrayNodes, $pstrResourceName, $pstrServiceType
    ) {
        foreach ($pobjArrayNodes as $strNodeKey => &$objArrayNode)
        {
            // if current node has children
            if (isset($objArrayNode['nodes'])) 
            {
                    $this->removeLeaf(
                        $objArrayNode['nodes'],
                        $pstrResourceName,
                        $pstrServiceType
                    );

                // if current node had all its children removed, remove it
                if (count($pobjArrayNodes[$strNodeKey]['nodes']) == 0) 
                {
                    unset($pobjArrayNodes[$strNodeKey]);
                }
            }
            // if current node is a leaf
            else
            {
                if (isset($objArrayNode['resourcename']) 
                    && $objArrayNode['resourcename'] == $pstrResourceName
                    && isset($objArrayNode['servicetype']) 
                    && $objArrayNode['servicetype'] == $pstrServiceType
                ) 
                {         
                    unset($pobjArrayNodes[$strNodeKey]);
                }
            }
        }
    }
    
    /**
    * Retourne le type du widget
    * 
    * @return string
    */
    public function getType()
    {
        return self::MAP_FISH_LAYER_TREE_WIDGET;   
    }
    
    /**
    *  Ajout le widget courant (this) dans le xml      
    * 
    * @param DOMElement  &$pobjDomElementWidgets Noeud xml ou le widget doit être ajouter
    * @param DOMDocument &$pobjDomDocument       Document XMl ou le noeud xml datastore est ajouter, utiliser pour construire les noeuds
    * 
    * @return void
    */
    public function insertIntoXML(DOMElement &$pobjDomElementWidgets, DOMDocument &$pobjDomDocument)
    {
        $this->m_objArrayOptions['nodes']
            = $this->removeNodesWithNoRight($this->m_objArrayOptions['nodes']);

        if ($this->getResources()->count() > 0) 
        {
            $objDomElementWidget = $pobjDomDocument->createElement('widget');
            $pobjDomElementWidgets->appendChild($objDomElementWidget);    
           
            $objDomElementType = $pobjDomDocument->createElement('type', $this->getType());
            $objDomElementName = $pobjDomDocument->createElement('name', $this->getName());
            $objDomElementWidget->appendChild($objDomElementType);
            $objDomElementWidget->appendChild($objDomElementName);       

            $objDomNodeParams = $this->getDomElements(
                'options', $this->m_objArrayOptions, $pobjDomDocument
            );

            $objDomElementWidget->appendChild($objDomNodeParams);  
            
            return $objDomElementWidget;
        }
        else
        {
            return null;   
        }
    }
    
    /**
    * Retourne une liste de nom de ressource utiliser par ce widget
    * 
    * @return ArrayObject of string
    */
    public function getResources()
    {    
        $objArrayResource = new ArrayObject();        
        $objArrayResourceNames 
            = $this->getResourceNames($this->m_objArrayOptions['nodes']);

        foreach ($objArrayResourceNames as $strResourceName) 
        {
            try
            {
                org_geoprisma_config_ConfigFactory::getConfig()->getResource($strResourceName);                
                $objArrayResource->append($strResourceName);
            } 
            catch(org_geoprisma_exception_ResourceNotFoundException $objError)
            {
                continue;
            }
        }

        return $objArrayResource;         
    }
    
    /**
    * Retourne une liste d'information sur les datastores utilisé
    * 
    * @return ArrayObject of array('resourcename' => ?, 'datastorename' => ?,  'servicename' => ? ) 
    */
    public function getDatastores()
    {
        return self::getDatastoresFromResources($this->getResources());        
    }

    /**
    * Return an array of resource name unique values from given "nodes" array.
    * 
    * @param array $pobjArrayNodes Array of node.
    *
    * @return array
    */
    public function getResourceNames($pobjArrayNodes) 
    {
        $objArrayResourceNames = array();

        foreach ($pobjArrayNodes as $objArrayNode) 
        {
            if (isset($objArrayNode['resourcename'])) 
            {
                array_push(
                    $objArrayResourceNames, $objArrayNode['resourcename']
                );
            }
            else if (isset($objArrayNode['nodes'])) 
            {
                $objArrayResourceNames = array_merge(
                    $objArrayResourceNames,
                    $this->getResourceNames($objArrayNode['nodes'])
                );
            }
        }

        return array_unique($objArrayResourceNames);
    }

    /**
    * Prepare and call to the 'addLayerNodeFromPathArray' method to set the
    *     options 'nodes' property with the new layer node added.
    * 
    * @param string                          &$pstrPath        The path in which
    *                                                          to add the new
    *                                                          layer node
    * @param org_geoprisma_resource_Resource &$pobjResource    Resource object
    * @param string                          &$pstrServiceType Service type
    * @param array                           &$pobjOptions     Array of layer
    *                                                          node options.
    *
    * @return void
    */
    public function addLayerNodeFromPath(
        &$pstrPath,
        &$pobjResource,
        &$pstrServiceType,
        &$pobjOptions
    ) {
        $objArrayNodesToAdd = explode(self::PATH_DELIMITER, $pstrPath);

        $this->addLayerNodeFromPathArray(
            $objArrayNodesToAdd,
            $this->m_objArrayOptions['nodes'],
            $pobjResource,
            $pstrServiceType,
            $pobjOptions
        );
    }

    /**
    * Add a new layer node to the 'nodes' array OR browse nodes using the
    *     'paths' array in order to add it to the according tree position.  If
    *     a 'path' node doesn't exists, it creates it at the same time.
    * 
    * @param array                           &$pobjArrayPaths  Array of nodes
    *                                                          in which to add
    *                                                          the new layer
    *                                                          node
    * @param array                           &$pobjArrayNodes  Array of nodes in
    *                                                          which to push the
    *                                                          a new node (leaf
    *                                                          or container).
    * @param org_geoprisma_resource_Resource &$pobjResource    Resource object
    * @param string                          &$pstrServiceType Service type
    * @param array                           &$pobjOptions     Array of layer
    *                                                          node options.
    *
    * @return array Nodes containing the new ones added.
    */
    public function addLayerNodeFromPathArray(
        &$pobjArrayPaths,
        &$pobjArrayNodes,
        &$pobjResource,
        &$pstrServiceType,
        &$pobjOptions
    ) {
        // if there are no more paths, that means we are in the leaf container,
        // so create and push the new leaf
        if (count($pobjArrayPaths) == 0) 
        {
            array_push(
                $pobjArrayNodes,
                array_merge(
                    $pobjOptions,
                    array(
                        'servicetype' => $pstrServiceType,
                        'resourcename' => $pobjResource->getName(),
                        'textkey' => $pobjResource->getTitle()
                    )
                )
            );
        }
        // see if the path has an existing node.  If if does, move in it, else
        // create a new one then move in it.
        else
        {
            // shift current path
            $strPath = array_shift($pobjArrayPaths);

            // check if path contains domain:key
            preg_match(
                self::DOMAIN_KEY_LOOKUP_PATTERN,
                $strPath,
                $objArrayMatchs
            );

            // if it does
            if (count($objArrayMatchs) == 3) 
            {
                $objArrayTextKey = array(
                    "domain" => $objArrayMatchs[1],
                    "key" => $objArrayMatchs[2]
                );
            }
            else
            {
                $objArrayTextKey = $strPath;
            }
            
            $iIndex = $this->indexOfNodeByTextKey(
                $pobjArrayNodes,
                $objArrayTextKey
            );

            // if node exists
            if ($iIndex != -1 && isset($pobjArrayNodes[$iIndex]['nodes'])) 
            {
                $this->addLayerNodeFromPathArray(
                    $pobjArrayPaths,
                    $pobjArrayNodes[$iIndex]['nodes'],
                    $pobjResource,
                    $pstrServiceType,
                    $pobjOptions
                );
            }
            // if node doesn't exists
            else
            {
                $objArrayNode = array(
                    "textkey" => $objArrayTextKey,
                    "checked" => "undefined",
                    "nodes" => array()
                );

                $this->addLayerNodeFromPathArray(
                    $pobjArrayPaths,
                    $objArrayNode['nodes'],
                    $pobjResource,
                    $pstrServiceType,
                    $pobjOptions
                );

                array_push(
                    $pobjArrayNodes,
                    $objArrayNode
                );
            }
        }
    }

    /**
    * Check if a node exists in an array of nodes using the 'textkey' property.
    *     It may sometimes be defined as a 'domain' & 'key' array, so it takes
    *     care of that too.
    *
    * If it does, returns the key of the node, else returns -1.
    * 
    * @param array  &$pobjArrayNodes Array of nodes
    * @param string $pstrTextKey     TextKey value.  Can also be an array with
    *                                'domain' and 'key' keys.
    *
    * @return integer
    */
    public function indexOfNodeByTextKey(&$pobjArrayNodes, $pstrTextKey) 
    {
        if (is_array($pstrTextKey)) 
        {
            foreach ($pobjArrayNodes as $iKey => $objNode) 
            {
                if (getType($objNode['textkey']) == 'array') 
                {
                    if (is_array($objNode['textkey'])
                        && isset($objNode['textkey']['domain'])
                        && isset($objNode['textkey']['key'])
                        && $objNode['textkey']['domain'] === $pstrTextKey['domain']
                        && $objNode['textkey']['key'] === $pstrTextKey['key']
                    ) 
                    {
                        return $iKey;
                    }
                }
            }
        }
        else
        {
            foreach ($pobjArrayNodes as $iKey => $objNode) 
            {
                if ($objNode['textkey'] === $pstrTextKey) 
                {
                    return $iKey;
                }
            }
        }

        return -1;
    }

    /**
    * Returns a reference to the root nodes of the layertree inside the options.
    *
    * @return array
    */
    public function &getNodes() 
    {
        return $this->m_objArrayOptions['nodes'];
    }
}
  
?>
