<?php
/**
* Widget de type map
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Widget de type map  
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Widget 
* @author     Pascal Martin
* @license    http://www.geoprisma.org/license BSD License
* @link       http://www.geoprisma.org
*/  
class org_geoprisma_widget_MapWidget extends org_geoprisma_widget_Widget
{
    const MAP_WIDGET = 'map';
    
    private $m_objArrayLayer = null;
    
    /**
    * Construit un instance de org_geoprisma_widget_MapWidget
    * 
    * @param array &$pobjArray Config node
    * 
    * @return org_geoprisma_widget_MapWidget
    */
    public function __construct(&$pobjArray) 
    {
        if (isset($pobjArray['id'])) 
        {
            $strName = (string)$pobjArray['id'];
        }
        else
        {
            $strName = (string)$pobjArray['name'];
        }

        parent::__construct($strName);
            
        $this->loadOptions($pobjArray);
        $this->loadLayer($pobjArray);
    }

    /**
    * Charge la liste des otions
    * 
    * @param array &$pobjArray Config array
    * 
    * @return void
    */
    private function loadOptions(&$pobjArray)
    {
        $this->m_objArrayOptions = new ArrayObject();
        //$objArrayOptions = $this->getConfigElements($pobjArray['options']);
        $objArrayOptions = $pobjArray['options'];

        foreach ($objArrayOptions as $strOption => $strValue) 
        {
            $this->m_objArrayOptions->append(
                new org_geoprisma_option_Option($strOption, $strValue)
            );
        }
    }
    
    /**
    * Charge la liste des layers a partir de la configuration
    * 
    * @param array &$pobjArray Config node
    * 
    * @return void
    */
    private function loadLayer(&$pobjArray)
    {
        $this->m_objArrayLayer = new ArrayObject();
        //$objArrayLayers = $this->getConfigElements($pobjArray['layers']);
        $objArrayLayers = $pobjArray['layers'];
        
        foreach ($objArrayLayers as $objArrayLayer) 
        {
            $this->m_objArrayLayer->append(
                new org_geoprisma_layer_Layer(
                    new ArrayObject($objArrayLayer['resourcenames']),
                    $objArrayLayer['servicetype'],
                    $objArrayLayer['options']
                )
            );
        }
    }
    
    /**
    * Supprime les layer ou l'acl nous indique des droit insufissant
    * 
    * @return void
    */
    private function removeLayerWithNoRight()
    {
        $objArrayLayer = new ArrayObject(); 
        $strAction = $this->getAction();
        
        foreach ($this->m_objArrayLayer as $objLayer) 
        {
            $objArrayResources = $objLayer->getResourceNames();
           
            $objArrayAuthorizedResources = new ArrayObject();

            foreach ($objArrayResources as $strResource) 
            {
                try
                {
                    $objResource = org_geoprisma_config_ConfigFactory::getConfig()->getResource(
                        $strResource
                    );
                    $bIsAuthorized
                        = org_geoprisma_acl_ACLFactory::getACL()->isAuthorized(
                            $objResource, $strAction
                        );
                    if ($bIsAuthorized) 
                    {
                        $objArrayAuthorizedResources->append($strResource);    
                    }
                } 
                catch(org_geoprisma_exception_ResourceNotFoundException $objError)
                {
                    continue;
                }
            }

            if ($objArrayAuthorizedResources->count() > 0) 
            {
                $objArrayLayer->append(
                    new org_geoprisma_layer_Layer(
                        $objArrayAuthorizedResources,
                        $objLayer->getServiceType(),
                        $objLayer->getOptions()
                    )
                );
            }
        }
        
        $this->m_objArrayLayer = $objArrayLayer;        
    }
    
    /**
    * Retourne le type du widget
    * 
    * @return string
    */
    public function getType()
    {
        return self::MAP_WIDGET;   
    }
    
    /**
    *  Ajout le widget courant (this) dans le xml      
    * 
    * @param DOMElement  &$pobjDomElementWidgets Noeud xml ou le widget doit
    *                                            être ajouter
    * @param DOMDocument &$pobjDomDocument       Document XMl ou le noeud xml 
    *                                            datastore est ajouter, utiliser
    *                                            pour construire les noeuds
    * 
    * @return void
    */   
    public function insertIntoXML(
        DOMElement &$pobjDomElementWidgets, DOMDocument &$pobjDomDocument
    ) {
        $this->removeLayerWithNoRight();
        $objArrayLayer = $this->getLayers();
        if ($objArrayLayer->count() > 0) 
        {
            $objDomElementWidget = $pobjDomDocument->createElement('widget');
            $pobjDomElementWidgets->appendChild($objDomElementWidget);    
           
            $objDomElementType = $pobjDomDocument->createElement(
                'type', $this->getType()
            );
            $objDomElementName = $pobjDomDocument->createElement(
                'name', $this->getName()
            );
            $objDomElementWidget->appendChild($objDomElementType);
            $objDomElementWidget->appendChild($objDomElementName);       
            
            $objDomElementOptions = $pobjDomDocument->createElement('options');
            $objDomElementWidget->appendChild($objDomElementOptions);        
            foreach ($this->getOptions() as $objOption) 
            {
                $objOption->insertIntoXML(
                    $objDomElementOptions, $pobjDomDocument
                );
            }    
            
            $objDomElementLayers = $pobjDomDocument->createElement('layers'); 
            $objDomElementWidget->appendChild($objDomElementLayers);
            foreach ($this->getLayers() as $objLayer) 
            {
                $objLayer->insertIntoXML(
                    $objDomElementLayers, $pobjDomDocument
                );       
            }
            
            return $objDomElementWidget;
        }
        else
        {
            return null;   
        }
    }
    
    /**
    * Retourne la liste de layer qui peuvent être afficher apres un filtre par l'acl
    * 
    * @return ArrayObject
    */
    private function getLayers()
    {
        $this->removeLayerWithNoRight();
        return $this->m_objArrayLayer;        
    }
    
    /**
    * Retourne une liste de nom de ressource utiliser par ce widget
    * 
    * @return ArrayObject of string
    */
    public function getResources()
    {
        $this->removeLayerWithNoRight(); 
        $objArrayResource = new ArrayObject();
        $objArrayLayer = $this->getLayers();
        foreach ($this->m_objArrayLayer as $objLayer) 
        {
            $objArrayResourceNames = $objLayer->getResourceNames();
            foreach ($objArrayResourceNames as $strResourceName) 
            {
                if (!in_array($strResourceName, (array)$objArrayResource)) 
                {
                    $objArrayResource->append($strResourceName);
                }                
            }
        }
        return $objArrayResource;
    }
    
    /**
    * Retourne une liste d'information sur les datastores utilisé
    * 
    * @return ArrayObject of array('resourcename' => ?, 'datastorename' => ?,
    *                     'servicename' => ? ) 
    */
    public function getDatastores()
    {
        return self::getDatastoresFromResources($this->getResources());
    }
}
  
?>
