<?php

class org_geoprisma_widget_GenericWidgetTest extends PHPUnit_Framework_TestCase
{
    
    public function testGetOptions()    
    {
        $objConfig = org_geoprisma_config_ConfigFactory::getConfig(); 
        
        $objWidget = $objConfig->getWidget('MergeParcel');
        $this->assertNotEquals(null, $objWidget);
        
        $objOptionsDSN = $objWidget->getOption('dsn');        
        $objOptionsFunction = $objWidget->getOption('function');
        
        $strDSN = (string)simplexml_load_string($objOptionsDSN->getValue());
        $strFunction = (string)simplexml_load_string($objOptionsFunction->getValue()); 
        $this->assertEquals('pgsql:host=192.168.3.204;port=5433;dbname=geoprisma;user=postgres;password=postgres', $strDSN);        
        $this->assertEquals('bis_geo_merge', $strFunction);        
    }
    
    public function testGetOption()    
    {
        $objConfig = org_geoprisma_config_ConfigFactory::getConfig(); 
        
        $objWidget = $objConfig->getWidget('MergeParcel');
        $this->assertNotEquals(null, $objWidget);
        
        $objOptionsDSN = $objWidget->getOption('dsn');        
        $objOptionsFunction = $objWidget->getOption('function');
    }
    
    /**
    * @expectedException org_geoprisma_exception_OptionNotFoundException 
    */
    public function testGetOption2()    
    {
        $objConfig = org_geoprisma_config_ConfigFactory::getConfig(); 
        
        $objWidget = $objConfig->getWidget('MergeParcel');
        $this->assertNotEquals(null, $objWidget);
        
        $objWidget->getOption('syfydsuhhfdhsui');        
    }
    

    public function testSetName()    
    {
        $objConfig = org_geoprisma_config_ConfigFactory::getConfig(); 
        
        $objWidget1 = $objConfig->getWidget('MergeParcel');
        $objWidget2 = clone $objWidget1;
        
        $objWidget2->setName('phpunit test name');
        $this->assertEquals('phpunit test name', $objWidget2->getName()); 
        
        $this->assertNotEquals($objWidget1->getName(), $objWidget2->getName());           
    }
    
    // Test add
    public function testAddOrReplaceOption()    
    {
        $objConfig = org_geoprisma_config_ConfigFactory::getConfig(); 
        
        $objWidget1 = $objConfig->getWidget('MergeParcel');
        
        $this->assertNotEquals(null, $objWidget1->getOptions());
        $this->assertEquals(2, $objWidget1->getOptions()->count());
        
        $objWidget2 = clone $objWidget1;
        
        $objWidget2->addOrReplaceOption(new org_geoprisma_option_Option('option1','value1'));
        
        $this->assertEquals(2, $objWidget1->getOptions()->count());
        $this->assertEquals(3, $objWidget2->getOptions()->count());           
    }
    
    // Test replace
    public function testAddOrReplaceOption2()    
    {
        $objConfig = org_geoprisma_config_ConfigFactory::getConfig(); 
        
        $objWidget1 = $objConfig->getWidget('MergeParcel');
        $objWidget2 = clone $objWidget1;        
        
        $objWidget2->addOrReplaceOption(new org_geoprisma_option_Option('option1','value1'));
        $objWidget3 = clone $objWidget2;        
        $objWidget3->addOrReplaceOption(new org_geoprisma_option_Option('option1','value2'));
        
        $this->assertEquals(2, $objWidget1->getOptions()->count());
        $this->assertEquals(3, $objWidget2->getOptions()->count()); 
        $this->assertEquals(3, $objWidget3->getOptions()->count()); 
        
        $this->assertEquals('value1', $objWidget2->getOption('option1')->getValue());            
        $this->assertEquals('value2', $objWidget3->getOption('option1')->getValue());            
    }
    
    /**
    * @expectedException Exception 
    */
    public function testAddOrReplaceOption3()    
    {
        $objConfig = org_geoprisma_config_ConfigFactory::getConfig(); 
        
        $objWidget1 = $objConfig->getWidget('LegendDemo');
        
        $this->assertEquals(null, $objWidget1->getOptions());
        
        $objWidget2 = clone $objWidget1;
        $objWidget2->addOrReplaceOption(new org_geoprisma_option_Option('option1','value1'));        
    }
}
  
?>